@@ -164,34 +164,38 @@ static void print_process_mmap(Task* t) {
 
 AddressSpace::Mapping::Mapping(const KernelMapping& map,
                                const KernelMapping& recorded_map,
-                               EmuFile::shr_ptr emu_file, void* local_addr,
+                               EmuFile::shr_ptr emu_file,
+                               std::unique_ptr<struct stat> mapped_file_stat,
+                               void* local_addr,
                                shared_ptr<MonitoredSharedMemory>&& monitored)
     : map(map),
       recorded_map(recorded_map),
       emu_file(emu_file),
+      mapped_file_stat(move(mapped_file_stat)),
       local_addr(static_cast<uint8_t*>(local_addr)),
       monitored_shared_memory(move(monitored)),
       flags(FLAG_NONE) {}
 
+static unique_ptr<struct stat> clone_stat(
+    const unique_ptr<struct stat>& other) {
+  return other ? unique_ptr<struct stat>(new struct stat(*other)) : nullptr;
+}
+
+AddressSpace::Mapping::Mapping(const Mapping& other)
+    : map(other.map),
+      recorded_map(other.recorded_map),
+      emu_file(other.emu_file),
+      mapped_file_stat(clone_stat(other.mapped_file_stat)),
+      local_addr(other.local_addr),
+      monitored_shared_memory(other.monitored_shared_memory),
+      flags(other.flags) {}
+
 AddressSpace::Mapping::~Mapping() {}
 
 AddressSpace::~AddressSpace() { session_->on_destroy(this); }
 
 void AddressSpace::after_clone() { allocate_watchpoints(); }
 
-static remote_ptr<void> find_rr_vdso(Task* t, size_t* len) {
-  for (KernelMapIterator it(t); !it.at_end(); ++it) {
-    auto& km = it.current();
-    if (km.fsname() == "[vdso]") {
-      *len = km.size();
-      ASSERT(t, uint32_t(*len) == *len) << "VDSO more than 4GB???";
-      return km.start();
-    }
-  }
-  ASSERT(t, false) << "rr VDSO not found?";
-  return nullptr;
-}
-
 static uint32_t find_offset_of_syscall_instruction_in(SupportedArch arch,
                                                       uint8_t* vdso_data,
                                                       size_t vdso_len) {
@@ -239,40 +243,63 @@ static string find_rr_page_file(Task* t) {
   return path;
 }
 
-void AddressSpace::map_rr_page(Task* t) {
+static vector<uint8_t> read_all(Task* t, ScopedFd& fd) {
+  char buf[4096];
+  vector<uint8_t> result;
+  while (true) {
+    int ret = read(fd, buf, sizeof(buf));
+    ASSERT(t, ret >= 0);
+    if (ret == 0) {
+      return result;
+    }
+    result.insert(result.end(), buf, buf + ret);
+  }
+}
+
+void AddressSpace::map_rr_page(AutoRemoteSyscalls& remote) {
   int prot = PROT_EXEC | PROT_READ;
   int flags = MAP_PRIVATE | MAP_FIXED;
 
-  struct stat fstat;
   string file_name;
-  {
-    AutoRemoteSyscalls remote(t);
-    SupportedArch arch = t->arch();
-
-    string path = find_rr_page_file(t);
-    AutoRestoreMem child_path(remote, path.c_str());
-    // skip leading '/' since we want the path to be relative to the root fd
-    int child_fd = remote.infallible_syscall(syscall_number_for_openat(arch),
-                                             RR_RESERVED_ROOT_DIR_FD,
-                                             child_path.get() + 1, O_RDONLY);
+  Task* t = remote.task();
+  SupportedArch arch = t->arch();
 
+  string path = find_rr_page_file(t);
+  AutoRestoreMem child_path(remote, path.c_str());
+  // skip leading '/' since we want the path to be relative to the root fd
+  long child_fd =
+      remote.syscall(syscall_number_for_openat(arch), RR_RESERVED_ROOT_DIR_FD,
+                     child_path.get() + 1, O_RDONLY);
+  if (child_fd >= 0) {
     remote.infallible_mmap_syscall(rr_page_start(), rr_page_size(), prot, flags,
                                    child_fd, 0);
 
-    fstat = t->stat_fd(child_fd);
+    struct stat fstat = t->stat_fd(child_fd);
     file_name = t->file_name_of_fd(child_fd);
 
     remote.infallible_syscall(syscall_number_for_close(arch), child_fd);
 
-    if (t->session().is_recording()) {
-      // brk() will not have been called yet so the brk area is empty.
-      brk_start = brk_end =
-          remote.infallible_syscall(syscall_number_for_brk(arch), 0);
-    }
+    map(t, rr_page_start(), rr_page_size(), prot, flags, 0, file_name,
+        fstat.st_dev, fstat.st_ino);
+  } else {
+    ASSERT(t, child_fd == -EACCES) << "Unexpected error mapping rr_page";
+    flags |= MAP_ANONYMOUS;
+    remote.infallible_mmap_syscall(rr_page_start(), rr_page_size(), prot, flags,
+                                   -1, 0);
+    ScopedFd page(path.c_str(), O_RDONLY);
+    ASSERT(t, page.is_open()) << "Error opening rr_page ourselves";
+    vector<uint8_t> page_data = read_all(t, page);
+    t->write_bytes_helper(rr_page_start(), page_data.size(), page_data.data());
+
+    map(t, rr_page_start(), rr_page_size(), prot, flags, 0, file_name, 0, 0);
   }
 
-  map(rr_page_start(), rr_page_size(), prot, flags, 0, file_name, fstat.st_dev,
-      fstat.st_ino);
+  if (t->session().is_recording()) {
+    // brk() will not have been called yet so the brk area is empty.
+    brk_start = brk_end =
+        remote.infallible_syscall(syscall_number_for_brk(arch), 0);
+    ASSERT(t, !brk_end.is_null());
+  }
 
   traced_syscall_ip_ = rr_page_syscall_entry_point(
       TRACED, UNPRIVILEGED, RECORDING_AND_REPLAY, t->arch());
@@ -390,21 +417,28 @@ void AddressSpace::post_exec_syscall(Task* t) {
   privileged_traced_syscall_ip_ = nullptr;
   // Now remote syscalls work, we can open_mem_fd.
   t->open_mem_fd();
+
+  // Set up AutoRemoteSyscalls again now that the mem-fd is open.
+  AutoRemoteSyscalls remote(t);
   // Now we can set up the "rr page" at its fixed address. This gives
   // us traced and untraced syscall instructions at known, fixed addresses.
-  map_rr_page(t);
+  map_rr_page(remote);
+  // Set up the preload_thread_locals shared area.
+  t->session().create_shared_mmap(remote, PRELOAD_THREAD_LOCALS_SIZE,
+                                  preload_thread_locals_start(),
+                                  "preload_thread_locals");
 }
 
-void AddressSpace::brk(remote_ptr<void> addr, int prot) {
+void AddressSpace::brk(Task* t, remote_ptr<void> addr, int prot) {
   LOG(debug) << "brk(" << addr << ")";
 
   remote_ptr<void> old_brk = ceil_page_size(brk_end);
   remote_ptr<void> new_brk = ceil_page_size(addr);
   if (old_brk < new_brk) {
-    map(old_brk, new_brk - old_brk, prot, MAP_ANONYMOUS | MAP_PRIVATE, 0,
-        "[heap]", KernelMapping::NO_DEVICE, KernelMapping::NO_INODE);
+    map(t, old_brk, new_brk - old_brk, prot, MAP_ANONYMOUS | MAP_PRIVATE, 0,
+        "[heap]");
   } else {
-    unmap(new_brk, old_brk - new_brk);
+    unmap(t, new_brk, old_brk - new_brk);
   }
   brk_end = addr;
 }
@@ -435,7 +469,7 @@ BreakpointType AddressSpace::get_breakpoint_type_at_addr(remote_code_ptr addr) {
 
 bool AddressSpace::is_breakpoint_in_private_read_only_memory(
     remote_code_ptr addr) {
-  for (const auto& m : maps_starting_at(addr.to_data_ptr<void>())) {
+  for (const auto& m : maps_containing_or_after(addr.to_data_ptr<void>())) {
     if (m.map.start() >=
         addr.increment_by_bkpt_insn_length(arch()).to_data_ptr<void>()) {
       break;
@@ -495,9 +529,11 @@ static void add_range(set<MemoryRange>& ranges, const MemoryRange& range) {
   // We could coalesce adjacent ranges, but there's probably no need.
 }
 
-KernelMapping AddressSpace::map(remote_ptr<void> addr, size_t num_bytes,
-                                int prot, int flags, off64_t offset_bytes,
-                                const string& fsname, dev_t device, ino_t inode,
+KernelMapping AddressSpace::map(Task* t, remote_ptr<void> addr,
+                                size_t num_bytes, int prot, int flags,
+                                off64_t offset_bytes, const string& fsname,
+                                dev_t device, ino_t inode,
+                                unique_ptr<struct stat> mapped_file_stat,
                                 const KernelMapping* recorded_map,
                                 EmuFile::shr_ptr emu_file, void* local_addr,
                                 shared_ptr<MonitoredSharedMemory>&& monitored) {
@@ -518,11 +554,11 @@ KernelMapping AddressSpace::map(remote_ptr<void> addr, size_t num_bytes,
   // In testing, the behavior seems to be as if the
   // overlapping region is unmapped and then remapped
   // per the arguments to the second call.
-  unmap_internal(addr, num_bytes);
+  unmap_internal(t, addr, num_bytes);
 
   const KernelMapping& actual_recorded_map = recorded_map ? *recorded_map : m;
-  map_and_coalesce(m, actual_recorded_map, emu_file, local_addr,
-                   move(monitored));
+  map_and_coalesce(t, m, actual_recorded_map, emu_file, move(mapped_file_stat),
+                   move(local_addr), move(monitored));
 
   if ((prot & PROT_EXEC) &&
       (fsname.find(SYSCALLBUF_LIB_FILENAME) != string::npos ||
@@ -542,7 +578,7 @@ KernelMapping AddressSpace::map(remote_ptr<void> addr, size_t num_bytes,
 
 template <typename Arch> void AddressSpace::at_preload_init_arch(Task* t) {
   auto params = t->read_mem(
-      remote_ptr<rrcall_init_preload_params<Arch> >(t->regs().arg1()));
+      remote_ptr<rrcall_init_preload_params<Arch>>(t->regs().arg1()));
 
   if (t->session().is_recording()) {
     ASSERT(t, t->session().as_record()->use_syscall_buffer() ==
@@ -595,7 +631,8 @@ bool AddressSpace::has_mapping(remote_ptr<void> addr) const {
   return it != mem.end() && it->first.contains(m);
 }
 
-void AddressSpace::protect(remote_ptr<void> addr, size_t num_bytes, int prot) {
+void AddressSpace::protect(Task* t, remote_ptr<void> addr, size_t num_bytes,
+                           int prot) {
   LOG(debug) << "mprotect(" << addr << ", " << num_bytes << ", " << HEX(prot)
              << ")";
 
@@ -630,7 +667,8 @@ void AddressSpace::protect(remote_ptr<void> addr, size_t num_bytes, int prot) {
       Mapping underflow(
           m.map.subrange(m.map.start(), rem.start()),
           m.recorded_map.subrange(m.recorded_map.start(), rem.start()),
-          m.emu_file, m.local_addr, move(monitored));
+          m.emu_file, clone_stat(m.mapped_file_stat), m.local_addr,
+          move(monitored));
       add_to_map(underflow);
     }
     // Remap the overlapping region with the new prot.
@@ -640,7 +678,7 @@ void AddressSpace::protect(remote_ptr<void> addr, size_t num_bytes, int prot) {
     Mapping overlap(
         m.map.subrange(new_start, new_end).set_prot(new_prot),
         m.recorded_map.subrange(new_start, new_end).set_prot(new_prot),
-        m.emu_file,
+        m.emu_file, clone_stat(m.mapped_file_stat),
         m.local_addr ? m.local_addr + (new_start - m.map.start()) : 0,
         m.monitored_shared_memory
             ? m.monitored_shared_memory->subrange(new_start - m.map.start(),
@@ -656,6 +694,7 @@ void AddressSpace::protect(remote_ptr<void> addr, size_t num_bytes, int prot) {
       Mapping overflow(
           m.map.subrange(rem.end(), m.map.end()),
           m.recorded_map.subrange(rem.end(), m.map.end()), m.emu_file,
+          clone_stat(m.mapped_file_stat),
           m.local_addr ? m.local_addr + (rem.end() - m.map.start()) : 0,
           m.monitored_shared_memory
               ? m.monitored_shared_memory->subrange(rem.end() - m.map.start(),
@@ -668,7 +707,7 @@ void AddressSpace::protect(remote_ptr<void> addr, size_t num_bytes, int prot) {
   if (last_overlap.size()) {
     // All mappings that we altered which might need coalescing
     // are adjacent to |last_overlap|.
-    coalesce_around(mem.find(last_overlap));
+    coalesce_around(t, mem.find(last_overlap));
   }
 }
 
@@ -689,17 +728,18 @@ void AddressSpace::fixup_mprotect_growsdown_parameters(Task* t) {
   }
 }
 
-void AddressSpace::remap(remote_ptr<void> old_addr, size_t old_num_bytes,
-                         remote_ptr<void> new_addr, size_t new_num_bytes) {
+void AddressSpace::remap(Task* t, remote_ptr<void> old_addr,
+                         size_t old_num_bytes, remote_ptr<void> new_addr,
+                         size_t new_num_bytes) {
   LOG(debug) << "mremap(" << old_addr << ", " << old_num_bytes << ", "
              << new_addr << ", " << new_num_bytes << ")";
 
-  auto mr = mapping_of(old_addr);
+  Mapping mr = mapping_of(old_addr);
   assert(!mr.monitored_shared_memory);
   const KernelMapping& m = mr.map;
 
   old_num_bytes = ceil_page_size(old_num_bytes);
-  unmap_internal(old_addr, old_num_bytes);
+  unmap_internal(t, old_addr, old_num_bytes);
   if (0 == new_num_bytes) {
     return;
   }
@@ -717,9 +757,9 @@ void AddressSpace::remap(remote_ptr<void> old_addr, size_t old_num_bytes,
   }
 
   remote_ptr<void> new_end = new_addr + new_num_bytes;
-  map_and_coalesce(m.set_range(new_addr, new_end),
+  map_and_coalesce(t, m.set_range(new_addr, new_end),
                    mr.recorded_map.set_range(new_addr, new_end), mr.emu_file,
-                   nullptr, nullptr);
+                   clone_stat(mr.mapped_file_stat), nullptr, nullptr);
 }
 
 void AddressSpace::remove_breakpoint(remote_code_ptr addr,
@@ -907,7 +947,7 @@ void AddressSpace::remove_all_watchpoints() {
   allocate_watchpoints();
 }
 
-void AddressSpace::unmap(remote_ptr<void> addr, ssize_t num_bytes) {
+void AddressSpace::unmap(Task* t, remote_ptr<void> addr, ssize_t num_bytes) {
   LOG(debug) << "munmap(" << addr << ", " << num_bytes << ")";
   num_bytes = ceil_page_size(num_bytes);
   if (!num_bytes) {
@@ -916,10 +956,11 @@ void AddressSpace::unmap(remote_ptr<void> addr, ssize_t num_bytes) {
 
   remove_range(dont_fork, MemoryRange(addr, num_bytes));
 
-  return unmap_internal(addr, num_bytes);
+  return unmap_internal(t, addr, num_bytes);
 }
 
-void AddressSpace::unmap_internal(remote_ptr<void> addr, ssize_t num_bytes) {
+void AddressSpace::unmap_internal(Task*, remote_ptr<void> addr,
+                                  ssize_t num_bytes) {
   LOG(debug) << "munmap(" << addr << ", " << num_bytes << ")";
 
   auto unmapper = [this](const Mapping& mm, const MemoryRange& rem) {
@@ -936,7 +977,8 @@ void AddressSpace::unmap_internal(remote_ptr<void> addr, ssize_t num_bytes) {
     if (m.map.start() < rem.start()) {
       Mapping underflow(m.map.subrange(m.map.start(), rem.start()),
                         m.recorded_map.subrange(m.map.start(), rem.start()),
-                        m.emu_file, m.local_addr, move(monitored));
+                        m.emu_file, clone_stat(m.mapped_file_stat),
+                        m.local_addr, move(monitored));
       add_to_map(underflow);
     }
     // If the last segment we unmap overflows the unmap
@@ -945,6 +987,7 @@ void AddressSpace::unmap_internal(remote_ptr<void> addr, ssize_t num_bytes) {
       Mapping overflow(
           m.map.subrange(rem.end(), m.map.end()),
           m.recorded_map.subrange(rem.end(), m.map.end()), m.emu_file,
+          clone_stat(m.mapped_file_stat),
           m.local_addr ? m.local_addr + (rem.end() - m.map.start()) : 0,
           m.monitored_shared_memory
               ? m.monitored_shared_memory->subrange(rem.end() - m.map.start(),
@@ -957,7 +1000,7 @@ void AddressSpace::unmap_internal(remote_ptr<void> addr, ssize_t num_bytes) {
   update_watchpoint_values(addr, addr + num_bytes);
 }
 
-void AddressSpace::advise(remote_ptr<void> addr, ssize_t num_bytes,
+void AddressSpace::advise(Task*, remote_ptr<void> addr, ssize_t num_bytes,
                           int advice) {
   LOG(debug) << "madvise(" << addr << ", " << num_bytes << ", " << advice
              << ")";
@@ -984,7 +1027,7 @@ void AddressSpace::did_fork_into(Task* t) {
       remote.infallible_syscall(syscall_number_for_munmap(remote.arch()),
                                 range.start(), range.size());
     }
-    t->vm()->unmap(range.start(), range.size());
+    t->vm()->unmap(t, range.start(), range.size());
   }
 }
 
@@ -1165,12 +1208,30 @@ void AddressSpace::verify(Task* t) const {
   ASSERT(t, kernel_it.at_end() && mem_it == mem.end());
 }
 
+// Just a place that rr's AutoSyscall functionality can use as a syscall
+// instruction in rr's address space for use before we have exec'd.
+extern "C" {
+extern char rr_syscall_addr;
+}
+static void __attribute__((noinline, used)) fake_syscall() {
+#ifdef __i386__
+  __asm__ __volatile__("rr_syscall_addr: int $0x80\n\t"
+                       "nop\n\t"
+                       "nop\n\t"
+                       "nop\n\t");
+#elif defined(__x86_64__)
+  __asm__ __volatile__("rr_syscall_addr: syscall\n\t"
+                       "nop\n\t"
+                       "nop\n\t"
+                       "nop\n\t");
+#endif
+}
+
 AddressSpace::AddressSpace(Task* t, const string& exe, uint32_t exec_count)
     : exe(exe),
       leader_tid_(t->rec_tid),
       leader_serial(t->tuid().serial()),
       exec_count(exec_count),
-      is_clone(false),
       session_(&t->session()),
       monkeypatch_state(t->session().is_recording() ? new Monkeypatcher()
                                                     : nullptr),
@@ -1182,20 +1243,10 @@ AddressSpace::AddressSpace(Task* t, const string& exe, uint32_t exec_count)
     populate_address_space(t);
     assert(!vdso_start_addr.is_null());
   } else {
-    // Find the location of the VDSO in the just-spawned process. This will
-    // match the VDSO in rr itself since we haven't execed yet. So, speed
-    // things up by searching rr's own VDSO for a syscall instruction.
-    size_t rr_vdso_len = 0;
-    remote_ptr<void> rr_vdso = find_rr_vdso(t, &rr_vdso_len);
-    // Here we rely on the VDSO location in the spawned tracee being the same
-    // as in rr itself.
-    uint8_t* local_vdso = reinterpret_cast<uint8_t*>(rr_vdso.as_int());
-    auto offset = find_offset_of_syscall_instruction_in(
-        NativeArch::arch(), local_vdso, rr_vdso_len);
-    offset_to_syscall_in_vdso[NativeArch::arch()] = offset;
     // Setup traced_syscall_ip_ now because we need to do AutoRemoteSyscalls
-    // (for open_mem_fd) before the first exec.
-    traced_syscall_ip_ = remote_code_ptr(rr_vdso.as_int() + offset);
+    // (for open_mem_fd) before the first exec. We rely on the fact that we
+    // haven't execed yet, so the address space layout is the same.
+    traced_syscall_ip_ = remote_code_ptr((uintptr_t)&rr_syscall_addr);
   }
 }
 
@@ -1208,7 +1259,6 @@ AddressSpace::AddressSpace(Session* session, const AddressSpace& o,
       exec_count(exec_count),
       brk_start(o.brk_start),
       brk_end(o.brk_end),
-      is_clone(true),
       mem(o.mem),
       monitored_mem(o.monitored_mem),
       session_(session),
@@ -1223,6 +1273,12 @@ AddressSpace::AddressSpace(Session* session, const AddressSpace& o,
       syscallbuf_enabled_(o.syscallbuf_enabled_),
       saved_auxv_(o.saved_auxv_),
       first_run_event_(0) {
+  for (auto& m : mem) {
+    // The original address space continues to have exclusive ownership of
+    // all local mappings.
+    m.second.local_addr = nullptr;
+  }
+
   for (auto& it : o.breakpoints) {
     breakpoints.insert(make_pair(it.first, it.second));
   }
@@ -1237,6 +1293,14 @@ AddressSpace::AddressSpace(Session* session, const AddressSpace& o,
   // cloned address-space memory, so we don't need to do any more work here.
 }
 
+void AddressSpace::post_vm_clone(Task* t) {
+  // Recreate preload_thread_locals mapping.
+  AutoRemoteSyscalls remote(t);
+  t->session().create_shared_mmap(remote, PRELOAD_THREAD_LOCALS_SIZE,
+                                  preload_thread_locals_start(),
+                                  "preload_thread_locals");
+}
+
 static bool try_split_unaligned_range(MemoryRange& range, size_t bytes,
                                       vector<MemoryRange>& result) {
   if ((range.start().as_int() & (bytes - 1)) || range.size() < bytes) {
@@ -1397,16 +1461,17 @@ bool AddressSpace::allocate_watchpoints() {
   return false;
 }
 
-static inline void assert_coalesceable(const AddressSpace::Mapping& lower,
+static inline void assert_coalesceable(Task* t,
+                                       const AddressSpace::Mapping& lower,
                                        const AddressSpace::Mapping& higher) {
-  assert(lower.emu_file == higher.emu_file);
-  assert(lower.flags == higher.flags);
-  assert((lower.local_addr == 0 && higher.local_addr == 0) ||
-         lower.local_addr + lower.map.size() == higher.local_addr);
-  assert(!lower.monitored_shared_memory && !higher.monitored_shared_memory);
+  ASSERT(t, lower.emu_file == higher.emu_file);
+  ASSERT(t, lower.flags == higher.flags);
+  ASSERT(t, (lower.local_addr == 0 && higher.local_addr == 0) ||
+                lower.local_addr + lower.map.size() == higher.local_addr);
+  ASSERT(t, !lower.monitored_shared_memory && !higher.monitored_shared_memory);
 }
 
-void AddressSpace::coalesce_around(MemoryMap::iterator it) {
+void AddressSpace::coalesce_around(Task* t, MemoryMap::iterator it) {
   auto first_kv = it;
   while (mem.begin() != first_kv) {
     auto next = first_kv;
@@ -1416,7 +1481,7 @@ void AddressSpace::coalesce_around(MemoryMap::iterator it) {
       first_kv = next;
       break;
     }
-    assert_coalesceable(first_kv->second, next->second);
+    assert_coalesceable(t, first_kv->second, next->second);
   }
   auto last_kv = it;
   while (true) {
@@ -1428,9 +1493,9 @@ void AddressSpace::coalesce_around(MemoryMap::iterator it) {
       last_kv = prev;
       break;
     }
-    assert_coalesceable(prev->second, last_kv->second);
+    assert_coalesceable(t, prev->second, last_kv->second);
   }
-  assert(last_kv != mem.end());
+  ASSERT(t, last_kv != mem.end());
   if (first_kv == last_kv) {
     LOG(debug) << "  no mappings to coalesce";
     return;
@@ -1438,7 +1503,9 @@ void AddressSpace::coalesce_around(MemoryMap::iterator it) {
 
   Mapping new_m(first_kv->second.map.extend(last_kv->first.end()),
                 first_kv->second.recorded_map.extend(last_kv->first.end()),
-                first_kv->second.emu_file, first_kv->second.local_addr);
+                first_kv->second.emu_file,
+                clone_stat(first_kv->second.mapped_file_stat),
+                first_kv->second.local_addr);
   new_m.flags = first_kv->second.flags;
   LOG(debug) << "  coalescing " << new_m.map;
 
@@ -1498,17 +1565,18 @@ void AddressSpace::for_each_in_range(
 }
 
 void AddressSpace::map_and_coalesce(
-    const KernelMapping& m, const KernelMapping& recorded_map,
-    EmuFile::shr_ptr emu_file, void* local_addr,
-    shared_ptr<MonitoredSharedMemory>&& monitored) {
+    Task* t, const KernelMapping& m, const KernelMapping& recorded_map,
+    EmuFile::shr_ptr emu_file, unique_ptr<struct stat> mapped_file_stat,
+    void* local_addr, shared_ptr<MonitoredSharedMemory>&& monitored) {
   LOG(debug) << "  mapping " << m;
 
   if (monitored) {
     monitored_mem.insert(m.start());
   }
   auto ins = mem.insert(MemoryMap::value_type(
-      m, Mapping(m, recorded_map, emu_file, local_addr, move(monitored))));
-  coalesce_around(ins.first);
+      m, Mapping(m, recorded_map, emu_file, move(mapped_file_stat), local_addr,
+                 move(monitored))));
+  coalesce_around(t, ins.first);
 
   update_watchpoint_values(m.start(), m.end());
 }
@@ -1525,14 +1593,16 @@ static bool could_be_stack(const KernelMapping& km) {
          km.inode() == KernelMapping::NO_INODE;
 }
 
-static dev_t check_device(Task* t, const KernelMapping& km) {
+static dev_t check_device(const KernelMapping& km) {
   if (km.fsname().c_str()[0] != '/') {
     return km.device();
   }
   // btrfs files can return the wrong device number in /proc/<pid>/maps
   struct stat st;
   int ret = stat(km.fsname().c_str(), &st);
-  ASSERT(t, ret == 0);
+  if (ret < 0) {
+    return km.device();
+  }
   return st.st_dev;
 }
 
@@ -1545,12 +1615,19 @@ void AddressSpace::populate_address_space(Task* t) {
     }
   }
 
+  // If we're being recorded by rr, we'll see the outer rr's rr_page and
+  // preload_thread_locals. In post_exec() we'll remap those with our
+  // own mappings. That's OK because a) the rr_page contents are the same
+  // anyway and immutable and b) the preload_thread_locals page is only
+  // used by the preload library, and the preload library only knows about
+  // the inner rr. I.e. as far as the outer rr is concerned, the tracee is
+  // not doing syscall buffering.
+
   int found_stacks = 0;
   for (KernelMapIterator it(t); !it.at_end(); ++it) {
     auto& km = it.current();
     int flags = km.flags();
     remote_ptr<void> start = km.start();
-    ASSERT(t, flags & MAP_PRIVATE);
     bool is_stack = found_proper_stack ? km.is_stack() : could_be_stack(km);
     if (is_stack) {
       ++found_stacks;
@@ -1565,8 +1642,8 @@ void AddressSpace::populate_address_space(Task* t) {
       }
     }
 
-    map(start, km.end() - start, km.prot(), flags, km.file_offset_bytes(),
-        km.fsname(), check_device(t, km), km.inode(), nullptr);
+    map(t, start, km.end() - start, km.prot(), flags, km.file_offset_bytes(),
+        km.fsname(), check_device(km), km.inode(), nullptr);
   }
   ASSERT(t, found_stacks == 1);
 }
