@@ -29,6 +29,7 @@
 
 namespace rr {
 
+class AutoRemoteSyscalls;
 class MonitoredSharedMemory;
 class RecordTask;
 class Session;
@@ -230,10 +231,12 @@ public:
   class Mapping {
   public:
     Mapping(const KernelMapping& map, const KernelMapping& recorded_map,
-            EmuFile::shr_ptr emu_file, void* local_addr = nullptr,
+            EmuFile::shr_ptr emu_file,
+            std::unique_ptr<struct stat> mapped_file_stat = nullptr,
+            void* local_addr = nullptr,
             std::shared_ptr<MonitoredSharedMemory>&& monitored = nullptr);
     ~Mapping();
-    Mapping(const Mapping&) = default;
+    Mapping(const Mapping&);
     Mapping() = default;
     const Mapping& operator=(const Mapping& other) {
       this->~Mapping();
@@ -246,6 +249,7 @@ public:
     // equal to 'map'.
     const KernelMapping recorded_map;
     const EmuFile::shr_ptr emu_file;
+    std::unique_ptr<struct stat> mapped_file_stat;
     // If this mapping has been mapped into the local address space,
     // this is the address of the first byte of the equivalent local mapping.
     // This mapping is always mapped as PROT_READ|PROT_WRITE regardless of the
@@ -287,7 +291,7 @@ public:
    * Change the program data break of this address space to
    * |addr|. Only called during recording!
    */
-  void brk(remote_ptr<void> addr, int prot);
+  void brk(Task* t, remote_ptr<void> addr, int prot);
 
   /**
    * This can only be called during recording.
@@ -305,12 +309,6 @@ public:
    */
   void dump() const;
 
-  /**
-   * Return true if this was created as the result of an exec()
-   * call, instead of cloned from another address space.
-   */
-  bool execed() const { return !is_clone; }
-
   /**
    * Return tid of the first task for this address space.
    */
@@ -373,14 +371,19 @@ public:
    * |prot| protection and |flags|.  The pages are (possibly
    * initially) backed starting at |offset| of |res|. |fsname|, |device| and
    * |inode| are values that will appear in the /proc/<pid>/maps entry.
+   * |mapped_file_stat| is a complete copy of the 'stat' data for the mapped
+   * file, or null if this isn't a file mapping or isn't during recording.
    * |*recorded_map| is the mapping during recording, or null if the mapping
    * during recording is known to be the same as the new map (e.g. because
    * we are recording!).
    */
   KernelMapping map(
-      remote_ptr<void> addr, size_t num_bytes, int prot, int flags,
-      off64_t offset_bytes, const std::string& fsname, dev_t device,
-      ino_t inode, const KernelMapping* recorded_map = nullptr,
+      Task* t, remote_ptr<void> addr, size_t num_bytes, int prot, int flags,
+      off64_t offset_bytes, const std::string& fsname,
+      dev_t device = KernelMapping::NO_DEVICE,
+      ino_t inode = KernelMapping::NO_INODE,
+      std::unique_ptr<struct stat> mapped_file_stat = nullptr,
+      const KernelMapping* recorded_map = nullptr,
       EmuFile::shr_ptr emu_file = nullptr, void* local_addr = nullptr,
       std::shared_ptr<MonitoredSharedMemory>&& monitored = nullptr);
 
@@ -451,8 +454,15 @@ public:
   friend class Maps;
   Maps maps() const { return Maps(*this, remote_ptr<void>()); }
   Maps maps_starting_at(remote_ptr<void> start) { return Maps(*this, start); }
+  Maps maps_containing_or_after(remote_ptr<void> start) {
+    if (has_mapping(start)) {
+      return Maps(*this, mapping_of(start).map.start());
+    } else {
+      return Maps(*this, start);
+    }
+  }
 
-  const std::set<remote_ptr<void> >& monitored_addrs() const {
+  const std::set<remote_ptr<void>>& monitored_addrs() const {
     return monitored_mem;
   }
 
@@ -460,7 +470,7 @@ public:
    * Change the protection bits of [addr, addr + num_bytes) to
    * |prot|.
    */
-  void protect(remote_ptr<void> addr, size_t num_bytes, int prot);
+  void protect(Task* t, remote_ptr<void> addr, size_t num_bytes, int prot);
 
   /**
    * Fix up mprotect registers parameters to take account of PROT_GROWSDOWN.
@@ -471,7 +481,7 @@ public:
    * Move the mapping [old_addr, old_addr + old_num_bytes) to
    * [new_addr, old_addr + new_num_bytes), preserving metadata.
    */
-  void remap(remote_ptr<void> old_addr, size_t old_num_bytes,
+  void remap(Task* t, remote_ptr<void> old_addr, size_t old_num_bytes,
              remote_ptr<void> new_addr, size_t new_num_bytes);
 
   /**
@@ -554,12 +564,12 @@ public:
    * Make [addr, addr + num_bytes) inaccessible within this
    * address space.
    */
-  void unmap(remote_ptr<void> addr, ssize_t num_bytes);
+  void unmap(Task* t, remote_ptr<void> addr, ssize_t num_bytes);
 
   /**
    * Notification of madvise call.
    */
-  void advise(remote_ptr<void> addr, ssize_t num_bytes, int advice);
+  void advise(Task* t, remote_ptr<void> addr, ssize_t num_bytes, int advice);
 
   /** Return the vdso mapping of this. */
   KernelMapping vdso() const;
@@ -618,6 +628,13 @@ public:
     return rr_page_start() + rr_page_size();
   }
 
+  static remote_ptr<void> preload_thread_locals_start() {
+    return rr_page_start() + PAGE_SIZE;
+  }
+  static uint32_t preload_thread_locals_size() {
+    return PRELOAD_THREAD_LOCALS_SIZE;
+  }
+
   enum Traced { TRACED, UNTRACED };
   enum Privileged { PRIVILEGED, UNPRIVILEGED };
   enum Enabled { RECORDING_ONLY, REPLAY_ONLY, RECORDING_AND_REPLAY };
@@ -682,24 +699,44 @@ public:
 
   PropertyTable& properties() { return properties_; }
 
+  void post_vm_clone(Task* t);
+  /**
+   * TaskUid for the task whose locals are stored in the preload_thread_locals
+   * area.
+   */
+  const TaskUid& thread_locals_tuid() { return thread_locals_tuid_; }
+  void set_thread_locals_tuid(const TaskUid& tuid) {
+    thread_locals_tuid_ = tuid;
+  }
+
 private:
   struct Breakpoint;
   typedef std::map<remote_code_ptr, Breakpoint> BreakpointMap;
   class Watchpoint;
 
+  /**
+   * Called after a successful execve to set up the new AddressSpace.
+   * Also called once for the initial spawn.
+   */
   AddressSpace(Task* t, const std::string& exe, uint32_t exec_count);
+  /**
+   * Called when an AddressSpace is cloned due to a fork() or a Session
+   * clone. After this, and the task is properly set up, post_vm_clone will
+   * be called.
+   */
   AddressSpace(Session* session, const AddressSpace& o, pid_t leader_tid,
                uint32_t leader_serial, uint32_t exec_count);
+
   /**
    * After an exec, populate the new address space of |t| with
    * the existing mappings we find in /proc/maps.
    */
   void populate_address_space(Task* t);
 
-  void unmap_internal(remote_ptr<void> addr, ssize_t num_bytes);
+  void unmap_internal(Task* t, remote_ptr<void> addr, ssize_t num_bytes);
 
   // Also sets brk_ptr.
-  void map_rr_page(Task* t);
+  void map_rr_page(AutoRemoteSyscalls& remote);
 
   bool update_watchpoint_value(const MemoryRange& range,
                                Watchpoint& watchpoint);
@@ -724,7 +761,7 @@ private:
    * well, for example have adjacent file offsets and the same
    * prot and flags.
    */
-  void coalesce_around(MemoryMap::iterator it);
+  void coalesce_around(Task* t, MemoryMap::iterator it);
 
   /**
    * Erase |it| from |breakpoints| and restore any memory in
@@ -752,9 +789,11 @@ private:
    * Map |m| of |r| into this address space, and coalesce any
    * mappings of |r| that are adjacent to |m|.
    */
-  void map_and_coalesce(const KernelMapping& m,
+  void map_and_coalesce(Task* t, const KernelMapping& m,
                         const KernelMapping& recorded_map,
-                        EmuFile::shr_ptr emu_file, void* local_addr,
+                        EmuFile::shr_ptr emu_file,
+                        std::unique_ptr<struct stat> mapped_file_stat,
+                        void* local_addr,
                         std::shared_ptr<MonitoredSharedMemory>&& monitored);
 
   void remove_from_map(const MemoryRange& range) {
@@ -916,16 +955,16 @@ private:
   remote_ptr<void> brk_start;
   /* Current brk. Not necessarily page-aligned. */
   remote_ptr<void> brk_end;
-  /* Were we cloned from another address space? */
-  bool is_clone;
   /* All segments mapped into this address space. */
   MemoryMap mem;
-  std::set<remote_ptr<void> > monitored_mem;
+  std::set<remote_ptr<void>> monitored_mem;
   /* madvise DONTFORK regions */
   std::set<MemoryRange> dont_fork;
   // The session that created this.  We save a ref to it so that
   // we can notify it when we die.
   Session* session_;
+  // tid of the task whose thread-locals are in preload_thread_locals
+  TaskUid thread_locals_tuid_;
   /* First mapped byte of the vdso. */
   remote_ptr<void> vdso_start_addr;
   // The monkeypatcher that's handling this address space.
@@ -934,7 +973,7 @@ private:
   // programmed per Task, but we track them per address space on
   // behalf of debuggers that assume that model.
   std::map<MemoryRange, Watchpoint> watchpoints;
-  std::vector<std::map<MemoryRange, Watchpoint> > saved_watchpoints;
+  std::vector<std::map<MemoryRange, Watchpoint>> saved_watchpoints;
   // Tracee memory is read and written through this fd, which is
   // opened for the tracee's magic /proc/[tid]/mem device.  The
   // advantage of this over ptrace is that we can access it even
