@@ -30,7 +30,8 @@ template <typename Arch> struct socketcall_args {
 } __attribute__((packed));
 
 void AutoRestoreMem::init(const void* mem, ssize_t num_bytes) {
-  ASSERT(remote.task(), !remote.regs().sp().is_null())
+  ASSERT(remote.task(),
+         remote.enable_mem_params() == AutoRemoteSyscalls::ENABLE_MEMORY_PARAMS)
       << "Memory parameters were disabled";
 
   len = num_bytes;
@@ -63,7 +64,9 @@ AutoRemoteSyscalls::AutoRemoteSyscalls(Task* t,
       initial_regs(t->regs()),
       initial_ip(t->ip()),
       initial_sp(t->regs().sp()),
-      pending_syscallno(-1) {
+      pending_syscallno(-1),
+      scratch_mem_was_mapped(false),
+      enable_mem_params_(enable_mem_params) {
   // We could use privilged_traced_syscall_ip() here, but we don't actually
   // need privileges because tracee seccomp filters are modified to only
   // produce PTRACE_SECCOMP_EVENTs that we ignore. And before the rr page is
@@ -71,9 +74,8 @@ AutoRemoteSyscalls::AutoRemoteSyscalls(Task* t,
   initial_regs.set_ip(t->vm()->traced_syscall_ip());
   if (enable_mem_params == ENABLE_MEMORY_PARAMS) {
     maybe_fix_stack_pointer();
-  } else {
-    initial_regs.set_sp(remote_ptr<void>());
   }
+
   // We need to make sure to clear any breakpoints or other alterations of
   // the syscall instruction we're using. Note that the tracee may have set its
   // own breakpoints or otherwise modified the instruction, so suspending our
@@ -116,14 +118,31 @@ void AutoRemoteSyscalls::maybe_fix_stack_pointer() {
       break;
     }
   };
-  ASSERT(t, !found_stack.start().is_null()) << "No stack area found";
 
-  initial_regs.set_sp(found_stack.end());
+  if (found_stack.start().is_null()) {
+    AutoRemoteSyscalls remote(t, DISABLE_MEMORY_PARAMS);
+    found_stack =
+        MemoryRange(remote.infallible_mmap_syscall(
+                        remote_ptr<void>(), 4096, PROT_READ | PROT_WRITE,
+                        MAP_PRIVATE | MAP_ANONYMOUS, -1, 0),
+                    4096);
+    scratch_mem_was_mapped = true;
+  }
+
+  fixed_sp = found_stack.end();
+  assert(!fixed_sp.is_null());
+  initial_regs.set_sp(fixed_sp);
 }
 
 AutoRemoteSyscalls::~AutoRemoteSyscalls() { restore_state_to(t); }
 
 void AutoRemoteSyscalls::restore_state_to(Task* t) {
+  // Unmap our scatch region if required
+  if (scratch_mem_was_mapped) {
+    AutoRemoteSyscalls remote(t, DISABLE_MEMORY_PARAMS);
+    remote.infallible_syscall(syscall_number_for_munmap(arch()), fixed_sp,
+                              4096);
+  }
   if (!replaced_bytes.empty()) {
     t->write_mem(remote_ptr<uint8_t>(
                      t->vm()->traced_syscall_ip().to_data_ptr<uint8_t>()),
@@ -137,10 +156,12 @@ void AutoRemoteSyscalls::restore_state_to(Task* t) {
 
 void AutoRemoteSyscalls::syscall_helper(SyscallWaiting wait, int syscallno,
                                         Registers& callregs) {
+  LOG(debug) << "syscall " << syscall_name(syscallno, t->arch());
+
   callregs.set_syscallno(syscallno);
   t->set_regs(callregs);
 
-  t->advance_syscall();
+  t->enter_syscall();
 
   ASSERT(t, t->regs().ip() - callregs.ip() ==
                 syscall_instruction_length(t->arch()))
@@ -162,9 +183,12 @@ void AutoRemoteSyscalls::wait_syscall(int syscallno) {
   ASSERT(t, pending_syscallno == syscallno || syscallno < 0);
 
   // Wait for syscall-exit trap (or PTRACE_EVENT_* trap).
+  // XXX we should handle stray signals here!
   t->wait();
   pending_syscallno = -1;
 
+  LOG(debug) << "done, result=" << t->regs().syscall_result();
+
   ASSERT(t, t->regs().original_syscallno() == syscallno || syscallno < 0)
       << "Should be entering " << t->syscall_name(syscallno)
       << ", but instead at " << t->syscall_name(t->regs().original_syscallno());
@@ -178,7 +202,7 @@ static void write_socketcall_args(Task* t, remote_ptr<void> remote_mem,
                                   typename Arch::signed_long arg2,
                                   typename Arch::signed_long arg3) {
   socketcall_args<Arch> sc_args = { { arg1, arg2, arg3 } };
-  t->write_mem(remote_mem.cast<socketcall_args<Arch> >(), sc_args);
+  t->write_mem(remote_mem.cast<socketcall_args<Arch>>(), sc_args);
 }
 
 static size_t align_size(size_t size) {
@@ -228,7 +252,7 @@ static int create_bind_and_listen_socket(const char* path) {
 
 template <typename Arch>
 static int child_create_socket(AutoRemoteSyscalls& remote,
-                               remote_ptr<socketcall_args<Arch> > sc_args) {
+                               remote_ptr<socketcall_args<Arch>> sc_args) {
   int child_sock;
   if (sc_args.is_null()) {
     child_sock =
@@ -245,10 +269,11 @@ static int child_create_socket(AutoRemoteSyscalls& remote,
 template <typename Arch>
 static void child_connect_socket(AutoRemoteSyscalls& remote,
                                  AutoRestoreMem& remote_buf,
-                                 remote_ptr<socketcall_args<Arch> > sc_args,
+                                 remote_ptr<socketcall_args<Arch>> sc_args,
                                  remote_ptr<void> buf_end, int child_sock,
                                  const char* path, int* cwd_fd) {
   typename Arch::sockaddr_un addr;
+  memset(&addr, 0, sizeof(addr)); // Make valgrind happy.
   addr.sun_family = AF_UNIX;
   assert(strlen(path) < sizeof(addr.sun_path));
   // Skip leading '/' since we're going to access this relative to the root
@@ -317,9 +342,10 @@ static void restore_cwd(AutoRemoteSyscalls& remote, int cwd_fd) {
 template <typename Arch>
 static void child_sendmsg(AutoRemoteSyscalls& remote,
                           AutoRestoreMem& remote_buf,
-                          remote_ptr<socketcall_args<Arch> > sc_args,
+                          remote_ptr<socketcall_args<Arch>> sc_args,
                           remote_ptr<void> buf_end, int child_sock, int fd) {
   char cmsgbuf[Arch::cmsg_space(sizeof(fd))];
+  memset(cmsgbuf, 0, sizeof(cmsgbuf));
   // Pull the puppet strings to have the child send its fd
   // to us.  Similarly to above, we DONT_WAIT on the
   // call to finish, since it's likely not defined whether the
@@ -405,14 +431,14 @@ template <typename Arch> ScopedFd AutoRemoteSyscalls::retrieve_fd_arch(int fd) {
                                     align_size(Arch::cmsg_space(sizeof(fd))) +
                                     reserve<typename Arch::iovec>());
   if (has_socketcall_syscall(Arch::arch())) {
-    data_length += reserve<socketcall_args<Arch> >();
+    data_length += reserve<socketcall_args<Arch>>();
   }
   AutoRestoreMem remote_buf(*this, nullptr, data_length);
 
   remote_ptr<void> sc_args_end = remote_buf.get();
-  remote_ptr<socketcall_args<Arch> > sc_args;
+  remote_ptr<socketcall_args<Arch>> sc_args;
   if (has_socketcall_syscall(Arch::arch())) {
-    sc_args = allocate<socketcall_args<Arch> >(&sc_args_end, remote_buf);
+    sc_args = allocate<socketcall_args<Arch>>(&sc_args_end, remote_buf);
   }
 
   char path[PATH_MAX];
