@@ -9,11 +9,13 @@
 
 #include <algorithm>
 
-#include "main.h"
 #include "TraceStream.h"
+#include "main.h"
 
 using namespace std;
 
+namespace rr {
+
 bool ParsedOption::verify_valid_int(int64_t min, int64_t max) const {
   if (int_value < min || int_value > max) {
     return false;
@@ -97,7 +99,7 @@ bool Command::parse_option(std::vector<std::string>& args,
   }
 
   for (size_t i = 0; i < count; ++i) {
-    if (args[0][1] == option_specs[i].short_name) {
+    if (args[0][1] == option_specs[i].short_name && args[0][1] >= 32) {
       out->short_name = option_specs[i].short_name;
       switch (option_specs[i].param) {
         case NO_PARAMETER:
@@ -167,3 +169,5 @@ bool Command::parse_optional_trace_dir(vector<string>& args, string* out) {
   }
   return true;
 }
+
+} // namespace rr
