@@ -7,11 +7,13 @@
 #include <stdint.h>
 
 #include <memory>
-#include <vector>
 #include <string>
+#include <vector>
 
 #include "ScopedFd.h"
 
+namespace rr {
+
 /**
  * CompressedReader opens an input file written by CompressedWriter
  * and reads data from it. Currently data is decompressed by the thread that
@@ -94,4 +96,6 @@ protected:
   size_t saved_buffer_read_pos;
 };
 
+} // namespace rr
+
 #endif /* RR_COMPRESSED_READER_H_ */
