@@ -194,14 +194,18 @@ static void dump_events_matching(TraceReader& trace, const DumpFlags& flags,
           if (km.flags() & MAP_SHARED) {
             prot_flags[3] = 's';
           }
+          const char* fsname = km.fsname().c_str();
+          if (data.source == TraceReader::SOURCE_ZERO) {
+            const char source_zero[] = "<ZERO>";
+            fsname = source_zero;
+          }
           fprintf(out, "  { map_file:\"%s\", addr:%p, length:%p, "
                        "prot_flags:\"%s\", file_offset:0x%llx, "
                        "data_file:\"%s\", data_offset:0x%llx, "
                        "file_size:0x%llx }\n",
-                  km.fsname().c_str(), (void*)km.start().as_int(),
-                  (void*)km.size(), prot_flags,
-                  (long long)km.file_offset_bytes(), data.file_name.c_str(),
-                  (long long)data.data_offset_bytes,
+                  fsname, (void*)km.start().as_int(), (void*)km.size(),
+                  prot_flags, (long long)km.file_offset_bytes(),
+                  data.file_name.c_str(), (long long)data.data_offset_bytes,
                   (long long)data.file_size_bytes);
         }
       }
@@ -209,8 +213,8 @@ static void dump_events_matching(TraceReader& trace, const DumpFlags& flags,
       TraceReader::RawData data;
       while (process_raw_data && trace.read_raw_data_for_frame(frame, data)) {
         if (flags.dump_recorded_data_metadata) {
-          fprintf(out, "  { addr:%p, length:%p }\n", (void*)data.addr.as_int(),
-                  (void*)data.data.size());
+          fprintf(out, "  { tid:%d, addr:%p, length:%p }\n", data.rec_tid,
+                  (void*)data.addr.as_int(), (void*)data.data.size());
         }
       }
       vector<uint8_t> buf;
