@@ -132,6 +132,15 @@ static RegData xsave_register_data(SupportedArch arch, GdbRegister regno) {
   return RegData(fxsave_387_ctrl_offsets[regno - DREG_64_FCTRL], 4);
 }
 
+static uint64_t xsave_features(const vector<uint8_t>& data) {
+  // If this is just FXSAVE(64) data then we we have no XSAVE header and no
+  // XSAVE(64) features enabled.
+  return data.size() < xsave_header_offset + xsave_header_size
+             ? 0
+             : *reinterpret_cast<const uint64_t*>(data.data() +
+                                                  xsave_header_offset);
+}
+
 size_t ExtraRegisters::read_register(uint8_t* buf, GdbRegister regno,
                                      bool* defined) const {
   if (format_ != XSAVE) {
@@ -146,20 +155,13 @@ size_t ExtraRegisters::read_register(uint8_t* buf, GdbRegister regno,
   }
 
   assert(reg_data.size > 0);
-  // If this is just FXSAVE(64) data then we we have no XSAVE header and no
-  // XSAVE(64) features enabled.
-  uint64_t xsave_features =
-      data_.size() < xsave_header_offset + xsave_header_size
-          ? 0
-          : *reinterpret_cast<const uint64_t*>(data_.data() +
-                                               xsave_header_offset);
 
   *defined = true;
 
   // Apparently before any AVX registers are used, the feature bit is not set
   // in the XSAVE data, so we'll just return 0 for them here.
   if (reg_data.xsave_feature_bit >= 0 &&
-      !(xsave_features & (1 << reg_data.xsave_feature_bit))) {
+      !(xsave_features(data_) & (1 << reg_data.xsave_feature_bit))) {
     memset(buf, 0, reg_data.size);
   } else {
     assert(size_t(reg_data.offset + reg_data.size) <= data_.size());
@@ -168,6 +170,23 @@ size_t ExtraRegisters::read_register(uint8_t* buf, GdbRegister regno,
   return reg_data.size;
 }
 
+void ExtraRegisters::validate(Task* t) {
+  if (format_ != XSAVE) {
+    return;
+  }
+
+  ASSERT(t, data_.size() >= 512);
+  uint32_t offset = 512;
+  if (data_.size() > offset) {
+    ASSERT(t, data_.size() >= offset + 64);
+    offset += 64;
+    uint64_t features = xsave_features(data_);
+    if (features & AVX_FEATURE) {
+      ASSERT(t, data_.size() >= offset + 256);
+    }
+  }
+}
+
 static void print_reg(const ExtraRegisters& r, GdbRegister low, GdbRegister hi,
                       const char* name, FILE* f) {
   uint8_t buf[128];
@@ -182,10 +201,16 @@ static void print_reg(const ExtraRegisters& r, GdbRegister low, GdbRegister hi,
     }
   }
   char out[257];
+  bool printed_digit = false;
+  char* p = out;
   for (int i = len - 1; i >= 0; --i) {
-    sprintf(out + (len - 1 - i) * 2, "%02x", buf[i]);
+    if (!printed_digit && !buf[i] && i > 0) {
+      continue;
+    }
+    p += sprintf(p, printed_digit ? "%02x" : "%x", buf[i]);
+    printed_digit = true;
   }
-  fprintf(f, "%s:%s ", name, out);
+  fprintf(f, "%s:0x%s", name, out);
 }
 
 static void print_regs(const ExtraRegisters& r, GdbRegister low, GdbRegister hi,
@@ -195,6 +220,9 @@ static void print_regs(const ExtraRegisters& r, GdbRegister low, GdbRegister hi,
     sprintf(buf, "%s%d", name_base, i);
     print_reg(r, (GdbRegister)(low + i),
               hi == GdbRegister(0) ? hi : (GdbRegister)(hi + i), buf, f);
+    if (i < num_regs - 1) {
+      fputc(' ', f);
+    }
   }
 }
 
@@ -315,4 +343,33 @@ void ExtraRegisters::set_user_fpxregs_struct(
   memcpy(data_.data(), &regs, sizeof(regs));
 }
 
+static void set_word(SupportedArch arch, vector<uint8_t>& v, GdbRegister r,
+                     int word) {
+  RegData d = xsave_register_data(arch, r);
+  assert(d.size == 4);
+  assert(d.offset + d.size <= (int)v.size());
+  assert(-1 == d.xsave_feature_bit);
+  *reinterpret_cast<int*>(v.data() + d.offset) = word;
+}
+
+void ExtraRegisters::reset() {
+  assert(format_ == XSAVE);
+  memset(data_.data(), 0, data_.size());
+  switch (arch()) {
+    case x86_64: {
+      set_word(arch(), data_, DREG_64_MXCSR, 0x1f80);
+      set_word(arch(), data_, DREG_64_FCTRL, 0x37f);
+      break;
+    }
+    case x86: {
+      set_word(arch(), data_, DREG_MXCSR, 0x1f80);
+      set_word(arch(), data_, DREG_FCTRL, 0x37f);
+      break;
+    }
+    default:
+      assert(0 && "Unknown arch");
+      break;
+  }
+}
+
 } // namespace rr
