@@ -101,8 +101,14 @@ class RRCmd(gdb.Command):
         response = gdb_unescape(rv_match.group(1))
         gdb.write(response)
 
-end
+def history_push(p):
+    gdb.execute("rr-history-push", to_string=True)
 
+#Automatically push an history entry when the program execution stops
+#(signal, breakpoint).This is fired before an interactive prompt is shown.
+gdb.events.stop.connect(history_push)
+
+end
 )Delimiter");
 
   if (gdb_command_list) {
@@ -110,6 +116,17 @@ end
       ss << gdb_macro_binding(*it);
     }
   }
+
+  ss << string(R"Delimiter(
+define hookpost-back
+frame
+end
+
+define hookpost-forward
+frame
+end
+)Delimiter");
+
   return ss.str();
 }
 
@@ -176,6 +193,12 @@ static vector<string> parse_cmd(string& str) {
   }
   LOG(debug) << "invoking command: " << cmd->name();
   string resp = cmd->invoke(gdb_server, t, args);
+
+  if (resp == GdbCommandHandler::cmd_end_diversion()) {
+    LOG(debug) << "cmd must run outside of diversion (" << resp << ")";
+    return resp;
+  }
+
   LOG(debug) << "cmd response: " << resp;
   return gdb_escape(resp + "\n");
 }
