@@ -233,7 +233,7 @@ void GdbServer::dispatch_regs_request(const Registers& regs,
       break;
     default:
       FATAL() << "Unknown architecture";
-      break;
+      return;
   }
   vector<GdbRegisterValue> rs;
   for (GdbRegister r = GdbRegister(0); r < end; r = GdbRegister(r + 1)) {
@@ -244,7 +244,7 @@ void GdbServer::dispatch_regs_request(const Registers& regs,
 
 class GdbBreakpointCondition : public BreakpointCondition {
 public:
-  GdbBreakpointCondition(const vector<vector<uint8_t> >& bytecodes) {
+  GdbBreakpointCondition(const vector<vector<uint8_t>>& bytecodes) {
     for (auto& b : bytecodes) {
       expressions.push_back(GdbExpression(b.data(), b.size()));
     }
@@ -546,6 +546,48 @@ void GdbServer::dispatch_debugger_request(Session& session,
       dbg->reply_rr_cmd(
           GdbCommandHandler::process_command(*this, target, req.text()));
       return;
+    case DREQ_QSYMBOL: {
+      // When gdb sends "qSymbol::", it means that gdb is ready to
+      // respond to symbol requests.  This can be sent multiple times
+      // during the course of a session -- gdb sends it whenever
+      // something in the inferior has changed, making it possible
+      // that previous failed symbol lookups could now succeed.  In
+      // response to a qSymbol request from gdb, we either send back a
+      // qSymbol response, requesting the address of a symbol; or we
+      // send back OK.  We have to do this as an ordinary response and
+      // maintain our own state explicitly, as opposed to simply
+      // reading another packet from gdb, because when gdb looks up a
+      // symbol it might send other requests that must be served.  So,
+      // we keep a copy of the symbol names, and an iterator into this
+      // copy.  When gdb sends a plain "qSymbol::" packet, because gdb
+      // has detected some change in the inferior state that might
+      // enable more symbol lookups, we restart the iterator.
+      const string& name = req.sym().name;
+      if (req.sym().has_address) {
+        // Got a response holding a previously-requested symbol's name
+        // and address.
+        target->register_symbol(name, req.sym().address);
+      } else if (name == "") {
+        // Plain "qSymbol::" request.
+        symbols = target->get_symbols_and_clear_map();
+        symbols_iter = symbols.begin();
+      }
+
+      if (symbols_iter == symbols.end()) {
+        dbg->qsymbols_finished();
+      } else {
+        string symbol = *symbols_iter++;
+        dbg->send_qsymbol(symbol);
+      }
+      return;
+    }
+    case DREQ_TLS: {
+      remote_ptr<void> address;
+      bool ok = target->get_tls_address(req.tls().offset, req.tls().load_module,
+                                        &address);
+      dbg->reply_tls_addr(ok, address);
+      return;
+    }
     default:
       FATAL() << "Unknown debugger request " << req.type;
   }
@@ -564,7 +606,6 @@ bool GdbServer::diverter_process_debugger_requests(
     switch (req->type) {
       case DREQ_RESTART:
       case DREQ_DETACH:
-      case DREQ_RR_CMD:
         diversion_refcount = 0;
         return false;
 
@@ -599,10 +640,30 @@ bool GdbServer::diverter_process_debugger_requests(
         dbg->reply_write_siginfo();
         continue;
 
+      case DREQ_RR_CMD: {
+        assert(req->type == DREQ_RR_CMD);
+        Task* task = diversion_session.find_task(last_continue_tuid);
+        if (task) {
+          std::string reply =
+              GdbCommandHandler::process_command(*this, task, req->text());
+          // Certain commands cause the diversion to end immediately
+          // while other commands must work within a diversion.
+          if (reply == GdbCommandHandler::cmd_end_diversion()) {
+            diversion_refcount = 0;
+            return false;
+          }
+          dbg->reply_rr_cmd(reply);
+          continue;
+        } else {
+          diversion_refcount = 0;
+          return false;
+        }
+        break;
+      }
+
       default:
         break;
     }
-
     dispatch_debugger_request(diversion_session, *req, REPORT_NORMAL);
   }
 }
@@ -1069,7 +1130,7 @@ bool GdbServer::at_target() {
   return timeline.current_session().current_trace_frame().time() >
              target.event &&
          (!target.pid || t->tgid() == target.pid) &&
-         (!target.require_exec || t->vm()->execed()) &&
+         (!target.require_exec || t->execed()) &&
          // Ensure we're at the start of processing an event. We don't
          // want to attach while we're finishing an exec() since that's a
          // slightly confusing state for ReplayTimeline's reverse execution.
