@@ -104,6 +104,8 @@ public:
                                   const ExtraRegisters& extra_regs,
                                   GdbRegister which);
 
+  ReplayTimeline& get_timeline() { return timeline; }
+
 private:
   GdbServer(std::unique_ptr<GdbConnection>& dbg, Task* t);
 
@@ -234,6 +236,11 @@ private:
 
   // gdb checkpoints, indexed by ID
   std::map<int, Checkpoint> checkpoints;
+
+  // Set of symbols to look up, for qSymbol.
+  std::set<std::string> symbols;
+  // Iterator into |symbols|.
+  std::set<std::string>::iterator symbols_iter;
 };
 
 } // namespace rr
