@@ -85,10 +85,12 @@ template <typename Arch> static void setup_preload_library_path(RecordTask* t) {
     }
     size_t next_colon = env.find(':', lib_pos);
     if (next_colon != string::npos) {
-      while (env[next_colon + 1] == ':' || env[next_colon + 1] == 0) {
+      while ((next_colon + 1 < env.length()) &&
+             (env[next_colon + 1] == ':' || env[next_colon + 1] == 0)) {
         ++next_colon;
       }
-      if (next_colon < lib_pos + sizeof(SYSCALLBUF_LIB_FILENAME_PADDED) - 1) {
+      if (next_colon + 1 <
+          lib_pos + sizeof(SYSCALLBUF_LIB_FILENAME_PADDED) - 1) {
         LOG(debug) << "Insufficient space for " << lib_name
                    << " in LD_PRELOAD before next ':'";
         return;
@@ -198,8 +200,8 @@ static remote_ptr<uint8_t> allocate_extended_jump(
       KernelMapping recorded(addr, addr + page_size(), string(),
                              KernelMapping::NO_DEVICE, KernelMapping::NO_INODE,
                              prot, flags);
-      t->vm()->map(addr, page_size(), prot, flags, 0, string(),
-                   KernelMapping::NO_DEVICE, KernelMapping::NO_INODE,
+      t->vm()->map(t, addr, page_size(), prot, flags, 0, string(),
+                   KernelMapping::NO_DEVICE, KernelMapping::NO_INODE, nullptr,
                    &recorded);
       t->trace_writer().write_mapped_region(t, recorded, recorded.fake_stat(),
                                             TraceWriter::PATCH_MAPPING);
@@ -379,17 +381,18 @@ bool Monkeypatcher::try_patch_syscall(RecordTask* t) {
       }
 
       if (!found_potential_interfering_branch) {
-        // Get out of executing the current syscall before we patch it.
-        t->exit_syscall_and_prepare_restart();
-
-        patch_syscall_with_hook(*this, t, hook);
-
         LOG(debug) << "Patched syscall at " << r.ip() << " syscall "
                    << syscall_name(syscallno, t->arch()) << " tid " << t->tid
                    << " bytes "
                    << bytes_to_string(
                           following_bytes,
                           sizeof(syscall_patch_hook::next_instruction_bytes));
+
+        // Get out of executing the current syscall before we patch it.
+        t->exit_syscall_and_prepare_restart();
+
+        patch_syscall_with_hook(*this, t, hook);
+
         // Return to caller, which resume normal execution.
         return true;
       }
@@ -566,7 +569,7 @@ template <>
 void patch_at_preload_init_arch<X86Arch>(RecordTask* t,
                                          Monkeypatcher& patcher) {
   auto params = t->read_mem(
-      remote_ptr<rrcall_init_preload_params<X86Arch> >(t->regs().arg1()));
+      remote_ptr<rrcall_init_preload_params<X86Arch>>(t->regs().arg1()));
   if (!params.syscallbuf_enabled) {
     return;
   }
@@ -658,7 +661,7 @@ template <>
 void patch_at_preload_init_arch<X64Arch>(RecordTask* t,
                                          Monkeypatcher& patcher) {
   auto params = t->read_mem(
-      remote_ptr<rrcall_init_preload_params<X64Arch> >(t->regs().arg1()));
+      remote_ptr<rrcall_init_preload_params<X64Arch>>(t->regs().arg1()));
   if (!params.syscallbuf_enabled) {
     return;
   }
