@@ -8,9 +8,12 @@
 
 #include "preload/preload_interface.h"
 
-#include "remote_ptr.h"
 #include "remote_code_ptr.h"
+#include "remote_ptr.h"
 
+namespace rr {
+
+class RecordTask;
 class ScopedFd;
 class Task;
 
@@ -33,21 +36,21 @@ class Task;
  */
 class Monkeypatcher {
 public:
-  Monkeypatcher() : stub_buffer_allocated(0) {}
-  Monkeypatcher(const Monkeypatcher& o) = default;
+  Monkeypatcher() {}
+  Monkeypatcher(const Monkeypatcher&) = default;
 
   /**
    * Apply any necessary patching immediately after exec.
    * In this hook we patch everything that doesn't depend on the preload
    * library being loaded.
    */
-  void patch_after_exec(Task* t);
+  void patch_after_exec(RecordTask* t);
 
   /**
    * During librrpreload initialization, apply patches that require the
    * preload library to be initialized.
    */
-  void patch_at_preload_init(Task* t);
+  void patch_at_preload_init(RecordTask* t);
 
   /**
    * Try to patch the syscall instruction that |t| just entered. If this
@@ -58,24 +61,25 @@ public:
    * Zero or more mapping operations are also recorded to the trace and must
    * be replayed.
    */
-  bool try_patch_syscall(Task* t);
+  bool try_patch_syscall(RecordTask* t);
 
   void init_dynamic_syscall_patching(
-      Task* t, int syscall_patch_hook_count,
+      RecordTask* t, int syscall_patch_hook_count,
       remote_ptr<syscall_patch_hook> syscall_patch_hooks,
-      remote_ptr<void> stub_buffer, remote_ptr<void> stub_buffer_end);
+      remote_ptr<void> syscall_hook_trampoline,
+      remote_ptr<void> syscall_hook_end);
 
   /**
    * Try to allocate a stub from the sycall patching stub buffer. Returns null
    * if there's no buffer or we've run out of free stubs.
    */
-  remote_ptr<uint8_t> allocate_stub(Task* t, size_t bytes);
+  remote_ptr<uint8_t> allocate_stub(RecordTask* t, size_t bytes);
 
   /**
    * Apply any necessary patching immediately after an mmap. We use this to
    * patch libpthread.so.
    */
-  void patch_after_mmap(Task* t, remote_ptr<void> start, size_t size,
+  void patch_after_mmap(RecordTask* t, remote_ptr<void> start, size_t size,
                         size_t offset_pages, int child_fd);
 
   remote_ptr<void> x86_sysenter_vsyscall;
@@ -89,6 +93,15 @@ public:
   };
   std::vector<ExtendedJumpPage> extended_jump_pages;
 
+  /**
+   * Returns true if the instruction at address p should be considered
+   * "not part of the syscallbuf code", i.e. it's safe to deliver signals there
+   * without affecting the syscall buffering logic. If not sure, return false.
+   */
+  bool is_syscallbuf_excluded_instruction(remote_ptr<void> p) {
+    return p >= syscall_hook_trampoline && p < syscall_hook_end;
+  }
+
 private:
   /**
    * The list of supported syscall patches obtained from the preload
@@ -101,12 +114,12 @@ private:
    * (or are currently trying) to patch.
    */
   std::unordered_set<remote_code_ptr> tried_to_patch_syscall_addresses;
-  /**
-   * Writable executable memory where we can generate stubs.
-   */
-  remote_ptr<void> stub_buffer;
-  remote_ptr<void> stub_buffer_end;
-  size_t stub_buffer_allocated;
+
+  // The addresses that contain our syscall hooks
+  remote_ptr<void> syscall_hook_trampoline;
+  remote_ptr<void> syscall_hook_end;
 };
 
+} // namespace rr
+
 #endif /* RR_MONKEYPATCHER_H_ */
