@@ -38,6 +38,8 @@ public:
   PerfCounters(pid_t tid);
   ~PerfCounters() { stop(); }
 
+  void set_tid(pid_t tid);
+
   // Change this to 'true' to enable perf counters that may be interesting
   // for experimentation, but aren't necessary for core functionality.
   static bool extra_perf_counters_enabled() { return false; }
@@ -57,6 +59,13 @@ public:
    */
   void stop();
 
+  /**
+   * Suspend counting until the next reset. This may or may not actually stop
+   * the performance counters, depending on whether or not this is required
+   * for correctness on this kernel version.
+   */
+  void stop_counting();
+
   /**
    * Read the current value of the ticks counter.
    */
@@ -81,6 +90,7 @@ public:
   Extra read_extra();
 
   static const struct perf_event_attr& ticks_attr();
+  bool counting;
 
 private:
   pid_t tid;
