@@ -230,8 +230,11 @@ static void note_entering_syscall(RecordTask* t) {
   }
 }
 
-static void handle_seccomp_traced_syscall(
-    RecordTask* t, RecordSession::StepState* step_state) {
+void RecordSession::handle_seccomp_traced_syscall(RecordTask* t,
+                                                  StepState* step_state,
+                                                  RecordResult* result,
+                                                  bool* did_enter_syscall) {
+  *did_enter_syscall = false;
   int syscallno = t->regs().original_syscallno();
   if (syscallno < 0) {
     // negative syscall numbers after a SECCOMP event
@@ -244,10 +247,30 @@ static void handle_seccomp_traced_syscall(
     // enter syscall_state_changed and that will trigger a continue to
     // the syscall exit.
     step_state->continue_type = RecordSession::DONT_CONTINUE;
-  } else {
+    return;
+  }
+
+  if (syscall_seccomp_ordering_ == SECCOMP_BEFORE_PTRACE_SYSCALL) {
     // The next continue needs to be a PTRACE_SYSCALL to observe
     // the enter-syscall event.
     step_state->continue_type = RecordSession::CONTINUE_SYSCALL;
+  } else {
+    ASSERT(t, syscall_seccomp_ordering_ == PTRACE_SYSCALL_BEFORE_SECCOMP);
+    if (t->ev().is_syscall_event() &&
+        t->ev().Syscall().state == PROCESSING_SYSCALL) {
+      // We did PTRACE_SYSCALL and already saw a syscall trap. Just ignore this.
+      LOG(debug) << "Ignoring SECCOMP syscall trap since we already got a "
+                    "PTRACE_SYSCALL trap";
+      // The next continue needs to be a PTRACE_SYSCALL to observe
+      // the exit-syscall event.
+      step_state->continue_type = RecordSession::CONTINUE_SYSCALL;
+    } else {
+      // We've already passed the PTRACE_SYSCALL trap for syscall entry, so
+      // we need to handle that now.
+      t->emulate_syscall_entry(t->regs());
+      process_syscall_entry(t, step_state, result);
+      *did_enter_syscall = true;
+    }
   }
 }
 
@@ -328,7 +351,11 @@ static void handle_seccomp_errno(RecordTask* t,
   step_state->continue_type = RecordSession::DONT_CONTINUE;
 }
 
-bool RecordSession::handle_ptrace_event(RecordTask* t, StepState* step_state) {
+bool RecordSession::handle_ptrace_event(RecordTask* t, StepState* step_state,
+                                        RecordResult* result,
+                                        bool* did_enter_syscall) {
+  *did_enter_syscall = false;
+
   if (t->status().group_stop()) {
     last_task_switchable = ALLOW_SWITCH;
     step_state->continue_type = DONT_CONTINUE;
@@ -346,10 +373,15 @@ bool RecordSession::handle_ptrace_event(RecordTask* t, StepState* step_state) {
   switch (event) {
     case PTRACE_EVENT_SECCOMP_OBSOLETE:
     case PTRACE_EVENT_SECCOMP: {
-      t->seccomp_bpf_enabled = true;
+      if (syscall_seccomp_ordering_ == PTRACE_SYSCALL_BEFORE_SECCOMP_UNKNOWN) {
+        syscall_seccomp_ordering_ = SECCOMP_BEFORE_PTRACE_SYSCALL;
+      }
+
       uint16_t seccomp_data = t->get_ptrace_eventmsg_seccomp_data();
       if (seccomp_data == SECCOMP_RET_DATA) {
-        handle_seccomp_traced_syscall(t, step_state);
+        LOG(debug) << "  traced syscall entered: "
+                   << syscall_name(t->regs().original_syscallno(), t->arch());
+        handle_seccomp_traced_syscall(t, step_state, result, did_enter_syscall);
       } else {
         uint32_t real_result =
             seccomp_filter_rewriter().map_filter_data_to_real_result(
@@ -357,9 +389,11 @@ bool RecordSession::handle_ptrace_event(RecordTask* t, StepState* step_state) {
         uint16_t real_result_data = real_result & SECCOMP_RET_DATA;
         switch (real_result & SECCOMP_RET_ACTION) {
           case SECCOMP_RET_TRAP:
+            LOG(debug) << "  seccomp trap";
             handle_seccomp_trap(t, step_state, real_result_data);
             break;
           case SECCOMP_RET_ERRNO:
+            LOG(debug) << "  seccomp errno";
             handle_seccomp_errno(t, step_state, real_result_data);
             break;
           default:
@@ -371,6 +405,8 @@ bool RecordSession::handle_ptrace_event(RecordTask* t, StepState* step_state) {
     }
 
     case PTRACE_EVENT_EXEC:
+      ASSERT(t, t->task_group()->task_set().size() == 1)
+          << "Found lingering task which is not the task-group leader???";
       t->post_exec();
 
       // Skip past the ptrace event.
@@ -435,12 +471,18 @@ void RecordSession::task_continue(const StepState& step_state) {
     if (singlestep) {
       resume = RESUME_SINGLESTEP;
     } else {
-      if (!t->seccomp_bpf_enabled || may_restart) {
-        /* We won't receive PTRACE_EVENT_SECCOMP events until
-         * the seccomp filter is installed by the
-         * syscall_buffer lib in the child, therefore we must
-         * record in the traditional way (with PTRACE_SYSCALL)
-         * until it is installed. */
+      /* We won't receive PTRACE_EVENT_SECCOMP events until
+       * the seccomp filter is installed by the
+       * syscall_buffer lib in the child, therefore we must
+       * record in the traditional way (with PTRACE_SYSCALL)
+       * until it is installed. */
+      /* Kernel commit
+         https://github.com/torvalds/linux/commit/93e35efb8de45393cf61ed07f7b407629bf698ea
+         makes PTRACE_SYSCALL traps be delivered *before* seccomp RET_TRACE
+         traps.
+         Detect and handle this. */
+      if (!t->seccomp_bpf_enabled || may_restart ||
+          syscall_seccomp_ordering_ == PTRACE_SYSCALL_BEFORE_SECCOMP_UNKNOWN) {
         resume = RESUME_SYSCALL;
       } else {
         /* When the seccomp filter is on, instead of capturing
@@ -851,6 +893,42 @@ void RecordSession::check_initial_task_syscalls(RecordTask* t,
   }
 }
 
+RecordTask* RecordSession::revive_task_for_exec(pid_t rec_tid) {
+  unsigned long msg = 0;
+  int ret =
+      ptrace(__ptrace_request(PTRACE_GETEVENTMSG), rec_tid, nullptr, &msg);
+  if (ret < 0) {
+    FATAL() << "Can't get old tid for execve";
+  }
+  RecordTask* t = find_task(msg);
+  if (!t) {
+    FATAL() << "Can't find old task for execve";
+  }
+  ASSERT(t, rec_tid == t->tgid());
+
+  LOG(debug) << "Changing task tid from " << t->tid << " to " << rec_tid;
+
+  // Pretend the old task cloned a new task with the right tid, and then exited
+  trace_writer().write_task_event(TraceTaskEvent::for_clone(
+      rec_tid, t->tid, CLONE_VM | CLONE_FS | CLONE_FILES | CLONE_SIGHAND |
+                           CLONE_THREAD | CLONE_SYSVSEM));
+  trace_writer().write_task_event(
+      TraceTaskEvent::for_exit(t->tid, WaitStatus::for_exit_code(0)));
+
+  // Account for tid change
+  task_map.erase(t->tid);
+  task_map.insert(make_pair(rec_tid, t));
+  // Update the serial as if this task was really created by cloning the old
+  // task.
+  t->set_tid_and_update_serial(rec_tid);
+
+  // t probably would have been marked for unstable-exit when the old
+  // thread-group leader died.
+  t->unstable = false;
+
+  return t;
+}
+
 /**
  * Take a NativeArch::siginfo_t& here instead of siginfo_t because different
  * versions of system headers have inconsistent field naming.
@@ -884,7 +962,7 @@ static void setup_sigframe_siginfo(RecordTask* t, const siginfo_t& siginfo) {
 /**
  * Get t into a state where resume_execution with a signal will actually work.
  */
-static void preinject_signal(RecordTask* t) {
+static bool preinject_signal(RecordTask* t) {
   int sig = t->ev().Signal().siginfo.si_signo;
 
   /* Signal injection is tricky. Per the ptrace(2) man page, injecting
@@ -918,6 +996,10 @@ static void preinject_signal(RecordTask* t) {
          * it would... */
         continue;
       }
+      if (t->status().ptrace_event() == PTRACE_EVENT_EXIT) {
+        /* We raced with an exit (e.g. due to a pending SIGKILL). */
+        return false;
+      }
       ASSERT(t, t->stop_sig());
       if (t->stop_sig() == sig) {
         LOG(debug) << "    stopped with signal " << signal_name(sig);
@@ -946,6 +1028,7 @@ static void preinject_signal(RecordTask* t) {
    */
   LOG(debug) << "    injecting signal number " << t->ev().Signal().siginfo;
   t->set_siginfo(t->ev().Signal().siginfo);
+  return true;
 }
 
 /**
@@ -954,7 +1037,9 @@ static void preinject_signal(RecordTask* t) {
  * occurred during delivery.
  */
 static bool inject_handled_signal(RecordTask* t) {
-  preinject_signal(t);
+  if (!preinject_signal(t)) {
+    return false;
+  }
 
   int sig = t->ev().Signal().siginfo.si_signo;
   t->resume_execution(RESUME_SINGLESTEP, RESUME_WAIT, RESUME_NO_TICKS, sig);
@@ -971,13 +1056,22 @@ static bool inject_handled_signal(RecordTask* t) {
 
   if (t->stop_sig() == SIGSEGV) {
     // Constructing the signal handler frame must have failed. The kernel will
-    // kill the process after this. Stash the signal and mark it as blocked so
-    // we know to treat it as fatal when we inject it.
+    // kill the process after this. Stash the signal and make sure
+    // we know to treat it as fatal when we inject it. Also disable the
+    // signal handler to match what the kernel does.
+    t->set_sig_blocked(SIGSEGV, false);
+    t->set_sig_handler_default(SIGSEGV);
+
     t->stash_sig();
-    t->set_sig_blocked(SIGSEGV);
+    t->task_group()->received_sigframe_SIGSEGV = true;
     return false;
   }
 
+  t->apply_sig_sa_mask(sig);
+  if (!t->signal_handler_nodefer(sig)) {
+    t->set_sig_blocked(sig, true);
+  }
+
   // We stepped into a user signal handler.
   ASSERT(t, t->stop_sig() == SIGTRAP);
   ASSERT(t, t->get_signal_user_handler(sig) == t->ip());
@@ -995,6 +1089,11 @@ static bool inject_handled_signal(RecordTask* t) {
 
 static bool is_fatal_signal(RecordTask* t, int sig,
                             SignalDeterministic deterministic) {
+  if (t->task_group()->received_sigframe_SIGSEGV) {
+    // Can't be blocked, caught or ignored
+    return true;
+  }
+
   signal_action action = default_action(sig);
   if (action != DUMP_CORE && action != TERMINATE) {
     // If the default action doesn't kill the process, it won't die.
@@ -1016,9 +1115,6 @@ static bool is_fatal_signal(RecordTask* t, int sig,
 
 /**
  * |t| is being delivered a signal, and its state changed.
- *
- * Return true if execution was incidentally resumed to a new event,
- * false otherwise.
  */
 void RecordSession::signal_state_changed(RecordTask* t, StepState* step_state) {
   int sig = t->ev().Signal().siginfo.si_signo;
@@ -1039,15 +1135,18 @@ void RecordSession::signal_state_changed(RecordTask* t, StepState* step_state) {
 
       // If a signal is blocked but is still delivered (e.g. a synchronous
       // terminating signal such as SIGSEGV), user handlers do not run.
+      bool has_handler = false;
       if (t->signal_has_user_handler(sig) && !blocked) {
         LOG(debug) << "  " << t->tid << ": " << signal_name(sig)
                    << " has user handler";
+        has_handler = true;
 
         if (!inject_handled_signal(t)) {
           // Signal delivery isn't happening. Prepare to process the new
           // signal that aborted signal delivery.
           t->signal_delivered(sig);
-          t->pop_event(EV_SIGNAL_DELIVERY);
+          t->restore_sigmask_if_saved();
+          t->pop_signal_delivery();
           step_state->continue_type = DONT_CONTINUE;
           last_task_switchable = PREVENT_SWITCH;
           break;
@@ -1099,23 +1198,66 @@ void RecordSession::signal_state_changed(RecordTask* t, StepState* step_state) {
       // are unmapped, write 0 bytes.
       t->record_remote_fallible(t->regs().sp(), sigframe_size);
 
-      // This event is used by the replayer to set up the
-      // signal handler frame, or to record the resulting
-      // state of the stepi if there wasn't a signal
-      // handler.
-      t->record_current_event();
+      // This event is used by the replayer to set up the signal handler frame.
+      // But if we don't have a handler, we don't want to record the event
+      // until we deal with the EV_SIGNAL_DELIVERY.
+      if (has_handler) {
+        t->record_current_event();
+      }
       break;
     }
 
     case EV_SIGNAL_DELIVERY: {
+      bool is_fatal = is_fatal_signal(t, sig, t->ev().Signal().deterministic);
+
+      // A fatal signal or SIGSTOP requires us to allow switching to another
+      // task.
+      Switchable can_switch =
+          (is_fatal || sig == SIGSTOP) ? ALLOW_SWITCH : PREVENT_SWITCH;
+
+      // We didn't record this event above, so do that now.
+      // NB: If there is no handler, and we interrupted a syscall, and there are
+      // no more actionable signals, the kernel sets us up for a syscall
+      // restart. But it does that *after* the ptrace trap. To replay this
+      // correctly we need to fake those changes here. But we don't do this
+      // if we're going to switch away at the ptrace trap, and for the moment,
+      // 'can_switch' is actually 'will_switch'.
+      // This is essentially copied from do_signal in arch/x86/kernel/signal.c
+      bool has_other_signals = t->has_any_actionable_signal();
+      auto r = t->regs();
+      if (can_switch == PREVENT_SWITCH && !has_other_signals &&
+          r.original_syscallno() >= 0 && r.syscall_may_restart()) {
+        switch (r.syscall_result_signed()) {
+          case -ERESTARTNOHAND:
+          case -ERESTARTSYS:
+          case -ERESTARTNOINTR:
+            r.set_syscallno(r.original_syscallno());
+            r.set_ip(r.ip().decrement_by_syscall_insn_length(t->arch()));
+            break;
+          case -ERESTART_RESTARTBLOCK:
+            r.set_syscallno(syscall_number_for_restart_syscall(t->arch()));
+            r.set_ip(r.ip().decrement_by_syscall_insn_length(t->arch()));
+            break;
+        }
+
+        // Now that we've mucked with the registers, we can't switch tasks. That
+        // could allow more signals to be generated, breaking our assumption
+        // that we are the last signal.
+      } else {
+        // But if we didn't touch the registers switching here is ok.
+        can_switch = ALLOW_SWITCH;
+      }
+
+      t->record_event(t->ev(), RecordTask::FLUSH_SYSCALLBUF, &r);
+      // Don't actually set_regs(r), the kernel does these modifications.
+
       // Only inject fatal signals. Non-fatal signals with signal handlers
       // were taken care of above; for non-fatal signals without signal
       // handlers, there is no need to deliver the signal at all. In fact,
       // there is really no way to inject a non-fatal, non-handled signal
       // without letting the task execute at least one instruction, which
       // we don't want to do here.
-      if (is_fatal_signal(t, sig, t->ev().Signal().deterministic) &&
-          sig != get_continue_through_sig()) {
+      if (is_fatal && sig != get_continue_through_sig()) {
         preinject_signal(t);
         t->resume_execution(RESUME_CONT, RESUME_NONBLOCKING, RESUME_NO_TICKS,
                             sig);
@@ -1123,11 +1265,16 @@ void RecordSession::signal_state_changed(RecordTask* t, StepState* step_state) {
                      "CLONE_CHILD_CLEARTID memory race";
         t->task_group()->destabilize();
       }
+
       t->signal_delivered(sig);
       t->pop_signal_delivery();
-      // A fatal signal or SIGSTOP requires us to allow switching to another
-      // task.
-      last_task_switchable = ALLOW_SWITCH;
+      // The mask set by ppoll/pselect6 is restored, but only if there are no
+      // more actionable signals. The ordering is important, because there may
+      // be pending signals that will be unblocked when we restore the sigmask.
+      if (!has_other_signals) {
+        t->restore_sigmask_if_saved();
+      }
+      last_task_switchable = can_switch;
       step_state->continue_type = DONT_CONTINUE;
       break;
     }
@@ -1254,7 +1401,7 @@ static bool interrupt_syscall_to_handle_signals(RecordTask* t) {
   t->set_regs(r);
   // This exits the hijacked SYS_gettid.  Now the tracee is
   // ready to do our bidding.
-  t->advance_syscall();
+  t->exit_syscall();
 
   // Set these regs to emulate a syscall interruption.
   r.set_original_syscallno(syscallno);
@@ -1287,11 +1434,55 @@ static void maybe_do_fake_syscall_restart(RecordTask* t) {
   t->set_regs(r);
 }
 
+void RecordSession::process_syscall_entry(RecordTask* t, StepState* step_state,
+                                          RecordResult* step_result) {
+  // We just entered a syscall.
+  if (!maybe_restart_syscall(t)) {
+    // Emit FLUSH_SYSCALLBUF if necessary before we do any patching work
+    t->maybe_flush_syscallbuf();
+
+    if (syscall_seccomp_ordering_ == PTRACE_SYSCALL_BEFORE_SECCOMP_UNKNOWN &&
+        t->seccomp_bpf_enabled) {
+      // We received a PTRACE_SYSCALL notification before the seccomp
+      // notification. Ignore it and continue to the seccomp notification.
+      syscall_seccomp_ordering_ = PTRACE_SYSCALL_BEFORE_SECCOMP;
+      step_state->continue_type = CONTINUE;
+      return;
+    }
+
+    if (t->vm()->monkeypatcher().try_patch_syscall(t)) {
+      // Syscall was patched. Emit event and continue execution.
+      t->record_event(Event(EV_PATCH_SYSCALL, NO_EXEC_INFO, t->arch()));
+      return;
+    }
+
+    t->push_event(SyscallEvent(t->regs().original_syscallno(), t->arch()));
+
+    if (interrupt_syscall_to_handle_signals(t)) {
+      return;
+    }
+  }
+
+  check_initial_task_syscalls(t, step_result);
+  note_entering_syscall(t);
+  if (t->emulated_ptrace_cont_command == PTRACE_SYSCALL ||
+      t->emulated_ptrace_cont_command == PTRACE_SYSEMU ||
+      t->emulated_ptrace_cont_command == PTRACE_SYSEMU_SINGLESTEP) {
+    t->ev().Syscall().state = ENTERING_SYSCALL_PTRACE;
+    t->emulate_ptrace_stop(WaitStatus::for_syscall(t));
+    t->record_current_event();
+
+    t->ev().Syscall().in_sysemu =
+        t->emulated_ptrace_cont_command == PTRACE_SYSEMU ||
+        t->emulated_ptrace_cont_command == PTRACE_SYSEMU_SINGLESTEP;
+  }
+}
+
 /**
  * The execution of |t| has just been resumed, and it most likely has
  * a new event that needs to be processed.  Prepare that new event.
  */
-void RecordSession::runnable_state_changed(RecordTask* t,
+void RecordSession::runnable_state_changed(RecordTask* t, StepState* step_state,
                                            RecordResult* step_result,
                                            bool can_consume_wait_status) {
   switch (t->ev().type()) {
@@ -1309,36 +1500,8 @@ void RecordSession::runnable_state_changed(RecordTask* t,
       if (!can_consume_wait_status) {
         return;
       }
-      // We just entered a syscall.
-      if (!maybe_restart_syscall(t)) {
-        // Emit FLUSH_SYSCALLBUF if necessary before we do any patching work
-        t->maybe_flush_syscallbuf();
 
-        if (t->vm()->monkeypatcher().try_patch_syscall(t)) {
-          // Syscall was patched. Emit event and continue execution.
-          t->record_event(Event(EV_PATCH_SYSCALL, NO_EXEC_INFO, t->arch()));
-          break;
-        }
-
-        t->push_event(SyscallEvent(t->regs().original_syscallno(), t->arch()));
-
-        if (interrupt_syscall_to_handle_signals(t)) {
-          break;
-        }
-      }
-      check_initial_task_syscalls(t, step_result);
-      note_entering_syscall(t);
-      if (t->emulated_ptrace_cont_command == PTRACE_SYSCALL ||
-          t->emulated_ptrace_cont_command == PTRACE_SYSEMU ||
-          t->emulated_ptrace_cont_command == PTRACE_SYSEMU_SINGLESTEP) {
-        t->ev().Syscall().state = ENTERING_SYSCALL_PTRACE;
-        t->emulate_ptrace_stop(WaitStatus::for_syscall(t));
-        t->record_current_event();
-
-        t->ev().Syscall().in_sysemu =
-            t->emulated_ptrace_cont_command == PTRACE_SYSEMU ||
-            t->emulated_ptrace_cont_command == PTRACE_SYSEMU_SINGLESTEP;
-      }
+      process_syscall_entry(t, step_state, step_result);
       break;
 
     default:
@@ -1456,7 +1619,9 @@ static string lookup_by_path(const string& name) {
       *next = 0;
     }
     string file = string(s) + "/" + name;
-    if (!access(file.c_str(), X_OK)) {
+    struct stat st;
+    if (!stat(file.c_str(), &st) && S_ISREG(st.st_mode) &&
+        !access(file.c_str(), X_OK)) {
       free(p);
       return file;
     }
@@ -1547,12 +1712,17 @@ static string lookup_by_path(const string& name) {
   }
 
   env.push_back("RUNNING_UNDER_RR=1");
-  env.push_back("LIBGL_DRI3_DISABLE=1");
+  env.push_back("LIBGL_ALWAYS_SOFTWARE=1");
 
   // Disable Gecko's "wait for gdb to attach on process crash" behavior, since
   // it is useless when running under rr.
   env.push_back("MOZ_GDB_SLEEP=0");
 
+  // OpenSSL uses RDRAND, but we can disable it. These bitmasks are inverted
+  // and ANDed with the results of CPUID. The number below is 2^62, which is the
+  // bit for RDRAND support.
+  env.push_back("OPENSSL_ia32cap=~4611686018427387904:~0");
+
   shr_ptr session(
       new RecordSession(full_path, argv, env, syscallbuf, bind_cpu));
   return session;
@@ -1648,9 +1818,20 @@ RecordSession::RecordResult RecordSession::record_step() {
 
   StepState step_state(CONTINUE);
 
-  if (!(rescheduled.by_waitpid && handle_ptrace_event(t, &step_state)) &&
-      !(rescheduled.by_waitpid && handle_signal_event(t, &step_state))) {
-    runnable_state_changed(t, &result, rescheduled.by_waitpid);
+  bool did_enter_syscall;
+  if (rescheduled.by_waitpid &&
+      handle_ptrace_event(t, &step_state, &result, &did_enter_syscall)) {
+    if (result.status != STEP_CONTINUE ||
+        step_state.continue_type == DONT_CONTINUE) {
+      return result;
+    }
+
+    if (did_enter_syscall && t->ev().type() == EV_SYSCALL) {
+      syscall_state_changed(t, &step_state);
+    }
+  } else if (rescheduled.by_waitpid && handle_signal_event(t, &step_state)) {
+  } else {
+    runnable_state_changed(t, &step_state, &result, rescheduled.by_waitpid);
 
     if (result.status != STEP_CONTINUE ||
         step_state.continue_type == DONT_CONTINUE) {
@@ -1673,6 +1854,8 @@ RecordSession::RecordResult RecordSession::record_step() {
     }
   }
 
+  t->verify_signal_states();
+
   // We try to inject a signal if there's one pending; otherwise we continue
   // task execution.
   if (!prepare_to_inject_signal(t, &step_state) &&
@@ -1686,6 +1869,7 @@ RecordSession::RecordResult RecordSession::record_step() {
     debug_exec_state("EXEC_START", t);
 
     maybe_do_fake_syscall_restart(t);
+    t->handling_deterministic_signal = 0;
     task_continue(step_state);
   }
 
@@ -1700,14 +1884,18 @@ void RecordSession::terminate_recording() {
 
   LOG(info) << "Processing termination request ...";
 
+  pid_t ttid = t ? t->tid : 0;
+  auto tticks = t ? t->tick_count() : 0;
+
   // This will write unstable exit events for all tasks.
   kill_all_tasks();
+  t = nullptr; // t is now deallocated
 
   LOG(info) << "  recording final TRACE_TERMINATION event ...";
 
-  TraceFrame frame(trace_out.time(), t ? t->tid : 0,
+  TraceFrame frame(trace_out.time(), ttid,
                    Event(EV_TRACE_TERMINATION, NO_EXEC_INFO, RR_NATIVE_ARCH),
-                   t ? t->tick_count() : 0);
+                   tticks);
   trace_out.write_frame(frame);
   trace_out.close();
 }
