@@ -23,13 +23,19 @@ using namespace std;
 
 namespace rr {
 
+enum SignalDisposition { SIGNAL_DEFAULT, SIGNAL_IGNORE, SIGNAL_HANDLER };
+
 /**
  * Stores the table of signal dispositions and metadata for an
  * arbitrary set of tasks.  Each of those tasks must own one one of
  * the |refcount|s while they still refer to this.
  */
 struct Sighandler {
-  Sighandler() : resethand(false), takes_siginfo(false) {}
+  Sighandler()
+      : resethand(false),
+        takes_siginfo(false),
+        nodefer_set(false),
+        sa_mask(0) {}
 
   template <typename Arch>
   void init_arch(const typename Arch::kernel_sigaction& ksa) {
@@ -38,6 +44,10 @@ struct Sighandler {
     memcpy(sa.data(), &ksa, sizeof(ksa));
     resethand = (ksa.sa_flags & SA_RESETHAND) != 0;
     takes_siginfo = (ksa.sa_flags & SA_SIGINFO) != 0;
+    nodefer_set = (ksa.sa_flags & SA_NODEFER) != 0;
+    static_assert(sizeof(ksa.sa_mask) == sizeof(sa_mask),
+                  "Kernel mask size grew?");
+    memcpy(&sa_mask, &ksa.sa_mask, sizeof(ksa.sa_mask));
   }
 
   template <typename Arch> void reset_arch() {
@@ -47,25 +57,23 @@ struct Sighandler {
     init_arch<Arch>(ksa);
   }
 
-  bool ignored(int sig) const {
-    if (sig == SIGSTOP || sig == SIGKILL) {
-      // These can never be ignored
-      return false;
+  SignalDisposition disposition() const {
+    assert(uintptr_t(SIG_DFL) == 0);
+    assert(uintptr_t(SIG_IGN) == 1);
+    switch (k_sa_handler.as_int()) {
+      case 0:
+        return SIGNAL_DEFAULT;
+      case 1:
+        return SIGNAL_IGNORE;
+      default:
+        return SIGNAL_HANDLER;
     }
-    return (uintptr_t)SIG_IGN == k_sa_handler.as_int() ||
-           ((uintptr_t)SIG_DFL == k_sa_handler.as_int() &&
-            IGNORE == default_action(sig));
-  }
-  bool is_default() const {
-    return (uintptr_t)SIG_DFL == k_sa_handler.as_int() && !resethand;
-  }
-  bool is_user_handler() const {
-    assert(1 == uintptr_t(SIG_IGN));
-    return k_sa_handler.as_int() & ~(uintptr_t)SIG_IGN;
   }
+
   remote_code_ptr get_user_handler() const {
-    return is_user_handler() ? remote_code_ptr(k_sa_handler.as_int())
-                             : remote_code_ptr();
+    return disposition() == SIGNAL_HANDLER
+               ? remote_code_ptr(k_sa_handler.as_int())
+               : remote_code_ptr();
   }
 
   remote_ptr<void> k_sa_handler;
@@ -73,6 +81,8 @@ struct Sighandler {
   vector<uint8_t> sa;
   bool resethand;
   bool takes_siginfo;
+  bool nodefer_set;
+  uint64_t sa_mask;
 };
 
 static void reset_handler(Sighandler* handler, SupportedArch arch) {
@@ -103,17 +113,17 @@ struct Sighandlers {
   }
 
   void init_from_current_process() {
-    for (size_t i = 0; i < array_length(handlers); ++i) {
+    for (size_t i = 1; i < array_length(handlers); ++i) {
       Sighandler& h = handlers[i];
 
       NativeArch::kernel_sigaction sa;
-      if (::syscall(SYS_rt_sigaction, i, nullptr, &sa, sizeof(sigset_t))) {
+      if (::syscall(SYS_rt_sigaction, i, nullptr, &sa, sizeof(uint64_t))) {
         /* EINVAL means we're querying an
          * unused signal number. */
         assert(EINVAL == errno);
-        assert(h.is_default());
         continue;
       }
+      msan_unpoison(&sa, sizeof(NativeArch::kernel_sigaction));
 
       h.init_arch<NativeArch>(sa);
     }
@@ -135,7 +145,7 @@ struct Sighandlers {
       // If the handler was a user handler, reset to
       // default.  If it was SIG_IGN or SIG_DFL,
       // leave it alone.
-      if (h.is_user_handler()) {
+      if (h.disposition() == SIGNAL_HANDLER) {
         reset_handler(&h, arch);
       }
     }
@@ -174,10 +184,12 @@ RecordTask::RecordTask(RecordSession& session, pid_t _tid, uint32_t serial,
       in_wait_pid(0),
       emulated_stop_type(NOT_STOPPED),
       blocked_sigs(),
+      previously_blocked_sigs(),
+      has_previously_blocked_sigs(false),
+      handling_deterministic_signal(0),
       flushed_num_rec_bytes(0),
       flushed_syscallbuf(false),
       delay_syscallbuf_reset(false),
-      seccomp_bpf_enabled(false),
       prctl_seccomp_status(0),
       robust_futex_list_len(0),
       own_namespace_rec_tid(0),
@@ -216,6 +228,7 @@ RecordTask::~RecordTask() {
     ASSERT(this, t->emulated_ptracer == this);
     t->emulated_ptracer = nullptr;
     t->emulated_ptrace_options = 0;
+    t->emulated_stop_pending = false;
     t->emulated_stop_type = NOT_STOPPED;
   }
 
@@ -247,7 +260,14 @@ RecordTask::~RecordTask() {
 
   // Write the exit event here so that the value recorded above is captured.
   EventType e = unstable ? EV_UNSTABLE_EXIT : EV_EXIT;
-  record_event(Event(e, NO_EXEC_INFO, arch()));
+  // Don't flush syscallbuf. Whatever triggered the exit (syscall, signal)
+  // should already have flushed it, if it was running. If it was blocked,
+  // then the syscallbuf would already have been flushed too. The exception
+  // is kill_all_tasks() in which case it's OK to just drop the last chunk of
+  // execution. Trying to flush syscallbuf for an exiting task could be bad,
+  // e.g. it could be in the middle of syscallbuf code that's supposed to be
+  // atomic.
+  record_event(Event(e, NO_EXEC_INFO, arch()), DONT_FLUSH_SYSCALLBUF);
 
   // We expect tasks to usually exit by a call to exit() or
   // exit_group(), so it's not helpful to warn about that.
@@ -297,7 +317,7 @@ Task* RecordTask::clone(int flags, remote_ptr<void> stack, remote_ptr<void> tls,
   if (t->session().is_recording()) {
     RecordTask* rt = static_cast<RecordTask*>(t);
     rt->priority = priority;
-    rt->blocked_sigs = blocked_sigs;
+    rt->blocked_sigs = get_sigmask();
     rt->prctl_seccomp_status = prctl_seccomp_status;
     rt->robust_futex_list = robust_futex_list;
     rt->robust_futex_list_len = robust_futex_list_len;
@@ -369,7 +389,7 @@ void RecordTask::post_exec() {
 
 template <typename Arch> static void do_preload_init_arch(RecordTask* t) {
   auto params = t->read_mem(
-      remote_ptr<rrcall_init_preload_params<Arch> >(t->regs().arg1()));
+      remote_ptr<rrcall_init_preload_params<Arch>>(t->regs().arg1()));
 
   int cores = t->session().scheduler().pretend_num_cores();
   auto cores_ptr = REMOTE_PTR_FIELD(params.globals.rptr(), pretend_num_cores);
@@ -409,13 +429,14 @@ template <typename Arch> void RecordTask::init_buffers_arch() {
   AutoRemoteSyscalls remote(this);
 
   // Arguments to the rrcall.
-  remote_ptr<rrcall_init_buffers_params<Arch> > child_args = regs().arg1();
+  remote_ptr<rrcall_init_buffers_params<Arch>> child_args = regs().arg1();
   auto args = read_mem(child_args);
 
   args.cloned_file_data_fd = -1;
   if (as->syscallbuf_enabled()) {
     args.syscallbuf_size = syscallbuf_size = session().syscall_buffer_size();
     KernelMapping syscallbuf_km = init_syscall_buffer(remote, nullptr);
+    set_sigmask(blocked_sigs);
     args.syscallbuf_ptr = syscallbuf_child;
     desched_fd_child = args.desched_counter_fd;
     // Prevent the child from closing this fd
@@ -496,6 +517,20 @@ void RecordTask::on_syscall_exit_arch(int syscallno, const Registers& regs) {
     case Arch::set_tid_address:
       set_tid_addr(regs.arg1());
       return;
+    case Arch::ppoll: {
+      remote_ptr<sig_set_t> setp = regs.arg4();
+      if (!setp.is_null()) {
+        restore_sigmask();
+      }
+      return;
+    }
+    case Arch::pselect6: {
+      remote_ptr<sig_set_t> setp = regs.arg6();
+      if (!setp.is_null()) {
+        restore_sigmask();
+      }
+      return;
+    }
   }
 }
 
@@ -648,9 +683,16 @@ void RecordTask::send_synthetic_SIGCHLD_if_necessary() {
   }
 }
 
-void RecordTask::set_siginfo_for_synthetic_SIGCHLD(siginfo_t* si) {
+bool RecordTask::set_siginfo_for_synthetic_SIGCHLD(siginfo_t* si) {
   if (si->si_signo != SIGCHLD || si->si_value.sival_int != SIGCHLD_SYNTHETIC) {
-    return;
+    return true;
+  }
+
+  if (is_syscall_restart()) {
+    // ptrace generated signals don't interrupt syscalls such as wait.
+    // Return false to tell the caller to defer the signal and resume
+    // the syscall.
+    return false;
   }
 
   for (RecordTask* tracee : emulated_ptrace_tracees) {
@@ -659,7 +701,7 @@ void RecordTask::set_siginfo_for_synthetic_SIGCHLD(siginfo_t* si) {
       tracee->set_siginfo_for_waited_task<NativeArch>(
           reinterpret_cast<NativeArch::siginfo_t*>(si));
       si->si_value.sival_int = 0;
-      return;
+      return true;
     }
   }
 
@@ -671,10 +713,12 @@ void RecordTask::set_siginfo_for_synthetic_SIGCHLD(siginfo_t* si) {
         rchild->set_siginfo_for_waited_task<NativeArch>(
             reinterpret_cast<NativeArch::siginfo_t*>(si));
         si->si_value.sival_int = 0;
-        return;
+        return true;
       }
     }
   }
+
+  return true;
 }
 
 bool RecordTask::is_waiting_for_ptrace(RecordTask* t) {
@@ -805,28 +849,43 @@ bool RecordTask::is_signal_pending(int sig) {
   return !*end1 && !*end2 && ((mask1 | mask2) & (1 << (sig - 1)));
 }
 
+bool RecordTask::has_any_actionable_signal() {
+  auto sig_strs = read_proc_status_fields(tid, "SigPnd", "ShdPnd", "SigBlk");
+  if (sig_strs.size() < 3) {
+    return false;
+  }
+
+  char* end1;
+  uint64_t mask1 = strtoull(sig_strs[0].c_str(), &end1, 16);
+  char* end2;
+  uint64_t mask2 = strtoull(sig_strs[1].c_str(), &end2, 16);
+  char* end3;
+  uint64_t mask_blk = strtoull(sig_strs[2].c_str(), &end3, 16);
+  return !*end1 && !*end2 && !*end3 && ((mask1 | mask2) & ~mask_blk);
+}
+
 void RecordTask::emulate_SIGCONT() {
   // All threads in the process are resumed.
   for (Task* t : task_group()->task_set()) {
     auto rt = static_cast<RecordTask*>(t);
     LOG(debug) << "setting " << tid << " to NOT_STOPPED due to SIGCONT";
+    rt->emulated_stop_pending = false;
     rt->emulated_stop_type = NOT_STOPPED;
   }
 }
 
 void RecordTask::signal_delivered(int sig) {
   Sighandler& h = sighandlers->get(sig);
-  bool is_user_handler = h.is_user_handler();
   if (h.resethand) {
     reset_handler(&h, arch());
   }
 
-  if (!h.ignored(sig)) {
+  if (!is_sig_ignored(sig)) {
     switch (sig) {
       case SIGTSTP:
       case SIGTTIN:
       case SIGTTOU:
-        if (is_user_handler) {
+        if (h.disposition() == SIGNAL_HANDLER) {
           break;
         }
       // Fall through...
@@ -847,7 +906,7 @@ void RecordTask::signal_delivered(int sig) {
 }
 
 bool RecordTask::signal_has_user_handler(int sig) const {
-  return sighandlers->get(sig).is_user_handler();
+  return sighandlers->get(sig).disposition() == SIGNAL_HANDLER;
 }
 
 remote_code_ptr RecordTask::get_signal_user_handler(int sig) const {
@@ -862,21 +921,56 @@ bool RecordTask::signal_handler_takes_siginfo(int sig) const {
   return sighandlers->get(sig).takes_siginfo;
 }
 
-bool RecordTask::is_sig_blocked(int sig) const {
+bool RecordTask::signal_handler_nodefer(int sig) const {
+  return sighandlers->get(sig).nodefer_set;
+}
+
+static bool is_unstoppable_signal(int sig) {
+  return sig == SIGSTOP || sig == SIGKILL;
+}
+
+bool RecordTask::is_sig_blocked(int sig) {
+  if (is_unstoppable_signal(sig)) {
+    // These can never be blocked
+    return false;
+  }
   int sig_bit = sig - 1;
   if (sigsuspend_blocked_sigs) {
     return (*sigsuspend_blocked_sigs >> sig_bit) & 1;
   }
-  return (blocked_sigs >> sig_bit) & 1;
+  return (get_sigmask() >> sig_bit) & 1;
 }
 
-void RecordTask::set_sig_blocked(int sig) {
-  int sig_bit = sig - 1;
-  blocked_sigs |= (sig_set_t)1 << sig_bit;
+void RecordTask::set_sig_blocked(int sig, bool blocked) {
+  uint64_t mask = uint64_t(1) << (sig - 1);
+  sig_set_t sigs = get_sigmask();
+  if (blocked) {
+    sigs |= mask;
+  } else {
+    sigs &= ~mask;
+  }
+  set_sigmask(sigs);
+}
+
+void RecordTask::apply_sig_sa_mask(int sig) {
+  sig_set_t sigs = get_sigmask();
+  sigs |= sighandlers->get(sig).sa_mask;
+  set_sigmask(sigs);
 }
 
 bool RecordTask::is_sig_ignored(int sig) const {
-  return sighandlers->get(sig).ignored(sig);
+  if (is_unstoppable_signal(sig)) {
+    // These can never be ignored
+    return false;
+  }
+  switch (sighandlers->get(sig).disposition()) {
+    case SIGNAL_IGNORE:
+      return true;
+    case SIGNAL_DEFAULT:
+      return IGNORE == default_action(sig);
+    default:
+      return false;
+  }
 }
 
 void RecordTask::set_siginfo(const siginfo_t& si) {
@@ -894,11 +988,8 @@ void RecordTask::update_sigaction_arch(const Registers& regs) {
     // TODO: discard attempts to handle or ignore signals
     // that can't be by POSIX
     typename Arch::kernel_sigaction sa;
-    size_t sigset_size = min(sizeof(typename Arch::sigset_t), regs.arg4());
     memset(&sa, 0, sizeof(sa));
-    read_bytes_helper(
-        new_sigaction,
-        sizeof(sa) - (sizeof(typename Arch::sigset_t) - sigset_size), &sa);
+    read_bytes_helper(new_sigaction, sizeof(sa), &sa);
     sighandlers->get(sig).init_arch<Arch>(sa);
   }
 }
@@ -907,6 +998,10 @@ void RecordTask::update_sigaction(const Registers& regs) {
   RR_ARCH_FUNCTION(update_sigaction_arch, regs.arch(), regs);
 }
 
+void RecordTask::set_new_sigmask(uint64_t new_sigmask) {
+  set_sigmask(new_sigmask);
+}
+
 void RecordTask::update_sigmask(const Registers& regs) {
   int how = regs.arg1_signed();
   remote_ptr<sig_set_t> setp = regs.arg2();
@@ -916,21 +1011,106 @@ void RecordTask::update_sigmask(const Registers& regs) {
   }
 
   auto set = read_mem(setp);
+  sig_set_t sigs = get_sigmask();
 
   // Update the blocked signals per |how|.
   switch (how) {
     case SIG_BLOCK:
-      blocked_sigs |= set;
+      sigs |= set;
       break;
     case SIG_UNBLOCK:
-      blocked_sigs &= ~set;
+      sigs &= ~set;
       break;
     case SIG_SETMASK:
-      blocked_sigs = set;
+      sigs = set;
       break;
     default:
       FATAL() << "Unknown sigmask manipulator " << how;
   }
+
+  // Our signals are never blocked.
+  uint64_t enabled_sigs = (1 << (PerfCounters::TIME_SLICE_SIGNAL - 1)) |
+                          (1 << (SYSCALLBUF_DESCHED_SIGNAL - 1));
+  sigs &= ~enabled_sigs;
+  set_sigmask(sigs);
+}
+
+sig_set_t RecordTask::get_sigmask() {
+  if (syscallbuf_child) {
+    blocked_sigs = read_mem(REMOTE_PTR_FIELD(syscallbuf_child, blocked_sigs));
+  }
+  return blocked_sigs;
+}
+
+void RecordTask::set_sigmask(sig_set_t sigs) {
+  blocked_sigs = sigs;
+  if (syscallbuf_child) {
+    write_mem(REMOTE_PTR_FIELD(syscallbuf_child, blocked_sigs), blocked_sigs);
+  }
+}
+
+void RecordTask::save_sigmask() {
+  ASSERT(this, !has_previously_blocked_sigs);
+  previously_blocked_sigs = get_sigmask();
+  has_previously_blocked_sigs = true;
+}
+
+void RecordTask::restore_sigmask() {
+  ASSERT(this, has_previously_blocked_sigs);
+  set_sigmask(previously_blocked_sigs);
+  clear_saved_sigmask();
+}
+
+void RecordTask::restore_sigmask_if_saved() {
+  if (has_previously_blocked_sigs) {
+    restore_sigmask();
+  }
+}
+
+void RecordTask::clear_saved_sigmask() { has_previously_blocked_sigs = false; }
+
+void RecordTask::set_sig_handler_default(int sig) {
+  Sighandler& h = sighandlers->get(sig);
+  reset_handler(&h, arch());
+}
+
+void RecordTask::verify_signal_states() {
+#ifndef DEBUG
+  return;
+#endif
+  if (ev().is_syscall_event()) {
+    // If the syscall event is on the event stack with PROCESSING or EXITING
+    // states, we won't have applied the signal-state updates yet while the
+    // kernel may have.
+    return;
+  }
+  auto results = read_proc_status_fields(tid, "SigBlk", "SigIgn", "SigCgt");
+  ASSERT(this, results.size() == 3);
+  uint64_t blocked = strtoull(results[0].c_str(), NULL, 16);
+  uint64_t ignored = strtoull(results[1].c_str(), NULL, 16);
+  uint64_t caught = strtoull(results[2].c_str(), NULL, 16);
+  for (int sig = 1; sig < _NSIG; ++sig) {
+    uint64_t mask = uint64_t(1) << (sig - 1);
+    if (is_unstoppable_signal(sig)) {
+      ASSERT(this, !(blocked & mask)) << "Expected " << signal_name(sig)
+                                      << " to not be blocked, but it is";
+      ASSERT(this, !(ignored & mask)) << "Expected " << signal_name(sig)
+                                      << " to not be ignored, but it is";
+      ASSERT(this, !(caught & mask)) << "Expected " << signal_name(sig)
+                                     << " to not be caught, but it is";
+    } else if (sig != handling_deterministic_signal) {
+      ASSERT(this, !!(blocked & mask) == is_sig_blocked(sig))
+          << signal_name(sig)
+          << ((blocked & mask) ? " is blocked" : " is not blocked");
+      auto disposition = sighandlers->get(sig).disposition();
+      ASSERT(this, !!(ignored & mask) == (disposition == SIGNAL_IGNORE))
+          << signal_name(sig)
+          << ((ignored & mask) ? " is ignored" : " is not ignored");
+      ASSERT(this, !!(caught & mask) == (disposition == SIGNAL_HANDLER))
+          << signal_name(sig)
+          << ((caught & mask) ? " is caught" : " is not caught");
+    }
+  }
 }
 
 void RecordTask::stash_sig() {
@@ -1115,7 +1295,7 @@ void RecordTask::record_local(remote_ptr<void> addr, ssize_t num_bytes,
     return;
   }
 
-  trace_writer().write_raw(data, num_bytes, addr);
+  trace_writer().write_raw(rec_tid, data, num_bytes, addr);
 }
 
 bool RecordTask::record_remote_by_local_map(remote_ptr<void> addr,
@@ -1130,7 +1310,7 @@ bool RecordTask::record_remote_by_local_map(remote_ptr<void> addr,
 void RecordTask::record_remote(remote_ptr<void> addr, ssize_t num_bytes) {
   maybe_flush_syscallbuf();
 
-  assert(num_bytes >= 0);
+  ASSERT(this, num_bytes >= 0);
 
   if (!addr) {
     return;
@@ -1140,7 +1320,7 @@ void RecordTask::record_remote(remote_ptr<void> addr, ssize_t num_bytes) {
     return;
 
   auto buf = read_mem(addr.cast<uint8_t>(), num_bytes);
-  trace_writer().write_raw(buf.data(), num_bytes, addr);
+  trace_writer().write_raw(rec_tid, buf.data(), num_bytes, addr);
 }
 
 void RecordTask::record_remote_fallible(remote_ptr<void> addr,
@@ -1158,7 +1338,7 @@ void RecordTask::record_remote_fallible(remote_ptr<void> addr,
     ssize_t nread = read_bytes_fallible(addr, num_bytes, buf.data());
     buf.resize(max<ssize_t>(0, nread));
   }
-  trace_writer().write_raw(buf.data(), buf.size(), addr);
+  trace_writer().write_raw(rec_tid, buf.data(), buf.size(), addr);
 }
 
 void RecordTask::record_remote_even_if_null(remote_ptr<void> addr,
@@ -1168,7 +1348,7 @@ void RecordTask::record_remote_even_if_null(remote_ptr<void> addr,
   assert(num_bytes >= 0);
 
   if (!addr) {
-    trace_writer().write_raw(nullptr, 0, addr);
+    trace_writer().write_raw(rec_tid, nullptr, 0, addr);
     return;
   }
 
@@ -1176,7 +1356,7 @@ void RecordTask::record_remote_even_if_null(remote_ptr<void> addr,
     return;
 
   auto buf = read_mem(addr.cast<uint8_t>(), num_bytes);
-  trace_writer().write_raw(buf.data(), num_bytes, addr);
+  trace_writer().write_raw(rec_tid, buf.data(), num_bytes, addr);
 }
 
 void RecordTask::pop_event(EventType expected_type) {
@@ -1230,8 +1410,12 @@ void RecordTask::maybe_flush_syscallbuf() {
         read_mem(REMOTE_PTR_FIELD(preload_globals, mprotect_records[0]),
                  hdr.mprotect_record_count);
     for (auto& r : records) {
-      as->protect(r.start, r.size, r.prot);
+      as->protect(this, r.start, r.size, r.prot);
     }
+    // We write these out because some tools might benefit from them, and
+    // this is cheap.
+    trace_writer().write_generic(records.data(),
+                                 records.size() * sizeof(records[0]));
   }
 
   // Write the entire buffer in one shot without parsing it,
@@ -1316,7 +1500,8 @@ void RecordTask::record_event(const Event& ev, FlushSyscallbuf flush,
   trace_writer().write_frame(frame);
 
   if (!ev.has_ticks_slop()) {
-    ASSERT(this, flush == FLUSH_SYSCALLBUF);
+    ASSERT(this, flush == FLUSH_SYSCALLBUF || ev.type() == EV_UNSTABLE_EXIT ||
+                     ev.type() == EV_EXIT);
     // After we've output an event, it's safe to reset the syscallbuf (if not
     // explicitly delayed) since we will have exited the syscallbuf code that
     // consumed the syscallbuf data.
@@ -1346,12 +1531,10 @@ pid_t RecordTask::find_newborn_thread() {
   DIR* dir = opendir(path);
   ASSERT(this, dir);
   while (true) {
-    struct dirent* result;
-    struct dirent entry;
-    int ret = readdir_r(dir, &entry, &result);
-    ASSERT(this, !ret && result == &entry);
+    struct dirent* result = readdir(dir);
+    ASSERT(this, result);
     char* end;
-    pid_t thread_tid = strtol(entry.d_name, &end, 10);
+    pid_t thread_tid = strtol(result->d_name, &end, 10);
     if (*end == '\0' && !session().find_task(thread_tid)) {
       closedir(dir);
       return thread_tid;
@@ -1375,12 +1558,10 @@ pid_t RecordTask::find_newborn_child_process() {
   DIR* dir = opendir("/proc");
   ASSERT(this, dir);
   while (true) {
-    struct dirent* result;
-    struct dirent entry;
-    int ret = readdir_r(dir, &entry, &result);
-    ASSERT(this, !ret && result == &entry);
+    struct dirent* result = readdir(dir);
+    ASSERT(this, result);
     char* end;
-    pid_t proc_tid = strtol(entry.d_name, &end, 10);
+    pid_t proc_tid = strtol(result->d_name, &end, 10);
     if (*end == '\0' && !session().find_task(proc_tid) &&
         get_ppid(proc_tid) == real_tgid()) {
       closedir(dir);
@@ -1400,4 +1581,10 @@ void RecordTask::tgkill(int sig) {
 
 pid_t RecordTask::get_parent_pid() { return get_ppid(tid); }
 
+void RecordTask::set_tid_and_update_serial(pid_t tid) {
+  hpc.set_tid(tid);
+  this->tid = rec_tid = tid;
+  serial = session().next_task_serial();
+}
+
 } // namespace rr
