@@ -97,8 +97,9 @@ public:
    * ptraced task has had its SIGCHLD sent.
    * Note that we can't set the correct siginfo when we send the signal, because
    * it requires us to set information only the kernel has permission to set.
+   * Returns false if this signal should be deferred.
    */
-  void set_siginfo_for_synthetic_SIGCHLD(siginfo_t* si);
+  bool set_siginfo_for_synthetic_SIGCHLD(siginfo_t* si);
   /**
    * Sets up |si| as if we're delivering a SIGCHLD/waitid for this waited task.
    */
@@ -152,6 +153,10 @@ public:
    * Return true if |sig| is pending but hasn't been reported to ptrace yet
    */
   bool is_signal_pending(int sig);
+  /**
+   * Return true if there are any signals pending that are not blocked.
+   */
+  bool has_any_actionable_signal();
   /**
    * Get all threads out of an emulated GROUP_STOP
    */
@@ -172,16 +177,20 @@ public:
    * parameter.
    */
   bool signal_handler_takes_siginfo(int sig) const;
+  /**
+   * Return true if SA_NODEFER is set for |sig|
+   */
+  bool signal_handler_nodefer(int sig) const;
   /**
    * Return |sig|'s current sigaction. Returned as raw bytes since the
    * data is architecture-dependent.
    */
   const std::vector<uint8_t>& signal_action(int sig) const;
   /** Return true iff |sig| is blocked for this. */
-  bool is_sig_blocked(int sig) const;
+  bool is_sig_blocked(int sig);
 
-  /** Set |sig| to be treated as blocked. */
-  void set_sig_blocked(int sig);
+  /** Set |sig| to be treated as |blocked|. */
+  void set_sig_blocked(int sig, bool blocked);
   /**
    * Return true iff |sig| is SIG_IGN, or it's SIG_DFL and the
    * default disposition is "ignore".
@@ -191,6 +200,24 @@ public:
    * Set the siginfo for the signal-stop of this.
    */
   void set_siginfo(const siginfo_t& si);
+  /**
+   * Update this task's sigmask to be new_sigmask
+   */
+  void set_new_sigmask(uint64_t new_sigmask);
+  /**
+   * Update this task's sigmask to block all signals specified in sa_mask of
+   * |sig|'s current sigaction
+   */
+  void apply_sig_sa_mask(int sig);
+  /**
+   * Reset the signal handler for this signal to the default.
+   */
+  void set_sig_handler_default(int sig);
+
+  /**
+   * Check that our status for |sig| matches what's in /proc/<pid>/status.
+   */
+  void verify_signal_states();
 
   /**
    * Stashed-signal API: if a signal becomes pending at an
@@ -312,6 +339,9 @@ public:
   template <typename T> void record_remote(remote_ptr<T> addr) {
     record_remote(addr, sizeof(T));
   }
+  void record_remote(const MemoryRange& range) {
+    record_remote(range.start(), range.size());
+  }
   // Record as much as we can of the bytes in this range.
   void record_remote_fallible(remote_ptr<void> addr, ssize_t num_bytes);
 
@@ -426,6 +456,23 @@ public:
 
   enum { SYNTHETIC_TIME_SLICE_SI_CODE = -9999 };
 
+  /**
+   * Tasks normally can't change their tid. There is one very special situation
+   * where they can: when a non-main-thread does an execve, its tid changes
+   * to the tid of the thread-group leader.
+   */
+  void set_tid_and_update_serial(pid_t tid);
+
+  /**
+   * Save or restore the current sigmask (reloading it if necessary) to handle
+   * ppoll/pselect. If the sigmask is restored another way (e.g. because a
+   * signal handler was invoked) clear_saved_sigmask must be called.
+   */
+  void save_sigmask();
+  void restore_sigmask();
+  void restore_sigmask_if_saved();
+  void clear_saved_sigmask();
+
 private:
   ~RecordTask();
 
@@ -448,6 +495,14 @@ private:
    */
   void send_synthetic_SIGCHLD_if_necessary();
 
+  /**
+   * Reload, resp. write back the current mask of blocked signals to the shadow
+   * copy in this tasks's preload globals such that we can effectively track,
+   * changes made to the mask by buffered syscalls.
+   */
+  sig_set_t get_sigmask();
+  void set_sigmask(sig_set_t mask);
+
   /**
    * Call this when SYS_sigaction is finishing with |regs|.
    */
@@ -538,7 +593,12 @@ public:
   // If not NOT_STOPPED, then the task is logically stopped and this is the type
   // of stop.
   EmulatedStopType emulated_stop_type;
+  // Most accesses to this should use set_sigmask and get_sigmask to ensure
+  // the mirroring to syscallbuf is correct.
   sig_set_t blocked_sigs;
+  sig_set_t previously_blocked_sigs;
+  bool has_previously_blocked_sigs;
+  int handling_deterministic_signal;
 
   // Syscallbuf state
 
@@ -555,16 +615,6 @@ public:
    * record buffer from being reset when it normally would be.
    * Currently, the desched'd syscall code uses this. */
   bool delay_syscallbuf_reset;
-  /* True when the tracee has started using the syscallbuf, and
-   * the tracer will start receiving PTRACE_SECCOMP events for
-   * traced syscalls.  We don't make any attempt to guess at the
-   * OS's process/thread semantics; this flag goes on the first
-   * time rr sees a PTRACE_SECCOMP event from the task.
-   *
-   * NB: there must always be at least one traced syscall before
-   * any untraced ones; that's the magic "rrcall" the tracee
-   * uses to initialize its syscallbuf. */
-  bool seccomp_bpf_enabled;
   // Value to return from PR_GET_SECCOMP
   uint8_t prctl_seccomp_status;
 
