@@ -4,10 +4,12 @@
 #define RR_SCOPED_FD_H_
 
 #include <fcntl.h>
-#include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/types.h>
 #include <unistd.h>
 
+namespace rr {
+
 /**
  * RAII helper to open a file and then close the fd when the helper
  * goes out of scope.
@@ -42,4 +44,6 @@ private:
   int fd;
 };
 
+} // namespace rr
+
 #endif // RR_SCOPED_FD_H
