@@ -30,13 +30,14 @@ struct Session::CloneCompletion {
     Task* clone_leader;
     Task::CapturedState clone_leader_state;
     vector<Task::CapturedState> member_states;
-    vector<pair<remote_ptr<void>, vector<uint8_t> > > captured_memory;
+    vector<pair<remote_ptr<void>, vector<uint8_t>>> captured_memory;
   };
   vector<TaskGroup> task_groups;
 };
 
 Session::Session()
     : next_task_serial_(1),
+      syscall_seccomp_ordering_(PTRACE_SYSCALL_BEFORE_SECCOMP_UNKNOWN),
       done_initial_exec_(false),
       visible_execution_(true) {
   LOG(debug) << "Session " << this << " created";
@@ -373,15 +374,6 @@ void Session::finish_initializing() const {
     for (auto& mem : tgleader.captured_memory) {
       tgleader.clone_leader->write_bytes_helper(mem.first, mem.second.size(),
                                                 mem.second.data());
-      auto& m = tgleader.clone_leader->vm()->mapping_of(mem.first);
-      if (m.flags & AddressSpace::Mapping::IS_SYSCALLBUF) {
-        // Bytes after the used part of the syscallbuf must be zero.
-        remote_ptr<void> bytes_end = mem.first + mem.second.size();
-        ASSERT(tgleader.clone_leader, m.map.start() == mem.first);
-        ASSERT(tgleader.clone_leader, bytes_end <= m.map.end());
-        ASSERT(tgleader.clone_leader, m.local_addr);
-        memset(m.local_addr + mem.second.size(), 0, m.map.end() - bytes_end);
-      }
     }
     for (auto& tgmember : tgleader.member_states) {
       Task* t_clone =
@@ -442,10 +434,10 @@ static void remap_shared_mmap(AutoRemoteSyscalls& remote, EmuFs& emu_fs,
 
   // We update the AddressSpace mapping too, since that tracks the real file
   // name and we need to update that.
-  remote.task()->vm()->map(m.map.start(), m.map.size(), m.map.prot(),
-                           m.map.flags(), m.map.file_offset_bytes(),
-                           real_file_name, real_file.st_dev, real_file.st_ino,
-                           &m.recorded_map, emu_file);
+  remote.task()->vm()->map(
+      remote.task(), m.map.start(), m.map.size(), m.map.prot(), m.map.flags(),
+      m.map.file_offset_bytes(), real_file_name, real_file.st_dev,
+      real_file.st_ino, nullptr, &m.recorded_map, emu_file);
 
   remote.infallible_syscall(syscall_number_for_close(remote.arch()), remote_fd);
 }
@@ -498,8 +490,9 @@ KernelMapping Session::create_shared_mmap(
   struct stat st;
   ASSERT(remote.task(), 0 == ::fstat(shmem_fd, &st));
   KernelMapping km = remote.task()->vm()->map(
-      child_map_addr, size, tracee_prot, flags | tracee_flags, 0, path,
-      st.st_dev, st.st_ino, nullptr, nullptr, map_addr, std::move(monitored));
+      remote.task(), child_map_addr, size, tracee_prot, flags | tracee_flags, 0,
+      path, st.st_dev, st.st_ino, nullptr, nullptr, nullptr, map_addr,
+      std::move(monitored));
 
   shmem_fd.close();
   remote.infallible_syscall(syscall_number_for_close(remote.arch()),
@@ -512,9 +505,15 @@ static char* extract_name(char* name_buffer, size_t buffer_size) {
   // name between RR_MAPPING_PREFIX and the -%d-%d at the end.
   char* name_end = name_buffer + strnlen(name_buffer, buffer_size);
   char* name_start = name_buffer + strlen(RR_MAPPING_PREFIX);
-  for (int i = 0; i < 2; ++i) {
-    while (name_end > name_start + (1 - i) && *(name_end--) != '-')
-      ;
+  int hyphens_seen = 0;
+  while (name_end > name_start) {
+    --name_end;
+    if (*name_end == '-') {
+      ++hyphens_seen;
+    }
+    if (hyphens_seen == 2) {
+      break;
+    }
   }
   *name_end = '\0';
   return name_start;
@@ -551,7 +550,7 @@ bool Session::make_private_shared(AutoRemoteSyscalls& remote,
   remote_ptr<void> free_mem = remote.task()->vm()->find_free_memory(sz);
   remote.infallible_syscall(syscall_number_for_mremap(remote.arch()), start, sz,
                             sz, MREMAP_MAYMOVE | MREMAP_FIXED, free_mem);
-  remote.task()->vm()->remap(start, sz, free_mem, sz);
+  remote.task()->vm()->remap(remote.task(), start, sz, free_mem, sz);
 
   // AutoRemoteSyscalls may have gotten unlucky and picked the old stack
   // segment as it's scratch space, reevaluate that choice
@@ -583,7 +582,7 @@ bool Session::make_private_shared(AutoRemoteSyscalls& remote,
   // Finally unmap the original segment
   remote2.infallible_syscall(syscall_number_for_munmap(remote.arch()), free_mem,
                              sz);
-  remote.task()->vm()->unmap(free_mem, sz);
+  remote.task()->vm()->unmap(remote.task(), free_mem, sz);
   return true;
 }
 
