@@ -233,7 +233,7 @@ public:
   static KernelMapping create_shared_mmap(
       AutoRemoteSyscalls& remote, size_t size, remote_ptr<void> map_hint,
       const char* name, int tracee_prot = PROT_READ | PROT_WRITE,
-      int trace_flags = 0,
+      int tracee_flags = 0,
       MonitoredSharedMemory::shr_ptr&& monitored = nullptr);
 
   static bool make_private_shared(AutoRemoteSyscalls& remote,
@@ -242,6 +242,15 @@ public:
       AutoRemoteSyscalls& remote, const AddressSpace::Mapping& m,
       MonitoredSharedMemory::shr_ptr&& monitored = nullptr);
 
+  enum PtraceSyscallBeforeSeccomp {
+    PTRACE_SYSCALL_BEFORE_SECCOMP,
+    SECCOMP_BEFORE_PTRACE_SYSCALL,
+    PTRACE_SYSCALL_BEFORE_SECCOMP_UNKNOWN,
+  };
+  PtraceSyscallBeforeSeccomp syscall_seccomp_ordering() {
+    return syscall_seccomp_ordering_;
+  }
+
 protected:
   Session();
   virtual ~Session();
@@ -277,6 +286,8 @@ protected:
   uint32_t next_task_serial_;
   ScopedFd spawned_task_error_fd_;
 
+  PtraceSyscallBeforeSeccomp syscall_seccomp_ordering_;
+
   /**
    * True if we've done an exec so tracees are now in a state that will be
    * consistent across record and replay.
