@@ -41,6 +41,7 @@
 #include "ScopedFd.h"
 #include "StdioMonitor.h"
 #include "StringVectorToCharArray.h"
+#include "ThreadDb.h"
 #include "kernel_abi.h"
 #include "kernel_metadata.h"
 #include "kernel_supplement.h"
@@ -60,7 +61,6 @@ Task::Task(Session& session, pid_t _tid, pid_t _rec_tid, uint32_t serial,
            SupportedArch a)
     : unstable(false),
       stable_exit(false),
-      thread_locals_initialized(false),
       scratch_ptr(),
       scratch_size(),
       // This will be initialized when the syscall buffer is.
@@ -71,7 +71,6 @@ Task::Task(Session& session, pid_t _tid, pid_t _rec_tid, uint32_t serial,
       tid(_tid),
       rec_tid(_rec_tid > 0 ? _rec_tid : _tid),
       syscallbuf_size(0),
-      num_syscallbuf_bytes(0),
       stopping_breakpoint_table_entry_size(0),
       serial(serial),
       prname("???"),
@@ -79,12 +78,16 @@ Task::Task(Session& session, pid_t _tid, pid_t _rec_tid, uint32_t serial,
       registers(a),
       how_last_execution_resumed(RESUME_CONT),
       is_stopped(false),
+      seccomp_bpf_enabled(false),
       detected_unexpected_exit(false),
       extra_registers(a),
       extra_registers_known(false),
       session_(&session),
       top_of_stack(),
-      seen_ptrace_exit_event(false) {}
+      seen_ptrace_exit_event(false),
+      expecting_ptrace_interrupt_stop(0) {
+  memset(&thread_locals, 0, sizeof(thread_locals));
+}
 
 void Task::destroy() {
   LOG(debug) << "task " << tid << " (rec:" << rec_tid << ") is dying ...";
@@ -103,8 +106,18 @@ Task::~Task() {
   if (unstable) {
     LOG(warn) << tid << " is unstable; not blocking on its termination";
     // This will probably leak a zombie process for rr's lifetime.
+
+    // Destroying a Session may result in unstable exits during which
+    // Task::destroy_buffers() will not have been called.
+    if (!syscallbuf_child.is_null()) {
+      uint8_t* local_mapping = vm()->mapping_of(syscallbuf_child).local_addr;
+      vm()->unmap(this, syscallbuf_child, syscallbuf_size);
+      int ret = munmap(local_mapping, syscallbuf_size);
+      ASSERT(this, ret >= 0);
+    }
   } else {
     ASSERT(this, seen_ptrace_exit_event);
+    ASSERT(this, syscallbuf_child.is_null());
 
     if (tg->task_set().empty() && !session().is_recording()) {
       // Reap the zombie.
@@ -218,35 +231,59 @@ const siginfo_t& Task::get_siginfo() {
   return pending_siginfo;
 }
 
+/**
+ * Must be idempotent.
+ */
 void Task::destroy_buffers() {
   AutoRemoteSyscalls remote(this);
-  remote.infallible_syscall(syscall_number_for_munmap(arch()), scratch_ptr,
-                            scratch_size);
-  vm()->unmap(scratch_ptr, scratch_size);
-  if (!syscallbuf_child.is_null()) {
-    uint8_t* local_mapping = vm()->mapping_of(syscallbuf_child).local_addr;
+  unmap_buffers_for(remote, this);
+  scratch_ptr = nullptr;
+  syscallbuf_child = nullptr;
+  close_buffers_for(remote, this);
+  desched_fd_child = -1;
+  cloned_file_data_fd_child = -1;
+}
+
+void Task::unmap_buffers_for(AutoRemoteSyscalls& remote, Task* other) {
+  if (other->scratch_ptr) {
     remote.infallible_syscall(syscall_number_for_munmap(arch()),
-                              syscallbuf_child, num_syscallbuf_bytes);
-    vm()->unmap(syscallbuf_child, num_syscallbuf_bytes);
-    munmap(local_mapping, num_syscallbuf_bytes);
-    syscallbuf_child = nullptr;
-    if (desched_fd_child >= 0) {
-      if (session().is_recording()) {
-        remote.infallible_syscall(syscall_number_for_close(arch()),
-                                  desched_fd_child);
-      }
-      fds->did_close(desched_fd_child);
+                              other->scratch_ptr, other->scratch_size);
+    vm()->unmap(this, other->scratch_ptr, other->scratch_size);
+  }
+  if (!other->syscallbuf_child.is_null()) {
+    uint8_t* local_mapping =
+        vm()->mapping_of(other->syscallbuf_child).local_addr;
+    remote.infallible_syscall(syscall_number_for_munmap(arch()),
+                              other->syscallbuf_child, other->syscallbuf_size);
+    vm()->unmap(this, other->syscallbuf_child, other->syscallbuf_size);
+    if (local_mapping) {
+      int ret = munmap(local_mapping, other->syscallbuf_size);
+      ASSERT(this, ret >= 0);
     }
-    if (cloned_file_data_fd_child >= 0) {
+  }
+}
+
+/**
+ * Must be idempotent.
+ */
+void Task::close_buffers_for(AutoRemoteSyscalls& remote, Task* other) {
+  if (other->desched_fd_child >= 0) {
+    if (session().is_recording()) {
       remote.infallible_syscall(syscall_number_for_close(arch()),
-                                cloned_file_data_fd_child);
-      fds->did_close(cloned_file_data_fd_child);
+                                other->desched_fd_child);
     }
+    fds->did_close(other->desched_fd_child);
+  }
+  if (other->cloned_file_data_fd_child >= 0) {
+    remote.infallible_syscall(syscall_number_for_close(arch()),
+                              other->cloned_file_data_fd_child);
+    fds->did_close(other->cloned_file_data_fd_child);
   }
 }
 
 bool Task::is_desched_event_syscall() {
   return is_ioctl_syscall(regs().original_syscallno(), arch()) &&
+         desched_fd_child != -1 &&
          desched_fd_child == (int)regs().arg1_signed();
 }
 
@@ -298,40 +335,34 @@ void Task::on_syscall_exit_arch(int syscallno, const Registers& regs) {
   switch (syscallno) {
     case Arch::brk:
     case Arch::mmap:
-    case Arch::mmap2: {
-      LOG(debug)
-          << "(brk/mmap/mmap2 will receive / has received direct processing)";
+    case Arch::mmap2:
+    case Arch::mremap: {
+      LOG(debug) << "(brk/mmap/mmap2/mremap will receive / has received direct "
+                    "processing)";
       return;
     }
     case Arch::mprotect: {
       remote_ptr<void> addr = regs.arg1();
       size_t num_bytes = regs.arg2();
       int prot = regs.arg3_signed();
-      return vm()->protect(addr, num_bytes, prot);
-    }
-    case Arch::mremap: {
-      remote_ptr<void> old_addr = regs.arg1();
-      size_t old_num_bytes = regs.arg2();
-      remote_ptr<void> new_addr = regs.syscall_result();
-      size_t new_num_bytes = regs.arg3();
-      return vm()->remap(old_addr, old_num_bytes, new_addr, new_num_bytes);
+      return vm()->protect(this, addr, num_bytes, prot);
     }
     case Arch::munmap: {
       remote_ptr<void> addr = regs.arg1();
       size_t num_bytes = regs.arg2();
-      return vm()->unmap(addr, num_bytes);
+      return vm()->unmap(this, addr, num_bytes);
     }
     case Arch::shmdt: {
       remote_ptr<void> addr = regs.arg1();
       auto mapping = vm()->mapping_of(addr);
       ASSERT(this, mapping.map.start() == addr);
-      return vm()->unmap(addr, mapping.map.end() - addr);
+      return vm()->unmap(this, addr, mapping.map.end() - addr);
     }
     case Arch::madvise: {
       remote_ptr<void> addr = regs.arg1();
       size_t num_bytes = regs.arg2();
       int advice = regs.arg3();
-      return vm()->advise(addr, num_bytes, advice);
+      return vm()->advise(this, addr, num_bytes, advice);
     }
     case Arch::ipc: {
       switch ((int)regs.arg1_signed()) {
@@ -339,7 +370,7 @@ void Task::on_syscall_exit_arch(int syscallno, const Registers& regs) {
           remote_ptr<void> addr = regs.arg5();
           auto mapping = vm()->mapping_of(addr);
           ASSERT(this, mapping.map.start() == addr);
-          return vm()->unmap(addr, mapping.map.end() - addr);
+          return vm()->unmap(this, addr, mapping.map.end() - addr);
         }
         default:
           break;
@@ -349,20 +380,15 @@ void Task::on_syscall_exit_arch(int syscallno, const Registers& regs) {
 
     case Arch::set_thread_area:
       set_thread_area(regs.arg1());
-      // Assume any set_thread_area sets up TLS
-      thread_locals_initialized = true;
-      return;
-
-    case Arch::arch_prctl:
-      switch ((int)regs.arg1_signed()) {
-        case ARCH_SET_FS:
-          thread_locals_initialized = true;
-          break;
-      }
       return;
 
     case Arch::prctl:
       switch ((int)regs.arg1_signed()) {
+        case PR_SET_SECCOMP:
+          if (regs.arg2() == SECCOMP_MODE_FILTER && session().is_recording()) {
+            seccomp_bpf_enabled = true;
+          }
+          break;
         case PR_SET_NAME: {
           update_prname(regs.arg2());
           break;
@@ -528,13 +554,38 @@ void Task::move_ip_before_breakpoint() {
   set_regs(r);
 }
 
-void Task::advance_syscall() {
-  while (true) {
-    resume_execution(RESUME_SYSCALL, RESUME_WAIT, RESUME_NO_TICKS);
+void Task::enter_syscall() {
+  bool need_ptrace_syscall_event = !seccomp_bpf_enabled ||
+                                   session().syscall_seccomp_ordering() ==
+                                       Session::SECCOMP_BEFORE_PTRACE_SYSCALL;
+  bool need_seccomp_event = seccomp_bpf_enabled;
+  while (need_ptrace_syscall_event || need_seccomp_event) {
+    resume_execution(need_ptrace_syscall_event ? RESUME_SYSCALL : RESUME_CONT,
+                     RESUME_WAIT, RESUME_NO_TICKS);
     if (is_ptrace_seccomp_event()) {
+      ASSERT(this, need_seccomp_event);
+      need_seccomp_event = false;
       continue;
     }
     ASSERT(this, !ptrace_event());
+    if (!stop_sig()) {
+      ASSERT(this, need_ptrace_syscall_event);
+      need_ptrace_syscall_event = false;
+      continue;
+    }
+    if (ReplaySession::is_ignored_signal(stop_sig()) &&
+        session().is_replaying()) {
+      continue;
+    }
+    ASSERT(this, session().is_recording());
+    static_cast<RecordTask*>(this)->stash_sig();
+  }
+}
+
+void Task::exit_syscall() {
+  while (true) {
+    resume_execution(RESUME_SYSCALL, RESUME_WAIT, RESUME_NO_TICKS);
+    ASSERT(this, !ptrace_event());
     if (!stop_sig()) {
       break;
     }
@@ -550,11 +601,14 @@ void Task::advance_syscall() {
 void Task::exit_syscall_and_prepare_restart() {
   Registers r = regs();
   int syscallno = r.original_syscallno();
+  LOG(debug) << "exit_syscall_and_prepare_restart from syscall "
+             << rr::syscall_name(syscallno, r.arch());
   r.set_original_syscallno(syscall_number_for_gettid(r.arch()));
   set_regs(r);
   // This exits the hijacked SYS_gettid.  Now the tracee is
   // ready to do our bidding.
-  advance_syscall();
+  exit_syscall();
+  LOG(debug) << "exit_syscall_and_prepare_restart done";
 
   // Restore these regs to what they would have been just before
   // the tracee trapped at the syscall.
@@ -580,8 +634,6 @@ void Task::post_exec(SupportedArch a, const string& exe_file) {
   fds->erase_task(this);
 
   registers.set_arch(a);
-  extra_registers = ExtraRegisters(a);
-  extra_registers_known = false;
   struct user_regs_struct ptrace_regs;
   ptrace_if_alive(PTRACE_GETREGS, nullptr, &ptrace_regs);
   registers.set_from_ptrace(ptrace_regs);
@@ -592,13 +644,20 @@ void Task::post_exec(SupportedArch a, const string& exe_file) {
   registers.set_original_syscallno(syscall_number_for_execve(arch()));
   set_regs(registers);
 
+  extra_registers = ExtraRegisters(a);
+  extra_registers_known = false;
+  ExtraRegisters e = extra_regs();
+  e.reset();
+  set_extra_regs(e);
+
   syscallbuf_child = nullptr;
   cloned_file_data_fd_child = -1;
   desched_fd_child = -1;
   preload_globals = nullptr;
-  thread_locals_initialized = false;
+  task_group()->execed = true;
 
   thread_areas_.clear();
+  memset(&thread_locals, 0, sizeof(thread_locals));
 
   as = session().create_vm(this, exe_file, as->uid().exec_count() + 1);
   // It's barely-documented, but Linux unshares the fd table on exec
@@ -611,6 +670,8 @@ void Task::post_exec_syscall(TraceTaskEvent& event) {
   fds->update_for_cloexec(this, event);
 }
 
+bool Task::execed() const { return tg->execed; }
+
 void Task::flush_inconsistent_state() { ticks = 0; }
 
 string Task::read_c_str(remote_ptr<char> child_addr) {
@@ -673,9 +734,9 @@ const ExtraRegisters& Task::extra_regs() {
       struct iovec vec = { extra_registers.data_.data(),
                            extra_registers.data_.size() };
       xptrace(PTRACE_GETREGSET, NT_X86_XSTATE, &vec);
-      ASSERT(this, vec.iov_len == xsave_area_size)
-          << "Didn't get enough register data; expected " << xsave_area_size
-          << " but got " << vec.iov_len;
+      extra_registers.data_.resize(vec.iov_len);
+      // The kernel may return less than the full XSTATE
+      extra_registers.validate(this);
     } else {
 #if defined(__i386__)
       LOG(debug) << "  (refreshing extra-register cache using FPXREGS)";
@@ -777,7 +838,36 @@ TrapReasons Task::compute_trap_reasons() {
   return reasons;
 }
 
-static const Property<bool, AddressSpace> thread_locals_initialized_property;
+const Task::ThreadLocals& Task::fetch_preload_thread_locals() {
+  if (tuid() == as->thread_locals_tuid()) {
+    if (as->has_mapping(AddressSpace::preload_thread_locals_start())) {
+      memcpy(thread_locals,
+             as->mapping_of(AddressSpace::preload_thread_locals_start())
+                 .local_addr,
+             PRELOAD_THREAD_LOCALS_SIZE);
+    } else {
+      // The mapping might have been removed by crazy application code.
+      // That's OK, assuming the preload library was removed too.
+      memset(&thread_locals, 0, sizeof(thread_locals));
+    }
+  }
+  return thread_locals;
+}
+
+void Task::activate_preload_thread_locals() {
+  // Switch thread-locals to the new task.
+  if (tuid() != as->thread_locals_tuid() &&
+      as->has_mapping(AddressSpace::preload_thread_locals_start())) {
+    Task* t = session().find_task(as->thread_locals_tuid());
+    if (t) {
+      t->fetch_preload_thread_locals();
+    }
+    memcpy(
+        as->mapping_of(AddressSpace::preload_thread_locals_start()).local_addr,
+        thread_locals, PRELOAD_THREAD_LOCALS_SIZE);
+    as->set_thread_locals_tuid(tuid());
+  }
+}
 
 void Task::resume_execution(ResumeRequest how, WaitRequest wait_how,
                             TicksRequest tick_period, int sig) {
@@ -791,19 +881,7 @@ void Task::resume_execution(ResumeRequest how, WaitRequest wait_how,
     hpc.reset(tick_period == RESUME_UNLIMITED_TICKS
                   ? 0xffffffff
                   : max<Ticks>(1, tick_period));
-    // Ensure preload_globals.thread_locals_initialized is up to date. Avoid
-    // unnecessary writes by caching last written value per-AddressSpace.
-    if (preload_globals) {
-      bool* prop = thread_locals_initialized_property.get(*as);
-      if (!prop || *prop != thread_locals_initialized) {
-        write_mem(REMOTE_PTR_FIELD(preload_globals, thread_locals_initialized),
-                  (unsigned char)thread_locals_initialized);
-        if (!prop) {
-          prop = &thread_locals_initialized_property.create(*as);
-        }
-        *prop = thread_locals_initialized;
-      }
-    }
+    activate_preload_thread_locals();
   }
 
   LOG(debug) << "resuming execution of " << tid << " with "
@@ -1104,6 +1182,7 @@ void Task::wait(double interrupt_after_elapsed) {
     if (!sent_wait_interrupt && interrupt_after_elapsed) {
       ptrace_if_alive(PTRACE_INTERRUPT, nullptr, nullptr);
       sent_wait_interrupt = true;
+      expecting_ptrace_interrupt_stop = 2;
     }
   }
 
@@ -1128,29 +1207,11 @@ void Task::wait(double interrupt_after_elapsed) {
              << status;
   ASSERT(this, tid == ret) << "waitpid(" << tid << ") failed with " << ret;
 
-  // If some other ptrace-stop happened to race with our
-  // PTRACE_INTERRUPT, then let the other event win.  We only
-  // want to interrupt tracees stuck running in userspace.
-  // We convert the ptrace-stop to a reschedule signal.
-  if (sent_wait_interrupt &&
-      is_signal_triggered_by_ptrace_interrupt(status.group_stop())) {
-    LOG(warn) << "Forced to PTRACE_INTERRUPT tracee";
-    // Force this timeslice to end
-    if (session().is_recording()) {
-      session().as_record()->scheduler().expire_timeslice();
-    }
-    status = WaitStatus::for_stop_sig(PerfCounters::TIME_SLICE_SIGNAL);
-    siginfo_t si;
-    memset(&si, 0, sizeof(si));
-    si.si_signo = PerfCounters::TIME_SLICE_SIGNAL;
-    si.si_fd = hpc.ticks_fd();
-    si.si_code = POLL_IN;
-    did_waitpid(status, &si);
-    return;
-  }
-
   if (sent_wait_interrupt) {
-    LOG(warn) << "  PTRACE_INTERRUPT raced with another event " << status;
+    LOG(warn) << "Forced to PTRACE_INTERRUPT tracee";
+    if (!is_signal_triggered_by_ptrace_interrupt(status.group_stop())) {
+      LOG(warn) << "  PTRACE_INTERRUPT raced with another event " << status;
+    }
   }
   did_waitpid(status);
 }
@@ -1223,13 +1284,42 @@ void Task::emulate_syscall_entry(const Registers& regs) {
   set_regs(r);
 }
 
-void Task::did_waitpid(WaitStatus status, siginfo_t* override_siginfo) {
-  Ticks more_ticks = hpc.read_ticks();
-  // Stop PerfCounters ASAP to reduce the possibility that due to bugs or
-  // whatever they pick up something spurious later.
-  hpc.stop();
-  ticks += more_ticks;
+void Task::did_waitpid(WaitStatus status) {
+  Ticks more_ticks = hpc.counting ? hpc.read_ticks() : 0;
+  // We stop counting here because there may be things we want to do to the
+  // tracee that would otherwise generate ticks.
+  hpc.stop_counting();
   session().accumulate_ticks_processed(more_ticks);
+  ticks += more_ticks;
+
+  // After PTRACE_INTERRUPT, any next two stops may be a group stop caused by
+  // that PTRACE_INTERRUPT (or neither may be). This is because PTRACE_INTERRUPT
+  // generally lets other stops win (and thus doesn't inject it's own stop), but
+  // if the other stop was already done processing, even we didn't see it yet,
+  // the stop will still be queued, so we could see the other stop and then the
+  // PTRACE_INTERRUPT group stop.
+  // When we issue PTRACE_INTERRUPT, we this set this counter to 2, and here
+  // we decrement it on every stop such that while this counter is positive,
+  // any group-stop could be one induced by PTRACE_INTERRUPT
+  bool siginfo_overriden = false;
+  if (expecting_ptrace_interrupt_stop > 0) {
+    expecting_ptrace_interrupt_stop--;
+    if (is_signal_triggered_by_ptrace_interrupt(status.group_stop())) {
+      // Assume this was PTRACE_INTERRUPT and thus treat this as
+      // TIME_SLICE_SIGNAL instead.
+      if (session().is_recording()) {
+        // Force this timeslice to end
+        session().as_record()->scheduler().expire_timeslice();
+      }
+      status = WaitStatus::for_stop_sig(PerfCounters::TIME_SLICE_SIGNAL);
+      memset(&pending_siginfo, 0, sizeof(pending_siginfo));
+      pending_siginfo.si_signo = PerfCounters::TIME_SLICE_SIGNAL;
+      pending_siginfo.si_fd = hpc.ticks_fd();
+      pending_siginfo.si_code = POLL_IN;
+      siginfo_overriden = true;
+      expecting_ptrace_interrupt_stop = 0;
+    }
+  }
 
   LOG(debug) << "  (refreshing register cache)";
   intptr_t original_syscallno = registers.original_syscallno();
@@ -1246,14 +1336,10 @@ void Task::did_waitpid(WaitStatus status, siginfo_t* override_siginfo) {
       status = WaitStatus::for_ptrace_event(PTRACE_EVENT_EXIT);
     }
   }
-  if (status.stop_sig()) {
-    if (override_siginfo) {
-      pending_siginfo = *override_siginfo;
-    } else {
-      if (!ptrace_if_alive(PTRACE_GETSIGINFO, nullptr, &pending_siginfo)) {
-        LOG(debug) << "Unexpected process death for " << tid;
-        status = WaitStatus::for_ptrace_event(PTRACE_EVENT_EXIT);
-      }
+  if (!siginfo_overriden && status.stop_sig()) {
+    if (!ptrace_if_alive(PTRACE_GETSIGINFO, nullptr, &pending_siginfo)) {
+      LOG(debug) << "Unexpected process death for " << tid;
+      status = WaitStatus::for_ptrace_event(PTRACE_EVENT_EXIT);
     }
   }
 
@@ -1337,6 +1423,9 @@ Task* Task::clone(int flags, remote_ptr<void> stack, remote_ptr<void> tls,
   auto& sess = other_session ? *other_session : session();
   Task* t = sess.new_task(new_tid, new_rec_tid, new_serial, arch());
 
+  bool unmap_buffers = false;
+  bool close_buffers = false;
+
   if (CLONE_SHARE_TASK_GROUP & flags) {
     t->tg = tg;
   } else {
@@ -1352,13 +1441,14 @@ Task* Task::clone(int flags, remote_ptr<void> stack, remote_ptr<void> tls,
         if (!mapping.recorded_map.is_heap()) {
           const KernelMapping& m = mapping.map;
           LOG(debug) << "mapping stack for " << new_tid << " at " << m;
-          t->as->map(m.start(), m.size(), m.prot(), m.flags(),
+          t->as->map(t, m.start(), m.size(), m.prot(), m.flags(),
                      m.file_offset_bytes(), "[stack]", m.device(), m.inode());
         }
       }
     }
   } else {
     t->as = sess.clone(t, as);
+    unmap_buffers = as->task_set().size() > 1;
   }
 
   t->syscallbuf_size = syscallbuf_size;
@@ -1366,6 +1456,7 @@ Task* Task::clone(int flags, remote_ptr<void> stack, remote_ptr<void> tls,
   t->stopping_breakpoint_table_entry_size =
       stopping_breakpoint_table_entry_size;
   t->preload_globals = preload_globals;
+  t->seccomp_bpf_enabled = seccomp_bpf_enabled;
 
   // FdTable is either shared or copied, so the contents of
   // syscallbuf_fds_disabled_child are still valid.
@@ -1374,6 +1465,7 @@ Task* Task::clone(int flags, remote_ptr<void> stack, remote_ptr<void> tls,
     t->fds->insert_task(t);
   } else {
     t->fds = fds->clone(t);
+    close_buffers = fds->task_set().size() > 1;
   }
 
   t->top_of_stack = stack;
@@ -1387,57 +1479,55 @@ Task* Task::clone(int flags, remote_ptr<void> stack, remote_ptr<void> tls,
 
   t->open_mem_fd_if_needed();
   t->thread_areas_ = thread_areas_;
-  // When cloning a task in the same session, the new task's thread-locals
-  // are not initialized ... unless CLONE_SET_TLS is set.
-  if (other_session) {
-    t->thread_locals_initialized = thread_locals_initialized;
-  }
   if (CLONE_SET_TLS & flags) {
     set_thread_area_from_clone(t, tls);
-    t->thread_locals_initialized = true;
   }
 
   t->as->insert_task(t);
 
-  if (!(CLONE_SHARE_VM & flags) && &session() == &t->session()) {
-    as->did_fork_into(t);
-
-    if (!syscallbuf_child.is_null()) {
+  if (&session() == &t->session()) {
+    if (unmap_buffers) {
+      // Unmap syscallbuf and scratch for tasks that were not cloned into
+      // the new address space
       AutoRemoteSyscalls remote(t);
-      // Unshare the syscallbuf memory so when we lock it below, we don't
-      // also lock it in the task we cloned from!
-      int prot = PROT_READ | PROT_WRITE;
-      int flags = MAP_PRIVATE | MAP_FIXED | MAP_ANONYMOUS;
-      remote.infallible_mmap_syscall(syscallbuf_child, num_syscallbuf_bytes,
-                                     prot, flags, -1, 0);
-      t->vm()->map(syscallbuf_child, num_syscallbuf_bytes, prot, flags, 0,
-                   string(), KernelMapping::NO_DEVICE, KernelMapping::NO_INODE);
-
-      // Mark the clone's syscallbuf as locked. This will prevent the
-      // clone using syscallbuf until the clone reinitializes the
-      // the buffer via its pthread_atfork handler. Otherwise the clone may
-      // log syscalls to its copy of the syscallbuf and we won't know about
-      // them since we don't have it mapped.
-      // In some cases (e.g. vfork(), or raw SYS_fork syscall) the
-      // pthread_atfork handler will never run. Syscallbuf will be permanently
-      // disabled but that's OK, those cases are rare (and in the case of vfork,
-      // tracees should immediately exit or exec anyway).
-      t->write_mem(REMOTE_PTR_FIELD(syscallbuf_child, locked), uint8_t(1));
-
-      if (CLONE_SHARE_FILES & flags) {
-        // Clear our desched_fd_child so that we don't try to close it.
-        // It should only be closed in |this|.
-        t->desched_fd_child = -1;
-        t->cloned_file_data_fd_child = -1;
+      for (Task* tt : as->task_set()) {
+        if (tt != this) {
+          t->unmap_buffers_for(remote, tt);
+        }
       }
     }
+    if (close_buffers) {
+      // Close syscallbuf fds for tasks that were not cloned into
+      // the new fd table
+      AutoRemoteSyscalls remote(t);
+      for (Task* tt : fds->task_set()) {
+        if (tt != this) {
+          t->close_buffers_for(remote, tt);
+        }
+      }
+    }
+
+    if (!(CLONE_SHARE_VM & flags)) {
+      as->did_fork_into(t);
+    }
+
+    if (CLONE_SHARE_FILES & flags) {
+      // Clear our desched_fd_child so that we don't try to close it.
+      // It should only be closed in |this|.
+      t->desched_fd_child = -1;
+      t->cloned_file_data_fd_child = -1;
+    }
+  }
+
+  if (!(CLONE_SHARE_VM & flags)) {
+    t->as->post_vm_clone(t);
   }
 
   return t;
 }
 
 Task* Task::os_fork_into(Session* session) {
-  AutoRemoteSyscalls remote(this);
+  AutoRemoteSyscalls remote(this, AutoRemoteSyscalls::DISABLE_MEMORY_PARAMS);
   Task* child = os_clone(this, session, remote, rec_tid, serial,
                          // Most likely, we'll be setting up a
                          // CLEARTID futex.  That's not done
@@ -1482,7 +1572,7 @@ template <typename Arch>
 static void copy_tls_arch(const Task::CapturedState& state,
                           AutoRemoteSyscalls& remote) {
   if (Arch::clone_tls_type == Arch::UserDescPointer) {
-    for (const struct user_desc& t : state.thread_areas) {
+    for (const auto& t : state.thread_areas) {
       AutoRestoreMem remote_tls(remote, (const uint8_t*)&t, sizeof(t));
       LOG(debug) << "    setting tls " << remote_tls.get();
       remote.infallible_syscall(
@@ -1529,13 +1619,14 @@ Task::CapturedState Task::capture_state() {
   state.extra_regs = extra_regs();
   state.prname = prname;
   state.thread_areas = thread_areas_;
-  state.num_syscallbuf_bytes = num_syscallbuf_bytes;
   state.desched_fd_child = desched_fd_child;
   state.cloned_file_data_fd_child = cloned_file_data_fd_child;
   state.cloned_file_data_offset =
       cloned_file_data_fd_child >= 0
           ? get_fd_offset(this, cloned_file_data_fd_child)
           : 0;
+  memcpy(&state.thread_locals, fetch_preload_thread_locals(),
+         PRELOAD_THREAD_LOCALS_SIZE);
   state.syscallbuf_child = syscallbuf_child;
   state.syscallbuf_size = syscallbuf_size;
   state.preload_globals = preload_globals;
@@ -1544,7 +1635,6 @@ Task::CapturedState Task::capture_state() {
   state.wait_status = wait_status;
   state.ticks = ticks;
   state.top_of_stack = top_of_stack;
-  state.thread_locals_initialized = thread_locals_initialized;
   return state;
 }
 
@@ -1572,7 +1662,6 @@ void Task::copy_state(const CapturedState& state) {
         << "Syscallbuf should not already be initialized in clone";
     if (!state.syscallbuf_child.is_null()) {
       // All these fields are preserved by the fork.
-      num_syscallbuf_bytes = state.num_syscallbuf_bytes;
       desched_fd_child = state.desched_fd_child;
       cloned_file_data_fd_child = state.cloned_file_data_fd_child;
       if (cloned_file_data_fd_child >= 0) {
@@ -1583,6 +1672,7 @@ void Task::copy_state(const CapturedState& state) {
     }
   }
   preload_globals = state.preload_globals;
+  memcpy(&thread_locals, &state.thread_locals, PRELOAD_THREAD_LOCALS_SIZE);
   // The scratch buffer (for now) is merely a private mapping in
   // the remote task.  The CoW copy made by fork()'ing the
   // address space has the semantics we want.  It's not used in
@@ -1595,8 +1685,6 @@ void Task::copy_state(const CapturedState& state) {
   wait_status = state.wait_status;
 
   ticks = state.ticks;
-
-  thread_locals_initialized = state.thread_locals_initialized;
 }
 
 remote_ptr<const struct syscallbuf_record> Task::next_syscallbuf_record() {
@@ -1657,8 +1745,10 @@ void Task::open_mem_fd_if_needed() {
 
 KernelMapping Task::init_syscall_buffer(AutoRemoteSyscalls& remote,
                                         remote_ptr<void> map_hint) {
-  KernelMapping km = Session::create_shared_mmap(remote, syscallbuf_size,
-                                                 map_hint, "syscallbuf");
+  char name[50];
+  sprintf(name, "syscallbuf.%d", rec_tid);
+  KernelMapping km =
+      Session::create_shared_mmap(remote, syscallbuf_size, map_hint, name);
   auto& m = remote.task()->vm()->mapping_of(km.start());
   remote.task()->vm()->mapping_flags_of(km.start()) |=
       AddressSpace::Mapping::IS_SYSCALLBUF;
@@ -1666,7 +1756,6 @@ KernelMapping Task::init_syscall_buffer(AutoRemoteSyscalls& remote,
   ASSERT(this, !syscallbuf_child)
       << "Should not already have syscallbuf initialized!";
 
-  num_syscallbuf_bytes = syscallbuf_size;
   syscallbuf_child = km.start().cast<struct syscallbuf_hdr>();
 
   // No entries to begin with.
@@ -1839,7 +1928,7 @@ void Task::read_bytes_helper(remote_ptr<void> addr, ssize_t buf_size, void* buf,
 static ssize_t safe_pwrite64(Task* t, const void* buf, ssize_t buf_size,
                              remote_ptr<void> addr) {
   vector<KernelMapping> mappings_to_fix;
-  for (auto m : t->vm()->maps_starting_at(floor_page_size(addr))) {
+  for (auto m : t->vm()->maps_containing_or_after(floor_page_size(addr))) {
     if (m.map.start() >= ceil_page_size(addr + buf_size)) {
       break;
     }
@@ -1972,7 +2061,7 @@ bool Task::clone_syscall_is_complete() {
 
 template <typename Arch> static void do_preload_init_arch(Task* t) {
   auto params = t->read_mem(
-      remote_ptr<rrcall_init_preload_params<Arch> >(t->regs().arg1()));
+      remote_ptr<rrcall_init_preload_params<Arch>>(t->regs().arg1()));
 
   t->preload_globals = params.globals.rptr();
 
@@ -2266,12 +2355,6 @@ static void run_initial_child(Session& session, const ScopedFd& error_fd,
     FATAL() << "Failed to fork";
   }
 
-  struct sigaction sa;
-  sa.sa_handler = handle_alarm_signal;
-  sigemptyset(&sa.sa_mask);
-  sa.sa_flags = 0; // No SA_RESTART, so waitpid() will be interrupted
-  sigaction(SIGALRM, &sa, nullptr);
-
   // Sync with the child process.
   // We minimize the code we run between fork()ing and PTRACE_SEIZE, because
   // any abnormal exit of the rr process will leave the child paused and
@@ -2316,6 +2399,14 @@ static void run_initial_child(Session& session, const ScopedFd& error_fd,
   t->fds = FdTable::create(t);
   setup_fd_table(*t->fds);
 
+  // Install signal handler here, so that when creating the first RecordTask
+  // it sees the exact same signal state in the parent as will be in the child.
+  struct sigaction sa;
+  sa.sa_handler = handle_alarm_signal;
+  sigemptyset(&sa.sa_mask);
+  sa.sa_flags = 0; // No SA_RESTART, so waitpid() will be interrupted
+  sigaction(SIGALRM, &sa, nullptr);
+
   t->wait();
   if (t->ptrace_event() == PTRACE_EVENT_EXIT) {
     FATAL() << "Tracee died before reaching SIGSTOP\n"
@@ -2339,4 +2430,17 @@ static void run_initial_child(Session& session, const ScopedFd& error_fd,
 string Task::syscall_name(int syscall) const {
   return rr::syscall_name(syscall, arch());
 } // namespace rr
+
+bool Task::get_tls_address(size_t offset, remote_ptr<void> load_module,
+                           remote_ptr<void>* result) {
+  return tg->thread_db()->get_tls_address(rec_tid, offset, load_module, result);
+}
+
+void Task::register_symbol(const std::string& name, remote_ptr<void> address) {
+  tg->thread_db()->register_symbol(name, address);
+}
+
+const std::set<std::string> Task::get_symbols_and_clear_map() {
+  return tg->thread_db()->get_symbols_and_clear_map();
+}
 }
