@@ -188,10 +188,8 @@ public:
    * Force the wait status of this to |status|, as if
    * |wait()/try_wait()| had returned it. Call this whenever a waitpid
    * returned activity for this past.
-   * If override_siginfo is non-null and status indicates a pending signal,
-   * use *override_siginfo as the siginfo instead of reading it from the kernel.
    */
-  void did_waitpid(WaitStatus status, siginfo_t* override_siginfo = nullptr);
+  void did_waitpid(WaitStatus status);
 
   /**
    * Syscalls have side effects on registers (e.g. setting the flags register).
@@ -228,6 +226,9 @@ public:
    */
   void destroy_buffers();
 
+  void unmap_buffers_for(AutoRemoteSyscalls& remote, Task* t);
+  void close_buffers_for(AutoRemoteSyscalls& remote, Task* t);
+
   remote_ptr<const struct syscallbuf_record> next_syscallbuf_record();
   long stored_record_size(remote_ptr<const struct syscallbuf_record> record);
 
@@ -304,11 +305,19 @@ public:
   void exit_syscall_and_prepare_restart();
 
   /**
-   * Resume execution until we get a syscall entry or exit event.
-   * During recording, any signals received are stashed.
-   * seccomp events are ignored; we assume this syscall is under rr's control.
+   * We're currently in user-space with registers set up to perform a system
+   * call. Continue into the kernel and stop where we can modify the syscall
+   * state.
    */
-  void advance_syscall();
+  void enter_syscall();
+
+  /**
+   * We have observed entry to a syscall (either by PTRACE_EVENT_SECCOMP or
+   * a syscall, depending on the value of Session::syscall_seccomp_ordering()).
+   * Continue into the kernel to perform the syscall and stop at the
+   * PTRACE_SYSCALL syscall-exit trap.
+   */
+  void exit_syscall();
 
   /**
    * Return the "task name"; i.e. what |prctl(PR_GET_NAME)| or
@@ -332,6 +341,11 @@ public:
    */
   void post_exec_syscall(TraceTaskEvent& event);
 
+  /**
+   * Return true if this task has execed.
+   */
+  bool execed() const;
+
   /**
    * Read |N| bytes from |child_addr| into |buf|, or don't
    * return.
@@ -617,6 +631,27 @@ public:
    */
   std::string syscall_name(int syscallno) const;
 
+  /**
+   * Look up a TLS address for this thread.  |offset| and
+   * |load_module| are as specified in the qGetTLSAddr packet.  If the
+   * address is found, set |*result| and return true.  Otherwise,
+   * return false.
+   */
+  bool get_tls_address(size_t offset, remote_ptr<void> load_module,
+                       remote_ptr<void>* result);
+
+  /**
+   * Indicate that the symbol |name| has the given address.
+   */
+  void register_symbol(const std::string& name, remote_ptr<void> address);
+
+  /**
+   * Return a set of the names of all the symbols that might be needed
+   * by libthread_db.  Also clears the current mapping of symbol names
+   * to addresses.
+   */
+  const std::set<std::string> get_symbols_and_clear_map();
+
   /* True when any assumptions made about the status of this
    * process have been invalidated, and must be re-established
    * with a waitpid() call. Only applies to tasks which are dying, usually
@@ -626,11 +661,6 @@ public:
    * the exit can be treated as stable. */
   bool stable_exit;
 
-  /* True when arch_prctl (x86_64) or set_thread_area/CLONE_SETTLS (x86) have
-   * been used to set up thread-local storage for this task.
-   */
-  bool thread_locals_initialized;
-
   /* Imagine that task A passes buffer |b| to the read()
    * syscall.  Imagine that, after A is switched out for task B,
    * task B then writes to |b|.  Then B is switched out for A.
@@ -689,13 +719,14 @@ public:
   pid_t rec_tid;
 
   size_t syscallbuf_size;
-  size_t num_syscallbuf_bytes;
   /* Points at the tracee's mapping of the buffer. */
   remote_ptr<struct syscallbuf_hdr> syscallbuf_child;
   remote_code_ptr stopping_breakpoint_table;
   int stopping_breakpoint_table_entry_size;
 
   remote_ptr<struct preload_globals> preload_globals;
+  typedef uint8_t ThreadLocals[PRELOAD_THREAD_LOCALS_SIZE];
+  ThreadLocals thread_locals;
 
   PropertyTable& properties() { return properties_; }
 
@@ -713,12 +744,12 @@ public:
     ssize_t scratch_size;
     remote_ptr<void> top_of_stack;
     uint64_t cloned_file_data_offset;
+    ThreadLocals thread_locals;
     pid_t rec_tid;
     uint32_t serial;
     int desched_fd_child;
     int cloned_file_data_fd_child;
     WaitStatus wait_status;
-    bool thread_locals_initialized;
   };
 
 protected:
@@ -764,6 +795,9 @@ protected:
    */
   void copy_state(const CapturedState& state);
 
+  const ThreadLocals& fetch_preload_thread_locals();
+  void activate_preload_thread_locals();
+
   /**
    * Make the ptrace |request| with |addr| and |data|, return
    * the ptrace return value.
@@ -890,6 +924,10 @@ protected:
   // True when we know via waitpid() that the task is stopped and we haven't
   // resumed it.
   bool is_stopped;
+  /* True when the seccomp filter has been enabled via prctl(). This happens
+   * in the first system call issued by the initial tracee (after it returns
+   * from kill(SIGSTOP) to synchronize with the tracer). */
+  bool seccomp_bpf_enabled;
   // True when we consumed a PTRACE_EVENT_EXIT that was about to race with
   // a resume_execution, that was issued while stopped (i.e. SIGKILL).
   bool detected_unexpected_exit;
@@ -918,6 +956,10 @@ protected:
 
   PropertyTable properties_;
 
+  // A counter for the number of stops for which the stop may have been caused
+  // by PTRACE_INTERRUPT. See description in do_waitpid
+  int expecting_ptrace_interrupt_stop;
+
   Task(Task&) = delete;
   Task operator=(Task&) = delete;
 };
