@@ -4,6 +4,7 @@
 
 #include "Session.h"
 #include "Task.h"
+#include "ThreadDb.h"
 #include "log.h"
 
 namespace rr {
@@ -13,6 +14,8 @@ TaskGroup::TaskGroup(Session* session, TaskGroup* parent, pid_t tgid,
     : tgid(tgid),
       real_tgid(real_tgid),
       dumpable(true),
+      execed(false),
+      received_sigframe_SIGSEGV(false),
       session_(session),
       parent_(parent),
       serial(serial) {
@@ -45,4 +48,11 @@ void TaskGroup::destabilize() {
   }
 }
 
+ThreadDb* TaskGroup::thread_db() {
+  if (!thread_db_) {
+    thread_db_ = std::unique_ptr<ThreadDb>(new ThreadDb(this));
+  }
+  return thread_db_.get();
+}
+
 } // namespace rr
