@@ -31,7 +31,7 @@ namespace rr {
 // MUST increment this version number.  Otherwise users' old traces
 // will become unreplayable and they won't know why.
 //
-#define TRACE_VERSION 58
+#define TRACE_VERSION 65
 
 struct SubstreamData {
   const char* name;
@@ -187,17 +187,23 @@ struct BasicInfo {
 void TraceWriter::write_frame(const TraceFrame& frame) {
   auto& events = writer(EVENTS);
 
-  BasicInfo basic_info = { frame.time(), frame.tid(), frame.event().encode(),
-                           frame.ticks(), frame.monotonic_time() };
+  BasicInfo basic_info;
+  memset(&basic_info, 0, sizeof(BasicInfo));
+  basic_info.global_time = frame.time();
+  basic_info.tid_ = frame.tid();
+  basic_info.ev = frame.event().encode();
+  basic_info.ticks_ = frame.ticks();
+  basic_info.monotonic_sec = frame.monotonic_time();
   events << basic_info;
   if (!events.good()) {
     FATAL() << "Tried to save " << sizeof(basic_info)
             << " bytes to the trace, but failed";
   }
-  // TODO: only store exec info for non-async-sig events when
-  // debugging assertions are enabled.
   if (frame.event().has_exec_info() == HAS_EXEC_INFO) {
-    events << frame.regs() << frame.extra_perf_values();
+    events << (char)frame.regs().arch();
+    auto raw_regs = frame.regs().get_ptrace_for_arch(frame.regs().arch());
+    events.write(raw_regs.data(), raw_regs.size());
+    events << frame.extra_perf_values();
     if (!events.good()) {
       FATAL() << "Tried to save registers to the trace, but failed";
     }
@@ -236,7 +242,25 @@ TraceFrame TraceReader::read_frame() {
                    Event(basic_info.ev), basic_info.ticks_,
                    basic_info.monotonic_sec);
   if (frame.event().has_exec_info() == HAS_EXEC_INFO) {
-    events >> frame.recorded_regs >> frame.extra_perf;
+    char a;
+    events >> a;
+    uint8_t buf[sizeof(X64Arch::user_regs_struct)];
+    frame.recorded_regs.set_arch((SupportedArch)a);
+    switch (frame.recorded_regs.arch()) {
+      case x86:
+        events.read(buf, sizeof(X86Arch::user_regs_struct));
+        frame.recorded_regs.set_from_ptrace_for_arch(
+            x86, buf, sizeof(X86Arch::user_regs_struct));
+        break;
+      case x86_64:
+        events.read(buf, sizeof(X64Arch::user_regs_struct));
+        frame.recorded_regs.set_from_ptrace_for_arch(
+            x86_64, buf, sizeof(X64Arch::user_regs_struct));
+        break;
+      default:
+        FATAL() << "Unknown arch";
+    }
+    events >> frame.extra_perf;
 
     int extra_reg_bytes;
     char extra_reg_format;
@@ -265,14 +289,11 @@ TraceFrame TraceReader::read_frame() {
 
 void TraceWriter::write_task_event(const TraceTaskEvent& event) {
   auto& tasks = writer(TASKS);
-  tasks << event.type() << event.tid();
+  tasks << global_time << (char)event.type() << event.tid();
   switch (event.type()) {
     case TraceTaskEvent::CLONE:
       tasks << event.parent_tid() << event.clone_flags();
       break;
-    case TraceTaskEvent::FORK:
-      tasks << event.parent_tid();
-      break;
     case TraceTaskEvent::EXEC:
       tasks << event.file_name() << event.cmd_line() << event.fds_to_close();
       break;
@@ -288,14 +309,14 @@ void TraceWriter::write_task_event(const TraceTaskEvent& event) {
 TraceTaskEvent TraceReader::read_task_event() {
   auto& tasks = reader(TASKS);
   TraceTaskEvent r;
-  tasks >> r.type_ >> r.tid_;
+  TraceFrame::Time time;
+  char type = TraceTaskEvent::NONE;
+  tasks >> time >> type >> r.tid_;
+  r.type_ = (TraceTaskEvent::Type)type;
   switch (r.type()) {
     case TraceTaskEvent::CLONE:
       tasks >> r.parent_tid_ >> r.clone_flags_;
       break;
-    case TraceTaskEvent::FORK:
-      tasks >> r.parent_tid_;
-      break;
     case TraceTaskEvent::EXEC:
       tasks >> r.file_name_ >> r.cmd_line_ >> r.fds_to_close_;
       break;
@@ -306,6 +327,8 @@ TraceTaskEvent TraceReader::read_task_event() {
       // Should be EOF only
       assert(!tasks.good());
       break;
+    default:
+      assert(false && "Corrupt Trace?");
   }
   return r;
 }
@@ -364,7 +387,9 @@ TraceWriter::RecordInTrace TraceWriter::write_mapped_region(
   auto& mmaps = writer(MMAPS);
   TraceReader::MappedDataSource source;
   string backing_file_name;
-  if (km.fsname().find("/SYSV") == 0) {
+  if (origin == REMAP_MAPPING || origin == PATCH_MAPPING) {
+    source = TraceReader::SOURCE_ZERO;
+  } else if (km.fsname().find("/SYSV") == 0) {
     source = TraceReader::SOURCE_TRACE;
   } else if (origin == SYSCALL_MAPPING &&
              (km.inode() == 0 || km.fsname() == "/dev/zero (deleted)")) {
@@ -473,10 +498,11 @@ KernelMapping TraceReader::read_mapped_region(MappedData* data, bool* found,
                        flags, file_offset_bytes);
 }
 
-void TraceWriter::write_raw(const void* d, size_t len, remote_ptr<void> addr) {
+void TraceWriter::write_raw(pid_t rec_tid, const void* d, size_t len,
+                            remote_ptr<void> addr) {
   auto& data = writer(RAW_DATA);
   auto& data_header = writer(RAW_DATA_HEADER);
-  data_header << global_time << addr.as_int() << len;
+  data_header << global_time << rec_tid << addr.as_int() << len;
   data.write(d, len);
 }
 
@@ -486,7 +512,7 @@ TraceReader::RawData TraceReader::read_raw_data() {
   TraceFrame::Time time;
   RawData d;
   size_t num_bytes;
-  data_header >> time >> d.addr >> num_bytes;
+  data_header >> time >> d.rec_tid >> d.addr >> num_bytes;
   assert(time == global_time);
   d.data.resize(num_bytes);
   data.read((char*)d.data.data(), num_bytes);
