@@ -116,6 +116,8 @@ public:
   enum RecordInTrace { DONT_RECORD_IN_TRACE, RECORD_IN_TRACE };
   enum MappingOrigin {
     SYSCALL_MAPPING,
+    // Just memory moved from one place to another, so no recording needed.
+    REMAP_MAPPING,
     EXEC_MAPPING,
     PATCH_MAPPING,
     RR_BUFFER_MAPPING
@@ -134,7 +136,8 @@ public:
    * 'addr' is the address in the tracee where the data came from/will be
    * restored to.
    */
-  void write_raw(const void* data, size_t len, remote_ptr<void> addr);
+  void write_raw(pid_t tid, const void* data, size_t len,
+                 remote_ptr<void> addr);
 
   /**
    * Write a task event (clone or exec record) to the trace.
@@ -183,7 +186,7 @@ private:
    * Files that have already been mapped without being copied to the trace,
    * i.e. that we have already assumed to be immutable.
    */
-  std::set<std::pair<dev_t, ino_t> > files_assumed_immutable;
+  std::set<std::pair<dev_t, ino_t>> files_assumed_immutable;
   uint32_t mmap_count;
   bool supports_file_data_cloning_;
 };
@@ -197,6 +200,7 @@ public:
   struct RawData {
     std::vector<uint8_t> data;
     remote_ptr<void> addr;
+    pid_t rec_tid;
   };
 
   /**
@@ -208,6 +212,10 @@ public:
    */
   TraceFrame read_frame();
 
+  /**
+   * For REMAP_MAPPING maps, the memory contents are preserved so we don't
+   * need a source. We use SOURCE_ZERO for that case and it's ignored.
+   */
   enum MappedDataSource { SOURCE_TRACE, SOURCE_FILE, SOURCE_ZERO };
   /**
    * Where to obtain data for the mapped region.
