@@ -23,21 +23,15 @@ class TraceTaskEvent {
 public:
   enum Type {
     NONE,
-    CLONE, // created by clone(2) syscall
-    FORK,  // created by fork(2) or vfork(2) syscalls
+    CLONE, // created by clone(2), fork(2), vfork(2) syscalls
     EXEC,
     EXIT
   };
 
   TraceTaskEvent(Type type = NONE, pid_t tid = 0) : type_(type), tid_(tid) {}
 
-  static TraceTaskEvent for_fork(pid_t tid, pid_t parent_tid) {
-    TraceTaskEvent result(FORK, tid);
-    result.parent_tid_ = parent_tid;
-    return result;
-  }
   static TraceTaskEvent for_clone(pid_t tid, pid_t parent_tid,
-                                  uint32_t clone_flags) {
+                                  uint64_t clone_flags) {
     TraceTaskEvent result(CLONE, tid);
     result.parent_tid_ = parent_tid;
     result.clone_flags_ = clone_flags;
@@ -59,10 +53,10 @@ public:
   Type type() const { return type_; }
   pid_t tid() const { return tid_; }
   pid_t parent_tid() const {
-    assert(type() == CLONE || type() == FORK);
+    assert(type() == CLONE);
     return parent_tid_;
   }
-  uintptr_t clone_flags() const {
+  uint64_t clone_flags() const {
     assert(type() == CLONE);
     return clone_flags_;
   }
@@ -83,10 +77,6 @@ public:
     return exit_status_;
   }
 
-  bool is_fork() const {
-    return type() == FORK || (type() == CLONE && !(clone_flags() & CLONE_VM));
-  }
-
   void set_fds_to_close(const std::vector<int> fds) {
     assert(type() == EXEC);
     fds_to_close_ = fds;
@@ -99,7 +89,7 @@ private:
   Type type_;
   pid_t tid_;
   pid_t parent_tid_;                  // CLONE only
-  uintptr_t clone_flags_;             // CLONE only
+  uint64_t clone_flags_;              // CLONE only
   std::string file_name_;             // EXEC only
   std::vector<std::string> cmd_line_; // EXEC only
   std::vector<int> fds_to_close_;     // EXEC only
