@@ -1,14 +1,13 @@
 /* -*- Mode: C++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 
-//#define DEBUGTAG "FastForward"
-
 #include "fast_forward.h"
 
 #include "log.h"
 
-using namespace rr;
 using namespace std;
 
+namespace rr {
+
 struct InstructionBuf {
   SupportedArch arch;
   uint8_t code_buf[32];
@@ -157,7 +156,7 @@ bool fast_forward_through_instruction(Task* t, ResumeRequest how,
   remote_code_ptr ip = t->ip();
 
   t->resume_execution(how, RESUME_WAIT, RESUME_UNLIMITED_TICKS);
-  if (t->pending_sig() != SIGTRAP) {
+  if (t->stop_sig() != SIGTRAP) {
     // we might have stepped into a system call...
     return false;
   }
@@ -165,7 +164,7 @@ bool fast_forward_through_instruction(Task* t, ResumeRequest how,
   if (t->ip() != ip) {
     return false;
   }
-  if (t->vm()->get_breakpoint_type_at_addr(ip) != TRAP_NONE) {
+  if (t->vm()->get_breakpoint_type_at_addr(ip) != BKPT_NONE) {
     // breakpoint must have fired
     return false;
   }
@@ -216,6 +215,8 @@ bool fast_forward_through_instruction(Task* t, ResumeRequest how,
 
     uintptr_t cur_cx = t->regs().cx();
     if (cur_cx == 0) {
+      // Fake singlestep status for trap diagnosis
+      t->set_debug_status(DS_SINGLESTEP);
       // This instruction will be skipped entirely.
       return did_execute;
     }
@@ -273,6 +274,8 @@ bool fast_forward_through_instruction(Task* t, ResumeRequest how,
     }
 
     if (iterations == 0) {
+      // Fake singlestep status for trap diagnosis
+      t->set_debug_status(DS_SINGLESTEP);
       return did_execute;
     }
 
@@ -297,20 +300,30 @@ bool fast_forward_through_instruction(Task* t, ResumeRequest how,
       LOG(debug) << "Set x86-string fast-forward watchpoint at " << watch_di;
       bool ok = t->vm()->add_watchpoint(watch_di, 1, WATCH_READWRITE);
       ASSERT(t, ok) << "Can't even handle one watchpoint???";
-      ok = t->vm()->add_breakpoint(limit_ip, TRAP_BKPT_INTERNAL);
+      ok = t->vm()->add_breakpoint(limit_ip, BKPT_INTERNAL);
       ASSERT(t, ok) << "Failed to add breakpoint";
 
       t->resume_execution(RESUME_CONT, RESUME_WAIT, RESUME_UNLIMITED_TICKS);
       did_execute = true;
-      ASSERT(t, t->pending_sig() == SIGTRAP);
+      ASSERT(t, t->stop_sig() == SIGTRAP);
       // Grab debug_status before restoring watchpoints, since the latter
       // clears the debug status
       bool triggered_watchpoint =
-          t->vm()->notify_watchpoint_fired(t->consume_debug_status());
-      t->vm()->remove_breakpoint(limit_ip, TRAP_BKPT_INTERNAL);
+          t->vm()->notify_watchpoint_fired(t->debug_status());
+      t->vm()->remove_breakpoint(limit_ip, BKPT_INTERNAL);
       t->vm()->restore_watchpoints();
 
-      iterations -= cur_cx - t->regs().cx();
+      ASSERT(t, cur_cx > t->regs().cx());
+      uintptr_t iterations_performed = cur_cx - t->regs().cx();
+      // we shoudn't execute more iterations than we asked for.
+      // In Ubuntu-14 4.2.0-27-generic in a KVM guest we have seen watchpoints
+      // failing to fire during string_instructions_replay. We plow through
+      // and hit the backup breakpoint. triggered_watchpoint is true because
+      // the memory has changed. This assertion should catch such errors.
+      ASSERT(t, iterations >= iterations_performed);
+      iterations -= iterations_performed;
+      // instructions that don't modify flags should not terminate too early
+      ASSERT(t, decoded.modifies_flags || iterations <= BYTES_COALESCED);
 
       if (!triggered_watchpoint) {
         // watchpoint didn't fire. We must have exited the loop early and
@@ -323,8 +336,11 @@ bool fast_forward_through_instruction(Task* t, ResumeRequest how,
         tmp.set_ip(limit_ip);
         t->set_regs(tmp);
       } else {
+        ASSERT(t, t->ip() == limit_ip || t->ip() == ip);
         watch_offset = decoded.operand_size * (iterations - 1);
         if (watch_offset > BYTES_COALESCED) {
+          // Fake singlestep status for trap diagnosis
+          t->set_debug_status(DS_SINGLESTEP);
           // We fired the watchpoint too early, perhaps because reads through SI
           // triggered it. Let's just bail out now; better for the caller to
           // retry
@@ -345,8 +361,7 @@ bool fast_forward_through_instruction(Task* t, ResumeRequest how,
       t->resume_execution(RESUME_SINGLESTEP, RESUME_WAIT,
                           RESUME_UNLIMITED_TICKS);
       did_execute = true;
-      ASSERT(t, t->pending_sig() == SIGTRAP);
-      t->consume_debug_status();
+      ASSERT(t, t->stop_sig() == SIGTRAP);
       // Watchpoints can fire spuriously because configure_watch_registers
       // can increase the size of the watched area to conserve watch registers.
       --iterations;
@@ -368,7 +383,7 @@ bool fast_forward_through_instruction(Task* t, ResumeRequest how,
     } else {
       LOG(debug) << "x86-string fast-forward done; ip()==" << t->ip();
       // Fake singlestep status for trap diagnosis
-      t->replace_debug_status(DS_SINGLESTEP);
+      t->set_debug_status(DS_SINGLESTEP);
       return did_execute;
     }
   }
@@ -469,3 +484,5 @@ bool maybe_at_or_after_x86_string_instruction(Task* t) {
   return is_string_instruction_at(t, t->ip()) ||
          is_string_instruction_before(t, t->ip());
 }
+
+} // namespace rr
