@@ -844,7 +844,7 @@ struct BaseArch : public wordsize,
   };
   RR_VERIFY_TYPE(ethtool_cmd);
 
-  struct flock {
+  struct _flock {
     signed_short l_type;
     signed_short l_whence;
     char __pad[sizeof(off_t) - 2 * sizeof(short)];
@@ -852,7 +852,7 @@ struct BaseArch : public wordsize,
     off_t l_len;
     pid_t l_pid;
   };
-  RR_VERIFY_TYPE(flock);
+  RR_VERIFY_TYPE_EXPLICIT(struct ::flock, _flock);
 
   struct flock64 {
     signed_short l_type;
@@ -881,7 +881,11 @@ struct BaseArch : public wordsize,
     ptr<socklen_t> addrlen;
   };
 
-  struct accept4_args : public accept_args {
+  struct accept4_args {
+    signed_int sockfd;
+    char __pad[sizeof(ptr<void>) - sizeof(int)];
+    ptr<sockaddr> addr;
+    ptr<socklen_t> addrlen;
     signed_long flags;
   };
 
@@ -901,6 +905,15 @@ struct BaseArch : public wordsize,
     ptr<socklen_t> optlen;
   };
 
+  struct setsockopt_args {
+    signed_int sockfd;
+    signed_int level;
+    signed_int optname;
+    char __pad[sizeof(ptr<void>) - sizeof(int)];
+    ptr<void> optval;
+    ptr<socklen_t> optlen;
+  };
+
   struct recv_args {
     signed_int sockfd;
     char __pad[sizeof(ptr<void>) - sizeof(int)];
@@ -1024,24 +1037,29 @@ struct BaseArch : public wordsize,
   };
   RR_VERIFY_TYPE(__sysctl_args);
 
+  typedef struct {
+    unsigned_long __val[64 / (8 * sizeof(unsigned_long))];
+  } kernel_sigset_t;
+
+  // libc reserves some space in the user facing structures for future
+  // extensibility.
   typedef struct {
     unsigned_long __val[1024 / (8 * sizeof(unsigned_long))];
   } __sigset_t;
   typedef __sigset_t sigset_t;
   RR_VERIFY_TYPE(sigset_t);
 
+  typedef struct {
+    ptr<const sigset_t> ss;
+    size_t ss_len;
+  } pselect6_arg6;
+
   struct kernel_sigaction {
     ptr<void> k_sa_handler;
     unsigned_long sa_flags;
     ptr<void> sa_restorer;
-    sigset_t sa_mask;
+    kernel_sigset_t sa_mask;
   };
-
-  // The 'size' parameter to pass to rt_sigaction. Only this value works,
-  // even though sizeof(sigset_t) > 8 (it's actually 128 with kernel 3.16,
-  // as above).
-  enum { sigaction_sigset_size = 8 };
-
   struct tms {
     clock_t tms_utime;
     clock_t tms_stime;
@@ -1288,6 +1306,33 @@ struct BaseArch : public wordsize,
     ptr<void> data;
   };
   RR_VERIFY_TYPE(usbdevfs_ctrltransfer);
+
+  struct dirent {
+    ino_t d_ino;
+    off_t d_off;
+    uint16_t d_reclen;
+    //    uint8_t d_type;
+    uint8_t d_name[256];
+  };
+  RR_VERIFY_TYPE(dirent);
+
+  struct dirent64 {
+    ino64_t d_ino;
+    off64_t d_off;
+    uint16_t d_reclen;
+    uint8_t d_type;
+    uint8_t d_name[256];
+  };
+  RR_VERIFY_TYPE(dirent64);
+
+  struct mq_attr {
+    signed_long mq_flags;
+    signed_long mq_maxmsg;
+    signed_long mq_msgsize;
+    signed_long mq_curmsgs;
+    signed_long __reserved[4];
+  };
+  RR_VERIFY_TYPE(mq_attr);
 };
 
 struct X86Arch : public BaseArch<SupportedArch::x86, WordSize32Defs> {
@@ -1367,6 +1412,68 @@ struct X86Arch : public BaseArch<SupportedArch::x86, WordSize32Defs> {
                       user_fpxregs_struct);
 #endif
 
+  struct sigcontext {
+    uint16_t gs, __gsh;
+    uint16_t fs, __fsh;
+    uint16_t es, __esh;
+    uint16_t ds, __dsh;
+    uint32_t di;
+    uint32_t si;
+    uint32_t bp;
+    uint32_t sp;
+    uint32_t bx;
+    uint32_t dx;
+    uint32_t cx;
+    uint32_t ax;
+    uint32_t trapno;
+    uint32_t err;
+    uint32_t ip;
+    uint16_t cs, __csh;
+    uint32_t flags;
+    uint32_t sp_at_signal;
+    uint16_t ss, __ssh;
+    uint32_t fpstate;
+    uint32_t oldmask;
+    uint32_t cr2;
+  };
+  RR_VERIFY_TYPE_ARCH(SupportedArch::x86, ::sigcontext, sigcontext);
+
+  struct ucontext {
+    uint32_t uc_flags;
+    uint32_t uc_link;
+    stack_t uc_stack;
+    sigcontext uc_mcontext;
+    kernel_sigset_t uc_sigmask;
+  };
+
+  struct rt_sigframe {
+    uint32_t pretcode;
+    int sig;
+    uint32_t pinfo;
+    uint32_t puc;
+    siginfo_t info;
+    struct ucontext uc;
+  };
+
+  struct _fpstate_32 {
+    uint32_t cw, sw, tag, ipoff, cssel, dataoff, datasel;
+    uint16_t _st[40];
+    uint16_t status, magic;
+    uint32_t _fxsr_env[6], mxcsr, reserved;
+    uint32_t _fxsr_st[32];
+    uint32_t _xmm[32];
+    uint32_t padding[56];
+  };
+
+  struct sigframe {
+    uint32_t pretcode;
+    int sig;
+    sigcontext sc;
+    _fpstate_32 unused;
+    uint32_t extramask;
+    char retcode[8];
+  };
+
   struct user {
     user_regs_struct regs;
     int u_fpvalid;
@@ -1493,6 +1600,38 @@ struct X64Arch : public BaseArch<SupportedArch::x86_64, WordSize64Defs> {
   RR_VERIFY_TYPE_ARCH(SupportedArch::x86_64, ::user_regs_struct,
                       user_regs_struct);
 
+  struct sigcontext {
+    uint64_t r8;
+    uint64_t r9;
+    uint64_t r10;
+    uint64_t r11;
+    uint64_t r12;
+    uint64_t r13;
+    uint64_t r14;
+    uint64_t r15;
+    uint64_t di;
+    uint64_t si;
+    uint64_t bp;
+    uint64_t bx;
+    uint64_t dx;
+    uint64_t ax;
+    uint64_t cx;
+    uint64_t sp;
+    uint64_t ip;
+    uint64_t flags;
+    uint16_t cs;
+    uint16_t gs;
+    uint16_t fs;
+    uint16_t __pad0;
+    uint64_t err;
+    uint64_t trapno;
+    uint64_t oldmask;
+    uint64_t cr2;
+    uint64_t fpstate;
+    uint64_t reserved[8];
+  };
+  RR_VERIFY_TYPE_ARCH(SupportedArch::x86_64, ::sigcontext, sigcontext);
+
   struct user_fpregs_struct {
     uint16_t cwd;
     uint16_t swd;
@@ -1509,6 +1648,22 @@ struct X64Arch : public BaseArch<SupportedArch::x86_64, WordSize64Defs> {
   RR_VERIFY_TYPE_ARCH(SupportedArch::x86_64, ::user_fpregs_struct,
                       user_fpregs_struct);
 
+  struct ucontext {
+    uint64_t ucflags;
+    ptr<struct ucontext> uc_link;
+    stack_t uc_stack;
+    struct sigcontext uc_mcontext;
+    sigset_t uc_sigmask;
+    user_fpregs_struct uc_fpregs;
+  };
+  RR_VERIFY_TYPE_ARCH(SupportedArch::x86_64, ::ucontext, ucontext);
+
+  struct rt_sigframe {
+    ptr<char> pretcode;
+    struct ucontext uc;
+    siginfo_t info;
+  };
+
   struct user {
     struct user_regs_struct regs;
     int u_fpvalid;
@@ -1615,4 +1770,9 @@ typedef X64Arch NativeArch;
 
 } // namespace rr
 
+// New in the 4.6 kernel.
+#ifndef CLONE_NEWCGROUP
+#define CLONE_NEWCGROUP 0x02000000
+#endif
+
 #endif /* RR_KERNEL_ABI_H */
