@@ -10,29 +10,31 @@
 
 #include "kernel_abi.h"
 
+namespace rr {
+
 /**
- * Return the symbolic name of |syscall|, f.e. "read", or "???syscall"
+ * Return the symbolic name of |syscall|, f.e. "read", or "syscall(%d)"
  * if unknown.
  */
 std::string syscall_name(int syscall, SupportedArch arch);
 
 /**
  * Return the symbolic name of the PTRACE_EVENT_* |event|, or
- * "???EVENT" if unknown.
+ * "PTRACE_EVENT(%d)" if unknown.
  */
-const char* ptrace_event_name(int event);
+std::string ptrace_event_name(int event);
 
 /**
- * Return the symbolic name of the PTRACE_ |request|, or "???REQ" if
+ * Return the symbolic name of the PTRACE_ |request|, or "PTRACE_REQUEST(%d)" if
  * unknown.
  */
-const char* ptrace_req_name(int request);
+std::string ptrace_req_name(int request);
 
 /**
- * Return the symbolic name of |sig|, f.e. "SIGILL", or "???signal" if
+ * Return the symbolic name of |sig|, f.e. "SIGILL", or "signal(%d)" if
  * unknown.
  */
-const char* signal_name(int sig);
+std::string signal_name(int sig);
 
 /**
  * Return true if this is some kind of sigreturn syscall.
@@ -47,7 +49,7 @@ std::string errno_name(int err);
 /**
  * Return the symbolic name (e.g. "SI_USER") for an si_code.
  */
-const char* sicode_name(int code, int sig);
+std::string sicode_name(int code, int sig);
 
 /**
  * Print siginfo on ostream.
@@ -56,4 +58,6 @@ std::ostream& operator<<(std::ostream& stream, const siginfo_t& siginfo);
 
 int shm_flags_to_mmap_prot(int flags);
 
+} // namespace rr
+
 #endif
