@@ -48,6 +48,9 @@ namespace rr {
 #ifndef SECCOMP_SET_MODE_FILTER
 #define SECCOMP_SET_MODE_FILTER 1
 #endif
+#ifndef SECCOMP_FILTER_FLAG_TSYNC
+#define SECCOMP_FILTER_FLAG_TSYNC 1
+#endif
 
 #ifndef SYS_SECCOMP
 #define SYS_SECCOMP 1
@@ -124,6 +127,9 @@ struct usbdevfs_streams {
 #define TIOCGEXCL _IOR('T', 0x40, int)
 #endif
 
+#ifndef MADV_FREE
+#define MADV_FREE 8
+#endif
 #ifndef MADV_DONTDUMP
 #define MADV_DONTDUMP 16
 #endif
