@@ -119,64 +119,68 @@ static int process_inited;
 static struct preload_globals globals;
 
 /**
- * Because this library is always loaded via LD_PRELOAD, we can use the
- * initial-exec TLS model (see http://www.akkadia.org/drepper/tls.pdf) which
- * lets the compiler generate better code which, crucially, does not call
- * helper functions outside of our library.
+ * Can be architecture dependent. The rr process does not manipulate
+ * these except to save and restore the values on task switches so that
+ * the values are always effectively local to the current task.
+ * We use this instead of regular libc TLS because sometimes buggy application
+ * code breaks libc TLS for some tasks. With this approach we can be sure
+ * thread-locals are usable for any task in any state.
  */
-#define TLS_STORAGE_MODEL __attribute__((tls_model("initial-exec")))
+struct preload_thread_locals {
+  /* Nonzero when thread-local state like the syscallbuf has been
+   * initialized.  */
+  int thread_inited;
+  /* When buffering is enabled, points at the thread's mapped buffer
+   * segment.  At the start of the segment is an object of type |struct
+   * syscallbuf_hdr|, so |buffer| is also a pointer to the buffer
+   * header. */
+  uint8_t* buffer;
+  size_t buffer_size;
+  /* This is used to support the buffering of "may-block" system calls.
+   * The problem that needs to be addressed can be introduced with a
+   * simple example; assume that we're buffering the "read" and "write"
+   * syscalls.
+   *
+   *  o (Tasks W and R set up a synchronous-IO pipe open between them; W
+   *    "owns" the write end of the pipe; R owns the read end; the pipe
+   *    buffer is full)
+   *  o Task W invokes the write syscall on the pipe
+   *  o Since write is a buffered syscall, the seccomp filter traps W
+   *    directly to the kernel; there's no trace event for W delivered
+   *    to rr.
+   *  o The pipe is full, so W is descheduled by the kernel because W
+   *    can't make progress.
+   *  o rr thinks W is still running and doesn't schedule R.
+   *
+   * At this point, progress in the recorded application can only be
+   * made by scheduling R, but no one tells rr to do that.  Oops!
+   *
+   * Thus enter the "desched counter".  It's a perf_event for the "sw t
+   * switches" event (which, more precisely, is "sw deschedule"; it
+   * counts schedule-out, not schedule-in).  We program the counter to
+   * deliver a signal to this task when there's new counter data
+   * available.  And we set up the "sample period", how many descheds
+   * are triggered before the signal is delivered, to be "1".  This
+   * means that when the counter is armed, the next desched (i.e., the
+   * next time the desched counter is bumped up) of this task will
+   * deliver the signal to it.  And signal delivery always generates a
+   * ptrace trap, so rr can deduce that this task was descheduled and
+   * schedule another.
+   *
+   * The description above is sort of an idealized view; there are
+   * numerous implementation details that are documented in
+   * handle_signal.c, where they're dealt with. */
+  int desched_counter_fd;
+  int cloned_file_data_fd;
+  off_t cloned_file_data_offset;
+  void* scratch_buf;
+  size_t scratch_size;
 
-/* Nonzero when thread-local state like the syscallbuf has been
- * initialized.  */
-static __thread int thread_inited TLS_STORAGE_MODEL;
-/* When buffering is enabled, points at the thread's mapped buffer
- * segment.  At the start of the segment is an object of type |struct
- * syscallbuf_hdr|, so |buffer| is also a pointer to the buffer
- * header. */
-static __thread uint8_t* buffer TLS_STORAGE_MODEL;
-static __thread size_t buffer_size TLS_STORAGE_MODEL;
-/* This is used to support the buffering of "may-block" system calls.
- * The problem that needs to be addressed can be introduced with a
- * simple example; assume that we're buffering the "read" and "write"
- * syscalls.
- *
- *  o (Tasks W and R set up a synchronous-IO pipe open between them; W
- *    "owns" the write end of the pipe; R owns the read end; the pipe
- *    buffer is full)
- *  o Task W invokes the write syscall on the pipe
- *  o Since write is a buffered syscall, the seccomp filter traps W
- *    directly to the kernel; there's no trace event for W delivered
- *    to rr.
- *  o The pipe is full, so W is descheduled by the kernel because W
- *    can't make progress.
- *  o rr thinks W is still running and doesn't schedule R.
- *
- * At this point, progress in the recorded application can only be
- * made by scheduling R, but no one tells rr to do that.  Oops!
- *
- * Thus enter the "desched counter".  It's a perf_event for the "sw t
- * switches" event (which, more precisely, is "sw deschedule"; it
- * counts schedule-out, not schedule-in).  We program the counter to
- * deliver a signal to this task when there's new counter data
- * available.  And we set up the "sample period", how many descheds
- * are triggered before the signal is delivered, to be "1".  This
- * means that when the counter is armed, the next desched (i.e., the
- * next time the desched counter is bumped up) of this task will
- * deliver the signal to it.  And signal delivery always generates a
- * ptrace trap, so rr can deduce that this task was descheduled and
- * schedule another.
- *
- * The description above is sort of an idealized view; there are
- * numerous implementation details that are documented in
- * handle_signal.c, where they're dealt with. */
-static __thread int desched_counter_fd TLS_STORAGE_MODEL;
+  struct msghdr* notify_control_msg;
+};
 
-static __thread int cloned_file_data_fd TLS_STORAGE_MODEL;
-static __thread off_t cloned_file_data_offset TLS_STORAGE_MODEL;
-static __thread void* scratch_buf TLS_STORAGE_MODEL;
-static __thread size_t scratch_size TLS_STORAGE_MODEL;
-
-static __thread struct msghdr* notify_control_msg TLS_STORAGE_MODEL;
+static struct preload_thread_locals* const thread_locals =
+    (struct preload_thread_locals*)PRELOAD_THREAD_LOCALS_ADDR;
 
 /* Points at the libc/pthread pthread_create().  We wrap
  * pthread_create, so need to retain this pointer to call out to the
@@ -193,7 +197,7 @@ static int (*real_pthread_mutex_timedlock)(pthread_mutex_t* mutex,
  * initial bytes in the mapped region.
  */
 static struct syscallbuf_hdr* buffer_hdr(void) {
-  return (struct syscallbuf_hdr*)buffer;
+  return (struct syscallbuf_hdr*)thread_locals->buffer;
 }
 
 /**
@@ -208,7 +212,9 @@ static uint8_t* buffer_last(void) {
  * Return a pointer to the byte just after the very end of the mapped
  * region.
  */
-static uint8_t* buffer_end(void) { return buffer + buffer_size; }
+static uint8_t* buffer_end(void) {
+  return thread_locals->buffer + thread_locals->buffer_size;
+}
 
 /**
  * Same as libc memcpy(), but usable within syscallbuf transaction
@@ -541,24 +547,27 @@ static int open_desched_event_counter(size_t nr_descheds, pid_t tid) {
 }
 
 /**
- * Initialize thread-local buffering state, if enabled.
+ * Initialize thread-local buffering state, if enabled and not already
+ * initialized.
  */
 static void init_thread(void) {
   struct rrcall_init_buffers_params args;
 
   assert(process_inited);
-  assert(!thread_inited);
-
-  if (!buffer_enabled) {
-    thread_inited = 1;
+  if (thread_locals->thread_inited) {
+    return;
+  }
+  /* Do not do any syscall buffering in a DiversionSession! */
+  if (!buffer_enabled || globals.in_diversion) {
+    thread_locals->thread_inited = 1;
     return;
   }
 
   /* NB: we want this setup emulated during replay. */
-  desched_counter_fd =
+  thread_locals->desched_counter_fd =
       open_desched_event_counter(1, privileged_traced_gettid());
 
-  args.desched_counter_fd = desched_counter_fd;
+  args.desched_counter_fd = thread_locals->desched_counter_fd;
 
   /* Trap to rr: let the magic begin!
    *
@@ -568,27 +577,14 @@ static void init_thread(void) {
    * the signal is unblocked. */
   rrcall_init_buffers(&args);
 
-  cloned_file_data_fd = args.cloned_file_data_fd;
+  thread_locals->cloned_file_data_fd = args.cloned_file_data_fd;
   /* rr initializes the buffer header. */
-  buffer = args.syscallbuf_ptr;
-  buffer_size = args.syscallbuf_size;
-  scratch_buf = args.scratch_buf;
-  scratch_size = args.scratch_size;
+  thread_locals->buffer = args.syscallbuf_ptr;
+  thread_locals->buffer_size = args.syscallbuf_size;
+  thread_locals->scratch_buf = args.scratch_buf;
+  thread_locals->scratch_size = args.scratch_size;
 
-  thread_inited = 1;
-}
-
-/**
- * After a fork(), we retain a CoW mapping of our parent's syscallbuf.
- * That's bad, because we don't want to use that buffer.  So drop the
- * parent's copy and reinstall our own.
- *
- * FIXME: this "leaks" the parent's old copy in our address space.
- */
-static void post_fork_child(void) {
-  buffer = NULL;
-  thread_inited = 0;
-  init_thread();
+  thread_locals->thread_inited = 1;
 }
 
 extern char _breakpoint_table_entry_start;
@@ -627,19 +623,23 @@ static void __attribute__((constructor)) init_process(void) {
   extern RR_HIDDEN void _syscall_hook_trampoline_89_c2_f7_da(void);
   extern RR_HIDDEN void _syscall_hook_trampoline_90_90_90(void);
   extern RR_HIDDEN void _syscall_hook_trampoline_ba_01_00_00_00(void);
+  extern RR_HIDDEN void _syscall_hook_trampoline_89_c1_31_d2(void);
 
   struct syscall_patch_hook syscall_patch_hooks[] = {
-    /* Many glibc syscall wrappers (e.g. read) have 'syscall' followed by
+    /* Many glibc syscall wrappers (e.g. read) have 'syscall' followed
+     * by
      * cmp $-4095,%rax (in glibc-2.18-16.fc20.x86_64) */
     { 6,
       { 0x48, 0x3d, 0x01, 0xf0, 0xff, 0xff },
       (uintptr_t)_syscall_hook_trampoline_48_3d_01_f0_ff_ff },
-    /* Many glibc syscall wrappers (e.g. __libc_recv) have 'syscall' followed by
+    /* Many glibc syscall wrappers (e.g. __libc_recv) have 'syscall'
+     * followed by
      * cmp $-4096,%rax (in glibc-2.18-16.fc20.x86_64) */
     { 6,
       { 0x48, 0x3d, 0x00, 0xf0, 0xff, 0xff },
       (uintptr_t)_syscall_hook_trampoline_48_3d_00_f0_ff_ff },
-    /* Many glibc syscall wrappers (e.g. read) have 'syscall' followed by
+    /* Many glibc syscall wrappers (e.g. read) have 'syscall' followed
+     * by
      * mov (%rsp),%rdi (in glibc-2.18-16.fc20.x86_64) */
     { 4,
       { 0x48, 0x8b, 0x3c, 0x24 },
@@ -652,30 +652,38 @@ static void __attribute__((constructor)) init_process(void) {
     { 4,
       { 0x89, 0xc2, 0xf7, 0xda },
       (uintptr_t)_syscall_hook_trampoline_89_c2_f7_da },
-    /* Our VDSO vsyscall patches have 'syscall' followed by "nop; nop; nop" */
+    /* Our VDSO vsyscall patches have 'syscall' followed by "nop; nop;
+       nop" */
     { 3,
       { 0x90, 0x90, 0x90 },
       (uintptr_t)_syscall_hook_trampoline_ba_01_00_00_00 },
-    /* glibc-2.22-17.fc23.x86_64 has 'syscall' followed by 'mov $1,%rdx' in
+    /* glibc-2.22-17.fc23.x86_64 has 'syscall' followed by 'mov $1,%rdx'
+     * in
      * pthread_barrier_wait.
      */
     { 5,
       { 0xba, 0x01, 0x00, 0x00, 0x00 },
-      (uintptr_t)_syscall_hook_trampoline_ba_01_00_00_00 }
+      (uintptr_t)_syscall_hook_trampoline_ba_01_00_00_00 },
+    /* pthread_sigmask has 'syscall' followed by 'mov %eax,%ecx; xor
+       %edx,%edx' */
+    { 4,
+      { 0x89, 0xc1, 0x31, 0xd2 },
+      (uintptr_t)_syscall_hook_trampoline_89_c1_31_d2 }
   };
 
   real_pthread_create = dlsym(RTLD_NEXT, "pthread_create");
 #else
 #error Unknown architecture
 #endif
+
+  assert(sizeof(struct preload_thread_locals) <= PRELOAD_THREAD_LOCALS_SIZE);
+
   if (process_inited) {
     return;
   }
 
   buffer_enabled = !!getenv(SYSCALLBUF_ENABLED_ENV_VAR);
 
-  pthread_atfork(NULL, NULL, post_fork_child);
-
   params.syscallbuf_enabled = buffer_enabled;
   params.syscall_hook_trampoline = (void*)_syscall_hook_trampoline;
   params.syscall_hook_end = (void*)_syscall_hook_end;
@@ -690,8 +698,6 @@ static void __attribute__((constructor)) init_process(void) {
   privileged_traced_syscall1(SYS_rrcall_init_preload, &params);
 
   process_inited = 1;
-
-  init_thread();
 }
 
 /**
@@ -708,8 +714,6 @@ static void* thread_trampoline(void* arg) {
   struct thread_func_data data = *(struct thread_func_data*)arg;
   free(arg);
 
-  init_thread();
-
   return data.start_routine(data.arg);
 }
 
@@ -725,7 +729,6 @@ static void* thread_trampoline(void* arg) {
 int pthread_create(pthread_t* thread, const pthread_attr_t* attr,
                    void* (*start_routine)(void*), void* arg) {
   struct thread_func_data* data = malloc(sizeof(*data));
-  void* saved_buffer = buffer;
   int ret;
 
   /* Init syscallbuf now if we haven't yet (e.g. if pthread_create is called
@@ -735,10 +738,7 @@ int pthread_create(pthread_t* thread, const pthread_attr_t* attr,
 
   data->start_routine = start_routine;
   data->arg = arg;
-  /* Don't let the new thread use our TLS pointer. */
-  buffer = NULL;
   ret = real_pthread_create(thread, attr, thread_trampoline, data);
-  buffer = saved_buffer;
   return ret;
 }
 
@@ -834,7 +834,7 @@ int pthread_mutex_trylock(pthread_mutex_t* mutex) {
  */
 
 static void* prep_syscall(void) {
-  if (!buffer) {
+  if (!thread_locals->buffer) {
     return NULL;
   }
   if (buffer_hdr()->locked) {
@@ -880,17 +880,19 @@ static void arm_desched_event(void) {
    * avoid! :) Although we don't allocate extra space for these
    * ioctl's, we do record that we called them; the replayer
    * knows how to skip over them. */
-  if ((int)privileged_untraced_syscall3(SYS_ioctl, desched_counter_fd,
+  if ((int)privileged_untraced_syscall3(SYS_ioctl,
+                                        thread_locals->desched_counter_fd,
                                         PERF_EVENT_IOC_ENABLE, 0)) {
-    fatal("Failed to ENABLE counter %d", desched_counter_fd);
+    fatal("Failed to ENABLE counter %d", thread_locals->desched_counter_fd);
   }
 }
 
 static void disarm_desched_event(void) {
   /* See above. */
-  if ((int)privileged_untraced_syscall3(SYS_ioctl, desched_counter_fd,
+  if ((int)privileged_untraced_syscall3(SYS_ioctl,
+                                        thread_locals->desched_counter_fd,
                                         PERF_EVENT_IOC_DISABLE, 0)) {
-    fatal("Failed to DISABLE counter %d", desched_counter_fd);
+    fatal("Failed to DISABLE counter %d", thread_locals->desched_counter_fd);
   }
 }
 
@@ -908,7 +910,7 @@ static int start_commit_buffered_syscall(int syscallno, void* record_end,
   void* stored_end;
   struct syscallbuf_record* rec;
 
-  if (!buffer) {
+  if (!thread_locals->buffer) {
     return 0;
   }
   record_start = buffer_last();
@@ -1661,9 +1663,21 @@ static long sys_madvise(const struct syscall_info* call) {
   long ret;
 
   switch (advice) {
-    case MADV_DOFORK:
-    case MADV_DONTFORK:
-    case MADV_REMOVE:
+    // Whitelist advice values that we know are OK to pass through to the
+    // kernel directly.
+    case MADV_NORMAL:
+    case MADV_RANDOM:
+    case MADV_SEQUENTIAL:
+    case MADV_WILLNEED:
+    case MADV_DONTNEED:
+    case MADV_MERGEABLE:
+    case MADV_UNMERGEABLE:
+    case MADV_HUGEPAGE:
+    case MADV_NOHUGEPAGE:
+    case MADV_DONTDUMP:
+    case MADV_DODUMP:
+      break;
+    default:
       return traced_raw_syscall(call);
   }
 
@@ -1826,8 +1840,9 @@ static long sys_read(const struct syscall_info* call) {
    * race, it's almost certainly a bad bug because Linux can return any
    * interleaving of old+new data for the read even without rr.
    */
-  if (buf && count >= CLONE_SIZE_THRESHOLD && cloned_file_data_fd >= 0 &&
-      is_bufferable_fd(fd) && sizeof(void*) == 8 && !(count & 4095)) {
+  if (buf && count >= CLONE_SIZE_THRESHOLD &&
+      thread_locals->cloned_file_data_fd >= 0 && is_bufferable_fd(fd) &&
+      sizeof(void*) == 8 && !(count & 4095)) {
     struct syscall_info lseek_call = { SYS_lseek,
                                        { fd, 0, SEEK_CUR, 0, 0, 0 } };
     off_t lseek_ret = sys_generic_nonblocking_fd(&lseek_call);
@@ -1838,7 +1853,7 @@ static long sys_read(const struct syscall_info* call) {
       ioctl_args.src_fd = fd;
       ioctl_args.src_offset = lseek_ret;
       ioctl_args.src_length = count;
-      ioctl_args.dest_offset = cloned_file_data_offset;
+      ioctl_args.dest_offset = thread_locals->cloned_file_data_offset;
 
       /* Don't call sys_ioctl here; cloned_file_data_fd has syscall buffering
        * disabled for it so rr can reject attempts to close/dup to it. But
@@ -1846,25 +1861,26 @@ static long sys_read(const struct syscall_info* call) {
        */
       if (!start_commit_buffered_syscall(SYS_ioctl, ioctl_ptr, WONT_BLOCK)) {
         struct syscall_info ioctl_call = { SYS_ioctl,
-                                           { cloned_file_data_fd,
+                                           { thread_locals->cloned_file_data_fd,
                                              BTRFS_IOC_CLONE_RANGE,
                                              (long)&ioctl_args, 0, 0, 0 } };
         ioctl_ret = traced_raw_syscall(&ioctl_call);
       } else {
-        ioctl_ret = untraced_syscall3(SYS_ioctl, cloned_file_data_fd,
-                                      BTRFS_IOC_CLONE_RANGE, &ioctl_args);
+        ioctl_ret =
+            untraced_syscall3(SYS_ioctl, thread_locals->cloned_file_data_fd,
+                              BTRFS_IOC_CLONE_RANGE, &ioctl_args);
         ioctl_ret = commit_raw_syscall(SYS_ioctl, ioctl_ptr, ioctl_ret);
       }
 
       if (ioctl_ret >= 0) {
         struct syscall_info read_call = { SYS_read,
                                           { fd, (long)buf, count, 0, 0, 0 } };
-        cloned_file_data_offset += count;
+        thread_locals->cloned_file_data_offset += count;
 
-        replay_only_syscall2(SYS_dup2, cloned_file_data_fd, fd);
+        replay_only_syscall2(SYS_dup2, thread_locals->cloned_file_data_fd, fd);
 
         ptr = prep_syscall();
-        if (count > scratch_size) {
+        if (count > thread_locals->scratch_size) {
           if (!start_commit_buffered_syscall(SYS_read, ptr, WONT_BLOCK)) {
             return traced_raw_syscall(&read_call);
           }
@@ -1873,8 +1889,9 @@ static long sys_read(const struct syscall_info* call) {
           if (!start_commit_buffered_syscall(SYS_read, ptr, MAY_BLOCK)) {
             return traced_raw_syscall(&read_call);
           }
-          ret = untraced_replayed_syscall3(SYS_read, fd, scratch_buf, count);
-          copy_output_buffer(ret, NULL, buf, scratch_buf);
+          ret = untraced_replayed_syscall3(SYS_read, fd,
+                                           thread_locals->scratch_buf, count);
+          copy_output_buffer(ret, NULL, buf, thread_locals->scratch_buf);
         }
         // Do this now before we finish processing the syscallbuf record.
         // This means the syscall will be executed in
@@ -2128,7 +2145,7 @@ static long sys_recvmsg(const struct syscall_info* call) {
       /* When we reach a safe point, notify rr that the control message was
        * received.
        */
-      notify_control_msg = msg;
+      thread_locals->notify_control_msg = msg;
     }
   } else {
     /* Allocate record space as least to cover the data we overwrote above.
@@ -2302,6 +2319,75 @@ static long sys_getrusage(const struct syscall_info* call) {
   return commit_raw_syscall(syscallno, ptr, ret);
 }
 
+// The alignment of this struct is incorrect, but as long as it's not
+// used inside other structures, defining it this way makes the code below
+// easier.
+typedef uint64_t kernel_sigset_t;
+
+static long sys_rt_sigprocmask(const struct syscall_info* call) {
+  const int syscallno = SYS_rt_sigprocmask;
+  long ret;
+  kernel_sigset_t modified_set;
+  void* oldset2 = NULL;
+  struct syscallbuf_hdr* hdr;
+  kernel_sigset_t previous_set;
+
+  void* ptr = prep_syscall();
+
+  int how = (int)call->args[0];
+  const kernel_sigset_t* set = (const kernel_sigset_t*)call->args[1];
+  kernel_sigset_t* oldset = (kernel_sigset_t*)call->args[2];
+
+  if (oldset) {
+    oldset2 = ptr;
+    ptr += sizeof(kernel_sigset_t);
+  }
+  if (!start_commit_buffered_syscall(syscallno, ptr, WONT_BLOCK)) {
+    return traced_raw_syscall(call);
+  }
+
+  if (set && (how == SIG_BLOCK || how == SIG_SETMASK)) {
+    local_memcpy(&modified_set, set, sizeof(kernel_sigset_t));
+    // SIGSTKFLT (PerfCounters::TIME_SLICE_SIGNAL) and
+    // SIGPWR(SYSCALLBUF_DESCHED_SIGNAL) are used by rr
+    modified_set &=
+        ~(((uint64_t)1) << (SIGSTKFLT - 1)) & ~(((uint64_t)1) << (SIGPWR - 1));
+    set = &modified_set;
+  }
+
+  hdr = buffer_hdr();
+  previous_set = hdr->blocked_sigs;
+
+  /* Update |blocked_sigs| now so that if the syscall succeeds and a blocked
+   * signal is raised before it returns, rr sees the updated signal mask.
+   */
+  if (set) {
+    switch (how) {
+      case SIG_UNBLOCK:
+        hdr->blocked_sigs &= ~*set;
+        break;
+      case SIG_BLOCK:
+        hdr->blocked_sigs |= *set;
+        break;
+      case SIG_SETMASK:
+        hdr->blocked_sigs = *set;
+        break;
+    }
+  }
+
+  ret = untraced_syscall4(syscallno, how, set, oldset2, call->args[3]);
+  if (ret == 0) {
+    if (oldset2) {
+      local_memcpy(oldset, oldset2, sizeof(kernel_sigset_t));
+    }
+  } else {
+    /* Ssyscall failed; don't update |blocked_sigs|! */
+    hdr->blocked_sigs = previous_set;
+  }
+
+  return commit_raw_syscall(syscallno, ptr, ret);
+}
+
 static long syscall_hook_internal(const struct syscall_info* call) {
   switch (call->no) {
 #define CASE(syscallname)                                                      \
@@ -2324,6 +2410,7 @@ static long syscall_hook_internal(const struct syscall_info* call) {
 #else
     CASE(fcntl);
 #endif
+    CASE(rt_sigprocmask);
     CASE(fgetxattr);
     CASE(flistxattr);
     CASE_GENERIC_NONBLOCKING_FD(fsetxattr);
@@ -2400,9 +2487,10 @@ static long syscall_hook_internal(const struct syscall_info* call) {
  * _syscall_hook_trampoline without doing all sorts of special PIC handling.
  */
 RR_HIDDEN long syscall_hook(const struct syscall_info* call) {
-  if (!globals.thread_locals_initialized) {
-    return traced_raw_syscall(call);
-  }
+  // Initialize thread-local state if this is the first syscall for this
+  // thread.
+  init_thread();
+
   long result = syscall_hook_internal(call);
   if (buffer_hdr() && buffer_hdr()->notify_on_syscall_hook_exit) {
     // SYS_rrcall_notify_syscall_hook_exit will clear
@@ -2444,10 +2532,10 @@ RR_HIDDEN long syscall_hook(const struct syscall_info* call) {
                           RR_PAGE_SYSCALL_PRIVILEGED_TRACED, result, call->no);
   }
   // Do work that can only be safely done after syscallbuf can be flushed
-  if (notify_control_msg) {
+  if (thread_locals->notify_control_msg) {
     privileged_traced_syscall1(SYS_rrcall_notify_control_msg,
-                               notify_control_msg);
-    notify_control_msg = NULL;
+                               thread_locals->notify_control_msg);
+    thread_locals->notify_control_msg = NULL;
   }
   return result;
 }
