@@ -6,6 +6,8 @@
 #include <signal.h>
 #include <stdint.h>
 #include <string.h>
+#include <sys/types.h>
+#include <sys/user.h>
 
 #ifndef RR_IMPLEMENT_PRELOAD
 #include "../remote_ptr.h"
@@ -59,6 +61,9 @@
   RR_PAGE_SYSCALL_ADDR(7)
 #define RR_PAGE_FF_BYTES (RR_PAGE_ADDR + RR_PAGE_SYSCALL_STUB_SIZE * 8)
 
+#define PRELOAD_THREAD_LOCALS_ADDR (RR_PAGE_ADDR + PAGE_SIZE)
+#define PRELOAD_THREAD_LOCALS_SIZE 64
+
 /* "Magic" (rr-implemented) syscalls that we use to initialize the
  * syscallbuf.
  *
@@ -160,8 +165,9 @@ struct preload_globals {
    * USE WITH CAUTION.
    */
   unsigned char in_replay;
-  /* 1 when thread-locals have been initialized for this task, 0 otherwise */
-  unsigned char thread_locals_initialized;
+  /* 0 during recording and replay, 1 during diversion. Set by rr.
+   */
+  unsigned char in_diversion;
   /* Number of cores to pretend we have. 0 means 1. rr sets this when
    * the preload library is initialized. */
   int pretend_num_cores;
@@ -293,6 +299,13 @@ struct syscallbuf_hdr {
    * When it's zero, the desched signal can safely be
    * discarded. */
   uint8_t desched_signal_may_be_relevant;
+  /* A copy of the tasks's signal mask. Updated by both preload and rr in
+   * response to events that change the signal mask. RR stores this in
+   * RecordTask::blocked_sigs, but since some signal mask affecting system
+   * calls may be buffered, that value may be out of date until rr reloads it
+   * from here.
+   */
+  uint64_t blocked_sigs;
 
   struct syscallbuf_record recs[0];
 } __attribute__((__packed__));
@@ -336,6 +349,10 @@ inline static int is_blacklisted_filename(const char* filename) {
          !strcmp("/usr/share/alsa/alsa.conf", filename);
 }
 
+inline static int is_gcrypt_deny_file(const char* filename) {
+  return !strcmp("/etc/gcrypt/hwf.deny", filename);
+}
+
 inline static int is_terminal(const char* filename) {
   return !strncmp("/dev/tty", filename, 8) || !strncmp("/dev/pts", filename, 8);
 }
@@ -347,6 +364,20 @@ inline static int is_proc_mem_file(const char* filename) {
   return !strcmp(filename + strlen(filename) - 4, "/mem");
 }
 
+inline static int is_proc_fd_dir(const char* filename) {
+  if (strncmp("/proc/", filename, 6)) {
+    return 0;
+  }
+
+  int len = strlen(filename);
+  const char* fd_bit = filename + len;
+  if (*fd_bit == '/') {
+    fd_bit--;
+  }
+
+  return !strncmp(fd_bit - 3, "/fd", 3);
+}
+
 /**
  * Returns nonzero if an attempted open() of |filename| can be syscall-buffered.
  * When this returns zero, the open must be forwarded to the rr process.
@@ -356,8 +387,9 @@ inline static int is_proc_mem_file(const char* filename) {
  * common cases) is a problem. Maybe we could afford fstat after every open...
  */
 inline static int allow_buffered_open(const char* filename) {
-  return !is_blacklisted_filename(filename) && !is_terminal(filename) &&
-         !is_proc_mem_file(filename);
+  return !is_blacklisted_filename(filename) && !is_gcrypt_deny_file(filename) &&
+         !is_terminal(filename) && !is_proc_mem_file(filename) &&
+         !is_proc_fd_dir(filename);
 }
 
 #endif /* RR_PRELOAD_INTERFACE_H_ */
