@@ -34,7 +34,7 @@ namespace rr {
 static __inline__ unsigned long long rdtsc(void) { return __rdtsc(); }
 
 template <typename Arch> static size_t sigaction_sigset_size_arch() {
-  return Arch::sigaction_sigset_size;
+  return sizeof(typename Arch::kernel_sigset_t);
 }
 
 static size_t sigaction_sigset_size(SupportedArch arch) {
@@ -42,23 +42,31 @@ static size_t sigaction_sigset_size(SupportedArch arch) {
 }
 
 /**
- * Restore the blocked-ness and sigaction for SIGSEGV from |t|'s local
+ * Restore the blocked-ness and sigaction for |sig| from |t|'s local
  * copy.
  */
-static void restore_sigsegv_state(RecordTask* t) {
-  const vector<uint8_t>& sa = t->signal_action(SIGSEGV);
+static void restore_signal_state(RecordTask* t, int sig) {
   AutoRemoteSyscalls remote(t);
+  size_t sigset_size = sigaction_sigset_size(remote.arch());
   {
+    const vector<uint8_t>& sa = t->signal_action(sig);
     AutoRestoreMem child_sa(remote, sa.data(), sa.size());
     remote.infallible_syscall(syscall_number_for_rt_sigaction(remote.arch()),
-                              SIGSEGV, child_sa.get().as_int(), nullptr,
-                              sigaction_sigset_size(remote.arch()));
+                              sig, child_sa.get().as_int(), nullptr,
+                              sigset_size);
+  }
+  if (t->is_sig_blocked(sig)) {
+    vector<uint8_t> bytes;
+    bytes.resize(sigset_size);
+    memset(bytes.data(), 0, sigset_size);
+    uint64_t mask = uint64_t(1) << (sig - 1);
+    ASSERT(t, sigset_size >= sizeof(mask));
+    memcpy(bytes.data(), &mask, sizeof(mask));
+    AutoRestoreMem child_block(remote, bytes.data(), bytes.size());
+    remote.infallible_syscall(syscall_number_for_rt_sigprocmask(remote.arch()),
+                              SIG_BLOCK, child_block.get().as_int(), nullptr,
+                              sigset_size);
   }
-  // NB: we would normally want to restore the SIG_BLOCK for
-  // SIGSEGV here, but doing so doesn't change the kernel's
-  // "SigBlk" mask.  There's no bug observed in the kernel's
-  // delivery of SIGSEGV after the RDTSC trap, so we do nothing
-  // here and move on.
 }
 
 /** Return true iff |t->ip()| points at a RDTSC instruction. */
@@ -163,7 +171,7 @@ static bool try_grow_map(RecordTask* t, siginfo_t* si) {
   }
 
   KernelMapping km =
-      t->vm()->map(new_start, it->map.start() - new_start, it->map.prot(),
+      t->vm()->map(t, new_start, it->map.start() - new_start, it->map.prot(),
                    it->map.flags() | MAP_ANONYMOUS, 0, string(),
                    KernelMapping::NO_DEVICE, KernelMapping::NO_INODE);
   t->trace_writer().write_mapped_region(t, km, km.fake_stat());
@@ -197,8 +205,7 @@ void arm_desched_event(RecordTask* t) {
 static void handle_desched_event(RecordTask* t, const siginfo_t* si) {
   ASSERT(t, (SYSCALLBUF_DESCHED_SIGNAL == si->si_signo &&
              si->si_code == POLL_IN && si->si_fd == t->desched_fd_child))
-      << "Tracee is using SIGPWR??? (code=" << si->si_code
-      << ", fd=" << si->si_fd << ")";
+      << "Tracee is using SIGPWR??? (siginfo=" << *si << ")";
 
   /* If the tracee isn't in the critical section where a desched
    * event is relevant, we can ignore it.  See the long comments
@@ -318,7 +325,7 @@ static void handle_desched_event(RecordTask* t, const siginfo_t* si) {
     // happen an unbounded number of consecutive times
     // and the tracee never switched out.
     int sig = t->stop_sig();
-    ASSERT(t, sig);
+    ASSERT(t, sig) << "expected stop-signal, got " << t->status();
     if (SYSCALLBUF_DESCHED_SIGNAL == sig ||
         PerfCounters::TIME_SLICE_SIGNAL == sig || t->is_sig_ignored(sig)) {
       LOG(debug) << "  dropping ignored " << signal_name(sig);
@@ -384,14 +391,6 @@ static void handle_desched_event(RecordTask* t, const siginfo_t* si) {
 }
 
 static bool is_safe_to_deliver_signal(RecordTask* t) {
-  if (!t->syscallbuf_child) {
-    /* Can't be in critical section because the lock
-     * doesn't exist yet! */
-    LOG(debug) << "Safe to deliver signal at " << t->ip()
-               << " because no syscallbuf_child";
-    return true;
-  }
-
   if (!t->is_in_syscallbuf()) {
     /* The tracee is outside the syscallbuf code,
      * so in most cases can't possibly affect
@@ -417,10 +416,16 @@ static bool is_safe_to_deliver_signal(RecordTask* t) {
     return true;
   }
 
-  // Our emulation of SYS_rrcall_notify_syscall_hook_exit clears this flag.
-  t->write_mem(
-      REMOTE_PTR_FIELD(t->syscallbuf_child, notify_on_syscall_hook_exit),
-      (uint8_t)1);
+  // If the syscallbuf buffer hasn't been created yet, just delay the signal
+  // with no need to set notify_on_syscall_hook_exit; the signal will be
+  // delivered when rrcall_init_buffers is called.
+  if (t->syscallbuf_child) {
+    // Our emulation of SYS_rrcall_notify_syscall_hook_exit clears this flag.
+    t->write_mem(
+        REMOTE_PTR_FIELD(t->syscallbuf_child, notify_on_syscall_hook_exit),
+        (uint8_t)1);
+  }
+
   LOG(debug) << "Not safe to deliver signal at " << t->ip();
   return false;
 }
@@ -444,32 +449,28 @@ SignalHandled handle_signal(RecordTask* t, siginfo_t* si,
     return DEFER_SIGNAL;
   }
 
-  t->set_siginfo_for_synthetic_SIGCHLD(si);
+  if (!t->set_siginfo_for_synthetic_SIGCHLD(si)) {
+    return DEFER_SIGNAL;
+  }
+
+  int sig = si->si_signo;
+  // When a deterministic signal is triggered, but the signal is currently
+  // blocked or ignored, the kernel (in |force_sig_info|) unblocks it and
+  // sets its disposition to SIG_DFL. It never undoes this (probably
+  // because it expects the signal to be fatal, which it always would be
+  // unless a ptracer intercepts the signal as we do). Therefore, if the signal
+  // was generated for rr's purposes, we need to restore the signal state
+  // ourselves. If the signal was not generated for rr's purposes, we'll
+  // modify our copy of the signal state to match what the kernel did.
 
   /* See if this signal occurred because of an rr implementation detail,
    * and fudge t appropriately. */
-  switch (si->si_signo) {
+  switch (sig) {
     case SIGSEGV:
-      if (try_handle_rdtsc(t, si) || try_grow_map(t, si)) {
-        // When SIGSEGV is blocked, apparently the kernel has to do
-        // some ninjutsu to raise the trap.  We see the SIGSEGV
-        // bit in the "SigBlk" mask in /proc/status cleared, and if
-        // there's a user handler the SIGSEGV bit in "SigCgt" is
-        // cleared too.  That's perfectly fine, except that it's
-        // unclear who's supposed to undo the signal-state munging.  A
-        // legitimate argument can be made that the tracer is
-        // responsible, so we go ahead and restore the old state.
-        //
-        // One could also argue that this is a kernel bug.  If so,
-        // then this is a workaround that can be removed in the
-        // future.
-        //
-        // If we don't restore the old state, at least firefox has
-        // been observed to hang at delivery of SIGSEGV.  However, the
-        // test written for this bug, fault_in_code_addr, doesn't hang
-        // without the restore.
-        if (t->is_sig_blocked(SIGSEGV)) {
-          restore_sigsegv_state(t);
+      if (deterministic == DETERMINISTIC_SIG &&
+          (try_handle_rdtsc(t, si) || try_grow_map(t, si))) {
+        if (t->is_sig_blocked(sig) || t->is_sig_ignored(sig)) {
+          restore_signal_state(t, sig);
         }
         return SIGNAL_HANDLED;
       }
@@ -480,10 +481,16 @@ SignalHandled handle_signal(RecordTask* t, siginfo_t* si,
       return SIGNAL_HANDLED;
   }
 
+  if (deterministic == DETERMINISTIC_SIG &&
+      (t->is_sig_blocked(sig) || t->is_sig_ignored(sig))) {
+    t->set_sig_blocked(sig, false);
+    t->set_sig_handler_default(sig);
+  }
+
   /* This signal was generated by the program or an external
    * source, record it normally. */
 
-  if (t->emulate_ptrace_stop(WaitStatus::for_stop_sig(si->si_signo), si)) {
+  if (t->emulate_ptrace_stop(WaitStatus::for_stop_sig(sig), si)) {
     // Record an event so that replay progresses the tracee to the
     // current point before we notify the tracer.
     // If the signal is deterministic, record it as an EV_SIGNAL so that
