@@ -29,10 +29,13 @@
 
 #include "AutoRemoteSyscalls.h"
 #include "EmuFs.h"
+#include "MmappedFileMonitor.h"
+#include "ProcFdDirMonitor.h"
 #include "ProcMemMonitor.h"
 #include "ReplaySession.h"
 #include "ReplayTask.h"
 #include "StdioMonitor.h"
+#include "TaskGroup.h"
 #include "TraceStream.h"
 #include "VirtualPerfCounterMonitor.h"
 #include "kernel_abi.h"
@@ -75,9 +78,19 @@ static string maybe_dump_written_string(ReplayTask* t) {
  * Proceeds until the next system call, which is being executed.
  */
 static void __ptrace_cont(ReplayTask* t, ResumeRequest resume_how,
-                          int expect_syscallno, int expect_syscallno2 = -1) {
+                          int expect_syscallno, int expect_syscallno2 = -1,
+                          pid_t new_tid = -1) {
   do {
-    t->resume_execution(resume_how, RESUME_WAIT, RESUME_NO_TICKS);
+    t->resume_execution(resume_how, RESUME_NONBLOCKING, RESUME_NO_TICKS);
+    if (new_tid > 0) {
+      // Set new tid before we wait, so we wait for the right task.
+      // XXX maybe a signal could arrive and be delivered to the task before
+      // its tid changes, so we hang? Hope not!
+      // Update the serial as if this task was really created by cloning the old
+      // task.
+      t->set_real_tid_and_update_serial(new_tid);
+    }
+    t->wait();
   } while (ReplaySession::is_ignored_signal(t->status().stop_sig()));
 
   ASSERT(t, !t->stop_sig()) << "Expected no pending signal, but got "
@@ -108,8 +121,8 @@ static void init_scratch_memory(ReplayTask* t, const KernelMapping& km,
   AutoRemoteSyscalls remote(t);
   remote.infallible_mmap_syscall(t->scratch_ptr, sz, km.prot(),
                                  km.flags() | MAP_FIXED, -1, 0);
-  t->vm()->map(t->scratch_ptr, sz, km.prot(), km.flags(), 0, string(),
-               KernelMapping::NO_DEVICE, KernelMapping::NO_INODE, &km);
+  t->vm()->map(t, t->scratch_ptr, sz, km.prot(), km.flags(), 0, string(),
+               KernelMapping::NO_DEVICE, KernelMapping::NO_INODE, nullptr, &km);
 }
 
 /**
@@ -164,7 +177,13 @@ template <typename Arch> static void prepare_clone(ReplayTask* t) {
     // and we can't allow that.
     // Block CLONE_CHILD_CLEARTID because we'll emulate that ourselves.
     // Block CLONE_VFORK for the reasons below.
-    flags = r.arg1() & ~(CLONE_UNTRACED | CLONE_CHILD_CLEARTID | CLONE_VFORK);
+    // Block CLONE_NEW* from replay, any effects it had were dealt with during
+    // recording.
+    uintptr_t disallowed_clone_flags =
+        CLONE_UNTRACED | CLONE_CHILD_CLEARTID | CLONE_VFORK | CLONE_NEWIPC |
+        CLONE_NEWNET | CLONE_NEWNS | CLONE_NEWPID | CLONE_NEWUSER |
+        CLONE_NEWUTS | CLONE_NEWCGROUP;
+    flags = r.arg1() & ~disallowed_clone_flags;
     r.set_arg1(flags);
   } else if (Arch::vfork == sys) {
     // We can't perform a real vfork, because the kernel won't let the vfork
@@ -221,9 +240,7 @@ template <typename Arch> static void prepare_clone(ReplayTask* t) {
   // Dig the recorded tid out out of the trace. The tid value returned in
   // the recorded registers could be in a different pid namespace from rr's,
   // so we can't use it directly.
-  TraceTaskEvent tte = read_task_trace_event(
-      t, Arch::clone == t->regs().original_syscallno() ? TraceTaskEvent::CLONE
-                                                       : TraceTaskEvent::FORK);
+  TraceTaskEvent tte = read_task_trace_event(t, TraceTaskEvent::CLONE);
   ASSERT(t, tte.parent_tid() == t->rec_tid);
   long rec_tid = tte.tid();
   pid_t new_tid = t->get_ptrace_eventmsg<pid_t>();
@@ -267,7 +284,7 @@ template <typename Arch> static void prepare_clone(ReplayTask* t) {
     new_task->vm()->remove_all_watchpoints();
 
     AutoRemoteSyscalls remote(new_task);
-    for (auto m : new_task->vm()->maps()) {
+    for (auto m : t->vm()->maps()) {
       // Recreate any tracee-shared mappings
       if (m.local_addr && !(m.flags & AddressSpace::Mapping::IS_SYSCALLBUF)) {
         memcpy(Session::recreate_shared_mmap(remote, m).local_addr,
@@ -372,8 +389,8 @@ static void restore_mapped_region(ReplayTask* t, AutoRemoteSyscalls& remote,
       break;
   }
 
-  t->vm()->map(km.start(), km.size(), km.prot(), flags, offset_bytes,
-               real_file_name, device, inode, &km);
+  t->vm()->map(t, km.start(), km.size(), km.prot(), flags, offset_bytes,
+               real_file_name, device, inode, nullptr, &km);
 }
 
 static void process_execve(ReplayTask* t, const TraceFrame& trace_frame,
@@ -383,11 +400,33 @@ static void process_execve(ReplayTask* t, const TraceFrame& trace_frame,
   /* First, exec a stub program */
   string stub_filename = find_exec_stub(trace_frame.regs().arch());
 
-  // Setup memory and registers for the execve call. We don't need to save
-  // the old values since they're going to be wiped out by execve.
+  // Setup memory and registers for the execve call. We may not have to save
+  // the old values since they're going to be wiped out by execve. We can
+  // determine this by checking if this address space has any tasks with a
+  // different tgid.
+  Task* memory_task = t;
+  for (auto task : t->vm()->task_set()) {
+    if (task->tgid() != t->tgid()) {
+      memory_task = task;
+      break;
+    }
+  }
+
+  // Old data if required
+  std::vector<uint8_t> saved_data;
+
+  // Set up everything
   Registers regs = t->regs();
   regs.set_ip(t->vm()->traced_syscall_ip());
   remote_ptr<void> remote_mem = floor_page_size(regs.sp());
+
+  // Determine how much memory we'll need
+  size_t filename_size = stub_filename.size() + 1;
+  size_t total_size = filename_size + sizeof(size_t);
+  if (memory_task != t) {
+    saved_data = t->read_mem(remote_mem.cast<uint8_t>(), total_size);
+  }
+
   // We write a zero word in the host size, not t's size, but that's OK,
   // since the host size must be bigger than t's size.
   // We pass no argv or envp, so exec params 2 and 3 just point to the NULL
@@ -395,10 +434,9 @@ static void process_execve(ReplayTask* t, const TraceFrame& trace_frame,
   t->write_mem(remote_mem.cast<size_t>(), size_t(0));
   regs.set_arg2(remote_mem);
   regs.set_arg3(remote_mem);
-  remote_mem += sizeof(size_t);
-  t->write_bytes_helper(remote_mem, stub_filename.size() + 1,
-                        stub_filename.c_str());
-  regs.set_arg1(remote_mem);
+  remote_ptr<void> filename_addr = remote_mem + sizeof(size_t);
+  t->write_bytes_helper(filename_addr, filename_size, stub_filename.c_str());
+  regs.set_arg1(filename_addr);
   /* The original_syscallno is execve in the old architecture. The kernel does
    * not update the original_syscallno when the architecture changes across
    * an exec.
@@ -407,12 +445,17 @@ static void process_execve(ReplayTask* t, const TraceFrame& trace_frame,
   regs.set_syscallno(expect_syscallno);
   t->set_regs(regs);
 
+  LOG(debug) << "Beginning execve";
   /* Enter our execve syscall. */
   __ptrace_cont(t, RESUME_SYSCALL, expect_syscallno);
   ASSERT(t, !t->stop_sig()) << "Stub exec failed on entry";
-  /* Complete the syscall */
+  /* Complete the syscall. The tid of the task will be the thread-group-leader
+   * tid, no matter what tid it was before.
+   */
+  pid_t tgid = t->task_group()->real_tgid;
   __ptrace_cont(t, RESUME_SYSCALL, expect_syscallno,
-                syscall_number_for_execve(trace_frame.regs().arch()));
+                syscall_number_for_execve(trace_frame.regs().arch()),
+                tgid == t->tid ? -1 : tgid);
   if (t->regs().syscall_result()) {
     errno = -t->regs().syscall_result();
     if (access(stub_filename.c_str(), 0) == -1 && errno == ENOENT &&
@@ -434,7 +477,8 @@ static void process_execve(ReplayTask* t, const TraceFrame& trace_frame,
     if (!found) {
       break;
     }
-    if (km.start() == AddressSpace::rr_page_start()) {
+    if (km.start() == AddressSpace::rr_page_start() ||
+        km.start() == AddressSpace::preload_thread_locals_start()) {
       // Skip rr-page mapping record, that gets mapped automatically
       continue;
     }
@@ -472,6 +516,7 @@ static void process_execve(ReplayTask* t, const TraceFrame& trace_frame,
     for (auto m : t->vm()->maps()) {
       // Do not attempt to unmap [vsyscall] --- it doesn't work.
       if (m.map.start() != AddressSpace::rr_page_start() &&
+          m.map.start() != AddressSpace::preload_thread_locals_start() &&
           !m.map.is_vsyscall()) {
         unmaps.push_back(m.map);
       }
@@ -479,7 +524,7 @@ static void process_execve(ReplayTask* t, const TraceFrame& trace_frame,
     for (auto& m : unmaps) {
       remote.infallible_syscall(syscall_number_for_munmap(t->arch()), m.start(),
                                 m.size());
-      t->vm()->unmap(m.start(), m.size());
+      t->vm()->unmap(t, m.start(), m.size());
     }
     // We will have unmapped the stack memory that |remote| would have used for
     // memory parameters. Fortunately process_mapped_region below doesn't
@@ -519,6 +564,13 @@ static void process_execve(ReplayTask* t, const TraceFrame& trace_frame,
   // Now it's safe to save the auxv data
   t->vm()->save_auxv(t);
 
+  // Restore any memory if required. We need to do this through memory_task,
+  // since the new task is now on the new address space.
+  if (memory_task != t) {
+    memory_task->write_mem(remote_mem.cast<uint8_t>(), saved_data.data(),
+                           saved_data.size());
+  }
+
   // Notify outer rr if there is one
   syscall(SYS_rrcall_reload_auxv, t->tid);
 }
@@ -533,14 +585,15 @@ static void process_brk(ReplayTask* t) {
     remote.infallible_mmap_syscall(km.start(), km.size(), km.prot(),
                                    MAP_ANONYMOUS | MAP_FIXED | km.flags(), -1,
                                    0);
-    t->vm()->map(km.start(), km.size(), km.prot(), MAP_ANONYMOUS | km.flags(),
-                 0, "[heap]", KernelMapping::NO_DEVICE, KernelMapping::NO_INODE,
+    t->vm()->map(t, km.start(), km.size(), km.prot(),
+                 MAP_ANONYMOUS | km.flags(), 0, "[heap]",
+                 KernelMapping::NO_DEVICE, KernelMapping::NO_INODE, nullptr,
                  &km);
   } else if (km.size() > 0) {
     AutoRemoteSyscalls remote(t);
     remote.infallible_syscall(syscall_number_for_munmap(t->arch()), km.start(),
                               km.size());
-    t->vm()->unmap(km.start(), km.size());
+    t->vm()->unmap(t, km.start(), km.size());
   }
 }
 
@@ -551,14 +604,10 @@ static void process_brk(ReplayTask* t) {
  */
 enum NoteTaskMap { DONT_NOTE_TASK_MAP = 0, NOTE_TASK_MAP };
 
-static remote_ptr<void> finish_anonymous_mmap(
-    ReplayTask* t, AutoRemoteSyscalls& remote, const TraceFrame& trace_frame,
-    size_t length, int prot, int flags, NoteTaskMap note_task_map) {
-  const Registers& rec_regs = trace_frame.regs();
-  /* *Must* map the segment at the recorded address, regardless
-     of what the recorded tracee passed as the |addr| hint. */
-  remote_ptr<void> rec_addr = rec_regs.syscall_result();
-
+static void finish_anonymous_mmap(ReplayTask* t, AutoRemoteSyscalls& remote,
+                                  remote_ptr<void> rec_addr, size_t length,
+                                  int prot, int flags,
+                                  NoteTaskMap note_task_map) {
   string file_name;
   dev_t device = KernelMapping::NO_DEVICE;
   ino_t inode = KernelMapping::NO_INODE;
@@ -584,10 +633,9 @@ static remote_ptr<void> finish_anonymous_mmap(
   }
 
   if (note_task_map) {
-    remote.task()->vm()->map(rec_addr, length, prot, flags, 0, file_name,
-                             device, inode, &recorded_km, emu_file);
+    remote.task()->vm()->map(t, rec_addr, length, prot, flags, 0, file_name,
+                             device, inode, nullptr, &recorded_km, emu_file);
   }
-  return rec_addr;
 }
 
 /* Ensure that accesses to the memory region given by start/length
@@ -631,40 +679,41 @@ static void create_sigbus_region(AutoRemoteSyscalls& remote, int prot,
   remote.infallible_syscall(syscall_number_for_close(remote.arch()), child_fd);
 
   KernelMapping km_slice = km.subrange(start, start + length);
-  remote.task()->vm()->map(start, length, prot, MAP_FIXED | MAP_PRIVATE, 0,
-                           file_name, fstat.st_dev, fstat.st_ino, &km_slice);
+  remote.task()->vm()->map(remote.task(), start, length, prot,
+                           MAP_FIXED | MAP_PRIVATE, 0, file_name, fstat.st_dev,
+                           fstat.st_ino, nullptr, &km_slice);
 }
 
 static void finish_private_mmap(ReplayTask* t, AutoRemoteSyscalls& remote,
-                                const TraceFrame& trace_frame, size_t length,
+                                remote_ptr<void> rec_addr, size_t length,
                                 int prot, int flags, off64_t offset_pages,
                                 const KernelMapping& km) {
   LOG(debug) << "  finishing private mmap of " << km.fsname();
 
-  size_t num_bytes = length;
-  remote_ptr<void> mapped_addr =
-      finish_anonymous_mmap(t, remote, trace_frame, length, prot,
-                            /* The restored region won't be backed
-                             * by file. */
-                            flags | MAP_ANONYMOUS, DONT_NOTE_TASK_MAP);
+  remote.infallible_mmap_syscall(
+      rec_addr, length, prot,
+      // Tell the kernel to take |rec_addr| seriously.
+      (flags & ~MAP_GROWSDOWN) | MAP_FIXED | MAP_ANONYMOUS, -1, 0);
+
   /* Restore the map region we copied. */
   ssize_t data_size = t->set_data_from_trace();
 
   /* Ensure pages past the end of the file fault on access */
   size_t data_pages = ceil_page_size(data_size);
-  size_t mapped_pages = ceil_page_size(num_bytes);
+  size_t mapped_pages = ceil_page_size(length);
 
-  t->vm()->map(mapped_addr, num_bytes, prot, flags | MAP_ANONYMOUS,
+  t->vm()->map(t, rec_addr, length, prot, flags | MAP_ANONYMOUS,
                page_size() * offset_pages, string(), KernelMapping::NO_DEVICE,
-               KernelMapping::NO_INODE, &km);
+               KernelMapping::NO_INODE, nullptr, &km);
 
-  create_sigbus_region(remote, prot, mapped_addr + data_pages,
+  create_sigbus_region(remote, prot, rec_addr + data_pages,
                        mapped_pages - data_pages, km);
 }
 
 static void finish_shared_mmap(ReplayTask* t, AutoRemoteSyscalls& remote,
-                               int prot, int flags, off64_t offset_pages,
-                               uint64_t file_size, const KernelMapping& km) {
+                               int prot, int flags, int fd,
+                               off64_t offset_pages, uint64_t file_size,
+                               const KernelMapping& km) {
   auto buf = t->trace_reader().read_raw_data();
   size_t rec_num_bytes = ceil_page_size(buf.data.size());
 
@@ -689,18 +738,27 @@ static void finish_shared_mmap(ReplayTask* t, AutoRemoteSyscalls& remote,
   // For full generality, we also need to emulate direct file
   // modifications through write/splice/etc.
   uint64_t offset_bytes = page_size() * offset_pages;
-  t->vm()->map(buf.addr, buf.data.size(), prot, flags, offset_bytes,
-               real_file_name, real_file.st_dev, real_file.st_ino, &km,
+  t->vm()->map(t, buf.addr, buf.data.size(), prot, flags, offset_bytes,
+               real_file_name, real_file.st_dev, real_file.st_ino, nullptr, &km,
                emufile);
 
   t->write_bytes_helper(buf.addr, buf.data.size(), buf.data.data());
   LOG(debug) << "  restored " << buf.data.size() << " bytes at "
              << HEX(offset_bytes) << " to " << emufile->real_path() << " for "
              << emufile->emu_path();
+
+  if (fd >= 0) {
+    if (t->fd_table()->is_monitoring(fd)) {
+      ASSERT(t, t->fd_table()->get_monitor(fd)->type() ==
+                    FileMonitor::Type::Mmapped);
+    } else {
+      t->fd_table()->add_monitor(fd, new MmappedFileMonitor(t, emufile));
+    }
+  }
 }
 
 static void process_mmap(ReplayTask* t, const TraceFrame& trace_frame,
-                         size_t length, int prot, int flags,
+                         size_t length, int prot, int flags, int fd,
                          off64_t offset_pages, ReplayTraceStep* step) {
   step->action = TSTEP_RETIRE;
 
@@ -713,8 +771,8 @@ static void process_mmap(ReplayTask* t, const TraceFrame& trace_frame,
                                   ? AutoRemoteSyscalls::DISABLE_MEMORY_PARAMS
                                   : AutoRemoteSyscalls::ENABLE_MEMORY_PARAMS);
     if (flags & MAP_ANONYMOUS) {
-      finish_anonymous_mmap(t, remote, trace_frame, length, prot, flags,
-                            NOTE_TASK_MAP);
+      finish_anonymous_mmap(t, remote, trace_frame.regs().syscall_result(),
+                            length, prot, flags, NOTE_TASK_MAP);
     } else {
       TraceReader::MappedData data;
       KernelMapping km = t->trace_reader().read_mapped_region(&data);
@@ -726,16 +784,16 @@ static void process_mmap(ReplayTask* t, const TraceFrame& trace_frame,
                            length, prot, flags, data.file_name, O_RDONLY,
                            data.data_offset_bytes / page_size(), real_file,
                            real_file_name);
-        t->vm()->map(km.start(), length, prot, flags,
+        t->vm()->map(t, km.start(), length, prot, flags,
                      page_size() * offset_pages, real_file_name,
-                     real_file.st_dev, real_file.st_ino, &km);
+                     real_file.st_dev, real_file.st_ino, nullptr, &km);
       } else {
         ASSERT(t, data.source == TraceReader::SOURCE_TRACE);
         if (MAP_PRIVATE & flags) {
-          finish_private_mmap(t, remote, trace_frame, length, prot, flags,
-                              offset_pages, km);
+          finish_private_mmap(t, remote, trace_frame.regs().syscall_result(),
+                              length, prot, flags, offset_pages, km);
         } else {
-          finish_shared_mmap(t, remote, prot, flags, offset_pages,
+          finish_shared_mmap(t, remote, prot, flags, fd, offset_pages,
                              data.file_size_bytes, km);
         }
       }
@@ -759,6 +817,68 @@ static void process_mmap(ReplayTask* t, const TraceFrame& trace_frame,
   t->validate_regs();
 }
 
+static void process_mremap(ReplayTask* t, const TraceFrame& trace_frame,
+                           ReplayTraceStep* step) {
+  step->action = TSTEP_RETIRE;
+
+  auto& trace_regs = trace_frame.regs();
+  remote_ptr<void> old_addr = trace_frame.regs().arg1();
+  size_t old_size = ceil_page_size(trace_regs.arg2());
+  remote_ptr<void> new_addr = trace_frame.regs().syscall_result();
+  size_t new_size = ceil_page_size(trace_regs.arg3());
+
+  {
+    // We must emulate mremap because the kernel's choice for the remap
+    // destination can vary (in particular, when we emulate exec it makes
+    // different decisions).
+    AutoRemoteSyscalls remote(t);
+    if (new_addr == old_addr) {
+      // Non-moving mremap. Don't pass MREMAP_FIXED or MREMAP_MAYMOVE
+      // since that triggers EINVAL when the new map overlaps the old map.
+      remote.infallible_syscall_ptr(trace_regs.original_syscallno(), new_addr,
+                                    old_size, new_size, 0);
+    } else {
+      // Force the mremap to use the destination address from recording.
+      // XXX could the new mapping overlap the old, with different start
+      // addresses? Hopefully the kernel doesn't do that to us!!!
+      remote.infallible_syscall_ptr(trace_regs.original_syscallno(), old_addr,
+                                    old_size, new_size,
+                                    MREMAP_MAYMOVE | MREMAP_FIXED, new_addr);
+    }
+
+    remote.regs().set_syscall_result(new_addr);
+  }
+
+  t->vm()->remap(t, old_addr, old_size, new_addr, new_size);
+
+  TraceReader::MappedData data;
+  t->trace_reader().read_mapped_region(&data);
+  ASSERT(t, data.source == TraceReader::SOURCE_ZERO);
+  // We don't need to do anything; this is the mapping record for the moved
+  // data.
+
+  // Try reading a mapping record for new data.
+  bool found;
+  t->trace_reader().read_mapped_region(&data, &found);
+  // SOURCE_FILE should be handled automatically by the mremap above.
+  // (If we started storing partial files, we'd have to careful to ensure this
+  // is still the case.)
+  if (found && data.source == TraceReader::SOURCE_TRACE) {
+    TraceReader::RawData buf;
+    if (t->trace_reader().read_raw_data_for_frame(trace_frame, buf)) {
+      auto& mapping = t->vm()->mapping_of(new_addr);
+      auto f = mapping.emu_file;
+      if (f) {
+        f->ensure_size(mapping.map.file_offset_bytes() + old_size +
+                       buf.data.size());
+      }
+      t->write_bytes_helper(buf.addr, buf.data.size(), buf.data.data());
+    }
+  }
+
+  t->validate_regs();
+}
+
 void process_grow_map(ReplayTask* t) {
   AutoRemoteSyscalls remote(t);
   TraceReader::MappedData data;
@@ -777,7 +897,7 @@ static void process_shmat(ReplayTask* t, const TraceFrame& trace_frame,
     KernelMapping km = t->trace_reader().read_mapped_region(&data);
     int prot = shm_flags_to_mmap_prot(shm_flags);
     int flags = MAP_SHARED;
-    finish_shared_mmap(t, remote, prot, flags, 0, data.file_size_bytes, km);
+    finish_shared_mmap(t, remote, prot, flags, -1, 0, data.file_size_bytes, km);
 
     // Finally, we finish by emulating the return value.
     remote.regs().set_syscall_result(trace_frame.regs().syscall_result());
@@ -829,8 +949,9 @@ static void rep_after_enter_syscall_arch(ReplayTask* t) {
     case Arch::exit:
     case Arch::exit_group:
       // We don't really need to destroy buffers in exit_group since they'll
-      // go away anyway, but it makes memory consistent with recording
-      // for checksumming.
+      // go away anyway. Also, we have a fallback destroy_buffers in
+      // prepare_exit. However, doing it here makes memory consistent with
+      // recording for checksumming.
       t->destroy_buffers();
       break;
 
@@ -907,6 +1028,8 @@ static void handle_opened_files(ReplayTask* t) {
       t->fd_table()->add_monitor(fd, new StdioMonitor(STDERR_FILENO));
     } else if (is_proc_mem_file(pathname.c_str())) {
       t->fd_table()->add_monitor(fd, new ProcMemMonitor(t, pathname));
+    } else if (is_proc_fd_dir(pathname.c_str())) {
+      t->fd_table()->add_monitor(fd, new ProcFdDirMonitor(t, pathname));
     } else {
       ASSERT(t, false) << "Why did we write filename " << pathname;
     }
@@ -978,18 +1101,20 @@ static void rep_process_syscall_arch(ReplayTask* t, ReplayTraceStep* step) {
           auto args = t->read_mem(
               remote_ptr<typename Arch::mmap_args>(trace_regs.arg1()));
           return process_mmap(t, trace_frame, args.len, args.prot, args.flags,
-                              args.offset / page_size(), step);
+                              args.fd, args.offset / page_size(), step);
         }
         case Arch::RegisterArguments:
           return process_mmap(t, trace_frame, trace_regs.arg2(),
                               trace_regs.arg3(), trace_regs.arg4(),
+                              trace_regs.arg5(),
                               trace_regs.arg6() / page_size(), step);
       }
       break;
     }
     case Arch::mmap2:
       return process_mmap(t, trace_frame, trace_regs.arg2(), trace_regs.arg3(),
-                          trace_regs.arg4(), trace_regs.arg6(), step);
+                          trace_regs.arg4(), trace_regs.arg5(),
+                          trace_regs.arg6(), step);
 
     case Arch::shmat:
       return process_shmat(t, trace_frame, trace_regs.arg3(), step);
@@ -997,27 +1122,8 @@ static void rep_process_syscall_arch(ReplayTask* t, ReplayTraceStep* step) {
     case Arch::shmdt:
       return process_shmdt(t, trace_frame, trace_regs.arg1(), step);
 
-    case Arch::mremap: {
-      // We must emulate mremap because the kernel's choice for the remap
-      // destination can vary (in particular, when we emulate exec it makes
-      // different decisions).
-      AutoRemoteSyscalls remote(t);
-      if (trace_regs.syscall_result() == trace_regs.arg1()) {
-        // Non-moving mremap. Don't pass MREMAP_FIXED or MREMAP_MAYMOVE
-        // since that triggers EINVAL when the new map overlaps the old map.
-        remote.infallible_syscall_ptr(sys, trace_regs.arg1(), trace_regs.arg2(),
-                                      trace_regs.arg3(), 0);
-      } else {
-        // Force the mremap to use the destination address from recording.
-        // XXX could the new mapping overlap the old, with different start
-        // addresses? Hopefully the kernel doesn't do that to us!!!
-        remote.infallible_syscall_ptr(
-            sys, trace_regs.arg1(), trace_regs.arg2(), trace_regs.arg3(),
-            MREMAP_MAYMOVE | MREMAP_FIXED, trace_regs.syscall_result());
-      }
-      // ReplayTask::on_syscall_exit takes care of updating AddressSpace.
-      return;
-    }
+    case Arch::mremap:
+      return process_mremap(t, trace_frame, step);
 
     case Arch::madvise:
       switch ((int)t->regs().arg3()) {
