@@ -15,25 +15,29 @@
 #define RR_SECCOMP_BPF_H_
 
 #define _GNU_SOURCE 1
-#include <stdio.h>
-#include <stddef.h>
-#include <stdlib.h>
 #include <errno.h>
 #include <signal.h>
+#include <stddef.h>
+#include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
 #include <asm/unistd.h>
 
-#include <sys/user.h>
+#include <vector>
+
+#include "remote_code_ptr.h"
+
 #include <sys/prctl.h>
+#include <sys/user.h>
 #ifndef PR_SET_NO_NEW_PRIVS
 #define PR_SET_NO_NEW_PRIVS 38
 #endif
 
-#include <linux/unistd.h>
 #include <linux/audit.h>
 #include <linux/filter.h>
+#include <linux/unistd.h>
 #ifdef HAVE_LINUX_SECCOMP_H
 #include <linux/seccomp.h>
 #endif
@@ -57,15 +61,28 @@ struct seccomp_data {
 };
 #endif
 
+namespace rr {
+
 #define inst_ptr (offsetof(struct seccomp_data, instruction_pointer))
 
-#define ALLOW_SYSCALLS_FROM_CALLSITE(callsite)                                 \
-  BPF_STMT(BPF_LD + BPF_W + BPF_ABS, inst_ptr),                                \
-      BPF_JUMP(BPF_JMP + BPF_JEQ + BPF_K, callsite, 0, 1), ALLOW_PROCESS
-
-#define ALLOW_PROCESS BPF_STMT(BPF_RET + BPF_K, SECCOMP_RET_ALLOW)
-
-#define TRACE_PROCESS                                                          \
-  BPF_STMT(BPF_RET + BPF_K, SECCOMP_RET_TRACE | SECCOMP_RET_DATA)
+template <typename T> class SeccompFilter {
+public:
+  void allow() {
+    filters.push_back(BPF_STMT(BPF_RET + BPF_K, SECCOMP_RET_ALLOW));
+  }
+  void trace() {
+    filters.push_back(
+        BPF_STMT(BPF_RET + BPF_K, SECCOMP_RET_TRACE | SECCOMP_RET_DATA));
+  }
+  void allow_syscalls_from_callsite(remote_code_ptr ip) {
+    uint32_t v(ip.register_value());
+    assert(ip.register_value() == v);
+    filters.push_back(BPF_STMT(BPF_LD + BPF_W + BPF_ABS, inst_ptr));
+    filters.push_back(BPF_JUMP(BPF_JMP + BPF_JEQ + BPF_K, v, 0, 1));
+    allow();
+  }
+  std::vector<T> filters;
+};
+}
 
 #endif /* RR_SECCOMP_BPF_H_ */
