@@ -612,7 +612,7 @@ fsync = EmulatedSyscall(x86=118, x64=74)
 # When the Linux kernel creates the stack frame for a signal handler,
 # a call to sigreturn() is inserted into the stack frame so that upon
 # return from the signal handler, sigreturn() will be called.
-sigreturn = EmulatedSyscall(x86=119)
+sigreturn = IrregularEmulatedSyscall(x86=119)
 
 #  int clone(int (*fn)(void *), void *child_stack, int flags, void *arg, (pid_t
 #*ptid, struct user_desc *tls, pid_t *ctid));
@@ -847,7 +847,7 @@ nanosleep = IrregularEmulatedSyscall(x86=162, x64=35)
 # mremap() expands (or shrinks) an existing memory mapping,
 # potentially moving it at the same time (controlled by the flags
 # argument and the available virtual address space).
-mremap = EmulatedSyscall(x86=163, x64=25)
+mremap = IrregularEmulatedSyscall(x86=163, x64=25)
 
 #  int setresuid(uid_t ruid, uid_t euid, uid_t suid);
 #
@@ -894,7 +894,7 @@ getresgid = EmulatedSyscall(x86=171, x64=120, arg1="typename Arch::legacy_gid_t"
 #
 prctl = IrregularEmulatedSyscall(x86=172, x64=157)
 
-rt_sigreturn = EmulatedSyscall(x86=173, x64=15)
+rt_sigreturn = IrregularEmulatedSyscall(x86=173, x64=15)
 rt_sigaction = EmulatedSyscall(x86=174, x64=13, arg3="typename Arch::kernel_sigaction")
 rt_sigprocmask = IrregularEmulatedSyscall(x86=175, x64=14)
 
@@ -1333,12 +1333,14 @@ vserver = InvalidSyscall(x86=273, x64=236)
 mbind = EmulatedSyscall(x86=274, x64=237)
 get_mempolicy = UnsupportedSyscall(x86=275, x64=239)
 set_mempolicy = UnsupportedSyscall(x86=276, x64=238)
-mq_open = UnsupportedSyscall(x86=277, x64=240)
-mq_unlink = UnsupportedSyscall(x86=278, x64=241)
-mq_timedsend = UnsupportedSyscall(x86=279, x64=242)
-mq_timedreceive = UnsupportedSyscall(x86=280, x64=243)
-mq_notify = UnsupportedSyscall(x86=281, x64=244)
-mq_getsetattr = UnsupportedSyscall(x86=282, x64=245)
+
+mq_open = EmulatedSyscall(x86=277, x64=240)
+mq_unlink = EmulatedSyscall(x86=278, x64=241)
+mq_timedsend = EmulatedSyscall(x86=279, x64=242)
+mq_timedreceive = IrregularEmulatedSyscall(x86=280, x64=243)
+mq_notify = EmulatedSyscall(x86=281, x64=244)
+mq_getsetattr = EmulatedSyscall(x86=282, x64=245, arg3="struct Arch::mq_attr")
+
 kexec_load = UnsupportedSyscall(x86=283, x64=246)
 
 #  int waitid(idtype_t idtype, id_t id, siginfo_t *infop, int options);
@@ -1664,7 +1666,7 @@ listen = EmulatedSyscall(x86=363, x64=50)
 getsockname = IrregularEmulatedSyscall(x86=367, x64=51)
 getpeername = IrregularEmulatedSyscall(x86=368, x64=52)
 socketpair = EmulatedSyscall(x86=360, x64=53, arg4="int[2]")
-setsockopt = EmulatedSyscall(x86=366, x64=54)
+setsockopt = IrregularEmulatedSyscall(x86=366, x64=54)
 getsockopt = IrregularEmulatedSyscall(x86=365, x64=55)
 accept4 = IrregularEmulatedSyscall(x86=364, x64=288)
 
