@@ -8,12 +8,11 @@
 
 static int sockfds[2];
 
-static const int msg_magic = 0x1337beef;
 const ssize_t num_sockbuf_bytes = 1 << 20;
 
 static void child_proc(void) { exit(MAGIC_EXIT_CODE); }
 
-static void* writer_thread(void* dontcare) {
+static void* writer_thread(__attribute__((unused)) void* dontcare) {
   char token = '!';
   int sock = sockfds[1];
   int i;
@@ -31,7 +30,7 @@ static void* writer_thread(void* dontcare) {
   return NULL;
 }
 
-int main(int argc, char* argv[]) {
+int main(void) {
   char token = '!';
   char c = '\0';
   pthread_t t;
