@@ -1,10 +1,10 @@
 from rrutil import *
 
-send_gdb('b breakpoint\n')
+send_gdb('b breakpoint')
 expect_gdb('Breakpoint 1')
 
 for i in xrange(2):
-    send_gdb('c\n')
+    send_gdb('c')
     expect_gdb('Breakpoint 1, breakpoint')
 
 ok()
