@@ -4,7 +4,7 @@ import re
 from rrutil import *
 
 send_gdb('checkpoint')
-expect_gdb('= 1')
+expect_gdb('Checkpoint 1 at')
 
 send_gdb('n')
 send_gdb('restart 1')
