@@ -2,9 +2,9 @@
 
 #include "rrutil.h"
 
-void crash(void) { *(int*)NULL = 0; }
+void crash(void) { crash_null_deref(); }
 
-int main(int argc, char* argv[]) {
+int main(void) {
   atomic_puts("EXIT-SUCCESS");
   return 0;
 }
