@@ -2,7 +2,7 @@
 
 #include "rrutil.h"
 
-int main(int argc, char* argv[]) {
+int main(void) {
   /* There's not a (simple) way to meaningfully test fadvise,
    * since it only provides optimization hints, so this just
    * checks that rr doesn't blow up when it sees one. */
