@@ -4,7 +4,7 @@
 
 int main(void) {
   struct passwd* p = getpwnam("root");
-  atomic_printf("%d\n", p->pw_uid);
+  atomic_printf("%d\n", p ? p->pw_uid : 0);
   atomic_puts("EXIT-SUCCESS");
   return 0;
 }
