@@ -4,7 +4,7 @@
 
 static int pipe_fds[2];
 
-static void* start_thread(void* p) {
+static void* start_thread(__attribute__((unused)) void* p) {
   char ch;
 
   sleep(1);
@@ -14,7 +14,7 @@ static void* start_thread(void* p) {
   return NULL;
 }
 
-int main(int argc, char** argv) {
+int main(void) {
   pid_t child;
   pthread_t thread;
   char ch;
