@@ -2,15 +2,15 @@
 
 #include "rrutil.h"
 
-static void* start_thread(void* p) {
+static void* start_thread(__attribute__((unused)) void* p) {
   atomic_puts("EXIT-SUCCESS");
 
-  *(int*)NULL = 0;
+  crash_null_deref();
 
   return NULL;
 }
 
-int main(int argc, char* argv[]) {
+int main(void) {
   struct sigaction act;
   pthread_t thread;
 
