@@ -3,7 +3,7 @@
 #include "rrutil.h"
 
 static int pipefds[2];
-static int pselect_pipe(int timeout) {
+static int pselect_pipe(int timeout, sigset_t* sigmask) {
   fd_set set;
 
   FD_ZERO(&set);
@@ -11,31 +11,40 @@ static int pselect_pipe(int timeout) {
   struct timespec t;
   t.tv_sec = timeout;
   t.tv_nsec = 0;
-  sigset_t sigmask;
-  sigemptyset(&sigmask);
 
   errno = 0;
   return pselect(pipefds[0] + 1, &set, NULL, NULL, timeout ? &t : NULL,
-                 &sigmask);
+                 sigmask);
 }
 
 static int caught_signal;
 static void handle_signal(__attribute__((unused)) int sig) { ++caught_signal; }
 
 int main(void) {
+  sigset_t sigmask;
+  sigemptyset(&sigmask);
+
   pipe(pipefds);
 
   signal(SIGALRM, SIG_IGN);
   alarm(1);
   atomic_puts("ignoring SIGALRM, going into pselect ...");
-  test_assert(0 == pselect_pipe(2) && 0 == errno);
+  test_assert(0 == pselect_pipe(2, &sigmask) && 0 == errno);
+
+  /* Test that the signal mask is correct in rr when the SIGALRM is delivered */
+  sigaddset(&sigmask, SIGCHLD);
 
   signal(SIGALRM, handle_signal);
   alarm(1);
   atomic_puts("handling SIGALRM, going into pselect ...");
-  test_assert(-1 == pselect_pipe(0) && EINTR == errno);
+  test_assert(-1 == pselect_pipe(0, &sigmask) && EINTR == errno);
   test_assert(1 == caught_signal);
 
+  sigaddset(&sigmask, SIGALRM);
+  alarm(1);
+  atomic_puts("blocking SIGALRM, going into pselect ...");
+  test_assert(0 == pselect_pipe(2, &sigmask) && 0 == errno);
+
   atomic_puts("EXIT-SUCCESS");
   return 1;
 }
