@@ -14,14 +14,16 @@ int main(void) {
     return 0;
   }
   setpriority(PRIO_PROCESS, 0, 0);
-  /* Hopefully we've raised our priority so the child doesn't run at all. */
-  test_assert(0 == kill(child, SIGKILL));
+  /* Hopefully we've raised our priority so the child doesn't run at all.
+   * However, it is possible for the scheduler to pick the child to run. */
+  kill(child, SIGKILL);
   for (i = 0; i < 200; ++i) {
     test_assert(1 == write(STDOUT_FILENO, ".", 1));
   }
   test_assert(child == wait(&status));
-  test_assert(WIFSIGNALED(status));
-  test_assert(WTERMSIG(status) == SIGKILL);
+  /* See above ... we can't guarantee these assertions will succeed */
+  /* test_assert(WIFSIGNALED(status));
+     test_assert(WTERMSIG(status) == SIGKILL); */
   atomic_puts("EXIT-SUCCESS");
   return 0;
 }
