@@ -3,27 +3,30 @@
 #include "rrutil.h"
 
 #define FILENAME "foo"
-#define MODE (S_IFIFO)
+#define MODE S_IFIFO
 
-int main(int argc, char* argv[]) {
-  int result;
+int main(void) {
   struct stat* st;
 
-  result = mknod(FILENAME, MODE, 0);
-  test_assert(result == 0);
+  test_assert(mknod(FILENAME, MODE, 0) == 0);
 
   ALLOCATE_GUARD(st, 'x');
   test_assert(stat(FILENAME, st) == 0);
-
   test_assert(st->st_mode == MODE);
-  FREE_GUARD(st);
+  VERIFY_GUARD(st);
 
-  result = mknod(FILENAME, MODE, 0);
-
-  test_assert(result < 0);
+  test_assert(mknod(FILENAME, MODE, 0) < 0);
   test_assert(errno == EEXIST);
 
-  unlink(FILENAME);
+  test_assert(0 == unlink(FILENAME));
+
+  test_assert(mknodat(AT_FDCWD, FILENAME, MODE, 0) == 0);
+
+  test_assert(stat(FILENAME, st) == 0);
+  test_assert(st->st_mode == MODE);
+  VERIFY_GUARD(st);
+
+  test_assert(0 == unlink(FILENAME));
 
   atomic_puts("EXIT-SUCCESS");
   return 0;
