@@ -24,7 +24,7 @@ static void run_test(void) {
   size_t num_bytes = sysconf(_SC_PAGESIZE);
   int fd = create_segment(num_bytes);
   int* wpage = mmap(NULL, num_bytes, PROT_WRITE, MAP_SHARED, fd, 0);
-  int i;
+  size_t i;
   int* rpage;
 
   close(128);
@@ -52,11 +52,11 @@ static void run_test(void) {
 
   for (i = 0; i < num_bytes / sizeof(int); ++i) {
     wpage[i] = i;
-    test_assert(rpage[i] == i);
+    test_assert(rpage[i] == (ssize_t)i);
   }
 }
 
-int main(int argc, char* argv[]) {
+int main(void) {
   pid_t c;
   int status;
 
