@@ -4,7 +4,7 @@
 
 static void breakpoint(void) {}
 
-int main(int argc, char* argv[]) {
+int main(void) {
   pid_t child;
   int status;
   char* p;
