@@ -5,7 +5,7 @@
 #define TEST_FILENAME "foo.so"
 #define TOKEN "hello kitty"
 
-int main(int argc, char* argv[]) {
+int main(void) {
   int fd = open(TEST_FILENAME, O_CREAT | O_EXCL | O_RDWR, 0700);
   char* bytes;
 
